@@ -129,17 +129,14 @@ namespace netgen
 
 
   extern int CSGGenerateMesh (CSGeometry & geom, 
-			      Mesh *& mesh, MeshingParameters & mparam,
-			      int perfstepsstart, int perfstepsend);
+			      shared_ptr<Mesh> & mesh, MeshingParameters & mparam);
 
 
-  int CSGeometry :: GenerateMesh (Mesh*& mesh, MeshingParameters & mparam,
-				 int perfstepsstart, int perfstepsend)
+  int CSGeometry :: GenerateMesh (shared_ptr<Mesh> & mesh, MeshingParameters & mparam)
   {
-    return CSGGenerateMesh (*this, mesh, mparam, perfstepsstart, perfstepsend);
+    return CSGGenerateMesh (*this, mesh, mparam);
   }
-
-
+  
   const Refinement & CSGeometry :: GetRefinement () const
   {
     // cout << "get CSGeometry - Refinement" << endl;
@@ -182,7 +179,7 @@ namespace netgen
 
   void CSGeometry :: Save (string filename) const
   {
-    fstream ost (filename.c_str());
+    ofstream ost (filename.c_str());
     Save (ost);
   }
   
@@ -348,7 +345,25 @@ namespace netgen
 	const ExtrusionFace * ef = dynamic_cast< const ExtrusionFace * > (GetSurface(i));
 	const RevolutionFace * rf = dynamic_cast< const RevolutionFace * > (GetSurface(i));
 	const DummySurface * dummyf = dynamic_cast< const DummySurface * > (GetSurface(i));
+        const SplineSurface * splines = dynamic_cast<const SplineSurface *> (GetSurface(i));
 
+        if (splines)
+          {
+            splines->GetBase()->GetPrimitiveData(classname,coeffs);
+            out << classname << " " << coeffs.Size() << "\n";
+            for (int j=0; j<coeffs.Size(); j++)
+              out << coeffs[j] << " ";
+            out << "\n";
+            for (auto cut : *(splines->GetCuts()))
+              {
+                cut->GetPrimitiveData(classname,coeffs);
+                out << classname << " " << coeffs.Size() << "\n";
+                for (int j=0; j<coeffs.Size(); j++)
+                  out << coeffs[j] << " ";
+                out << "\n";
+              }
+            return;
+          }
 
 	if(sp)
 	  {
@@ -462,6 +477,15 @@ namespace netgen
 	    delete_them.Append(cone);
 	  }
 
+       else if(classname == "ellipticcone")
+         {
+	    EllipticCone * ellipticcone = new EllipticCone(dummypoint,dummyvec,dummyvec,dummydouble,dummydouble);
+
+	    ellipticcone->SetPrimitiveData(coeffs);
+
+	    AddSurface(ellipticcone);
+	    delete_them.Append(ellipticcone);
+	  }
 	else if(classname == "extrusionface")
 	  {
 	    ExtrusionFace * ef =
@@ -511,7 +535,7 @@ namespace netgen
  
   void CSGeometry :: AddSurface (char * name, Surface * surf)
   { 
-    (*testout) << "Adding surface " << name << endl;
+    (*testout) << "Adding surface " << name << ": " << (*surf) << endl;
     surfaces.Set (name, surf); 
     surf->SetName (name);
     changeval++; 
@@ -651,7 +675,8 @@ namespace netgen
 
   int CSGeometry :: SetTopLevelObject (Solid * sol, Surface * surf)
   {
-    return toplevelobjects.Append (new TopLevelObject (sol, surf)) - 1;
+    toplevelobjects.Append (new TopLevelObject (sol, surf));
+    return toplevelobjects.Size()-1;
   }
 
   TopLevelObject * CSGeometry :: 
@@ -821,6 +846,7 @@ namespace netgen
     int inv;
     int nsurf = GetNSurf();
 
+
     isidenticto.SetSize(nsurf);
     for (int i = 0; i < nsurf; i++)
       isidenticto[i] = i;
@@ -1494,7 +1520,7 @@ namespace netgen
     // virtual VisualScene * GetVisualScene (const NetgenGeometry * geom) const;
   };
 
-  extern CSGeometry * ParseCSG (istream & istr);
+  extern CSGeometry * ParseCSG (istream & istr, CSGeometry *instance=nullptr);
 
   NetgenGeometry *  CSGeometryRegister :: Load (string filename) const
   {
