@@ -4,7 +4,7 @@
 #include <csg.hpp>
 
 
-#include <incvis.hpp>
+#include <inctcl.hpp>
 #include <visual.hpp>
 
 
@@ -18,14 +18,14 @@ extern "C" int Ng_CSG_Init (Tcl_Interp * interp);
 namespace netgen
 {
   // extern DLL_HEADER NetgenGeometry * ng_geometry;
-  extern DLL_HEADER AutoPtr<NetgenGeometry> ng_geometry;
-  extern DLL_HEADER AutoPtr<Mesh> mesh;
+  extern DLL_HEADER shared_ptr<NetgenGeometry> ng_geometry;
+  extern DLL_HEADER shared_ptr<Mesh> mesh;
 
   static VisualSceneGeometry vsgeom;
  
   char * err_needscsgeometry = (char*) "This operation needs an CSG geometry";
-  extern char * err_needsmesh;
-  extern char * err_jobrunning;
+  char * err_needsmesh = (char*) "This operation needs a mesh";
+  char * err_jobrunning = (char*) "Meshing Job already running";
 
  
 
@@ -34,7 +34,7 @@ namespace netgen
 			Tcl_Interp * interp,
 			int argc, tcl_const char *argv[])
   {
-    CSGeometry * csgeom = dynamic_cast<CSGeometry*> (ng_geometry.Ptr());
+    CSGeometry * csgeom = dynamic_cast<CSGeometry*> (ng_geometry.get());
     if (csgeom)
       {
 	double detail = atof (Tcl_GetVar (interp, "::geooptions.detail", 0));
@@ -53,7 +53,7 @@ namespace netgen
 			  Tcl_Interp * interp,
 			  int argc, tcl_const char *argv[])
   {
-    CSGeometry * geometry = dynamic_cast<CSGeometry*> (ng_geometry.Ptr());
+    CSGeometry * geometry = dynamic_cast<CSGeometry*> (ng_geometry.get());
 
 
     const char * command = argv[1];
@@ -107,8 +107,8 @@ namespace netgen
   int Ng_CreatePrimitive (ClientData clientData,
 			  Tcl_Interp * interp,
 			  int argc, tcl_const char *argv[])
-  {
-    CSGeometry * geometry = dynamic_cast<CSGeometry*> (ng_geometry.Ptr());
+  {/*
+    CSGeometry * geometry = dynamic_cast<CSGeometry*> (ng_geometry.get());
     if (!geometry)
       {
 	Tcl_SetResult (interp, err_needscsgeometry, TCL_STATIC);
@@ -134,7 +134,7 @@ namespace netgen
       }
 
     geometry->SetSolid (name, nsol);
-
+	*/
     return TCL_OK;
   }
 
@@ -143,7 +143,7 @@ namespace netgen
 			   Tcl_Interp * interp,
 			   int argc, tcl_const char *argv[])
   {
-    CSGeometry * geometry = dynamic_cast<CSGeometry*> (ng_geometry.Ptr());
+    CSGeometry * geometry = dynamic_cast<CSGeometry*> (ng_geometry.get());
     if (!geometry)
       {
 	Tcl_SetResult (interp, err_needscsgeometry, TCL_STATIC);
@@ -180,8 +180,8 @@ namespace netgen
   int Ng_SetSolidData (ClientData clientData,
 		       Tcl_Interp * interp,
 		       int argc, tcl_const char *argv[])
-  {
-    CSGeometry * geometry = dynamic_cast<CSGeometry*> (ng_geometry.Ptr());
+  {/*
+    CSGeometry * geometry = dynamic_cast<CSGeometry*> (ng_geometry.get());
     if (!geometry)
       {
 	Tcl_SetResult (interp, err_needscsgeometry, TCL_STATIC);
@@ -199,7 +199,7 @@ namespace netgen
 
     Solid * nsol = Solid::CreateSolid (vst, geometry->GetSolids());
     geometry->SetSolid (name, nsol);
-
+	*/
     return TCL_OK;
   }
 
@@ -208,7 +208,7 @@ namespace netgen
 			   Tcl_Interp * interp,
 			   int argc, tcl_const char *argv[])
   {
-    CSGeometry * geometry = dynamic_cast<CSGeometry*> (ng_geometry.Ptr());
+    CSGeometry * geometry = dynamic_cast<CSGeometry*> (ng_geometry.get());
     if (!geometry)
       {
 	Tcl_SetResult (interp, err_needscsgeometry, TCL_STATIC);
@@ -245,8 +245,8 @@ namespace netgen
   int Ng_GetSolidData (ClientData clientData,
 		       Tcl_Interp * interp,
 		       int argc, tcl_const char *argv[])
-  {
-    CSGeometry * geometry = dynamic_cast<CSGeometry*> (ng_geometry.Ptr());
+  {/*
+    CSGeometry * geometry = dynamic_cast<CSGeometry*> (ng_geometry.get());
     if (!geometry)
       {
 	Tcl_SetResult (interp, err_needscsgeometry, TCL_STATIC);
@@ -264,7 +264,7 @@ namespace netgen
     cout << "GetSolidData, name = " << name << ", data = " << vst.str() << endl;
 
     Tcl_SetVar  (interp, valuevar, (char*)vst.str().c_str(), 0);
-
+	*/
     return TCL_OK;
   }
 
@@ -273,7 +273,7 @@ namespace netgen
 			   Tcl_Interp * interp,
 			   int argc, tcl_const char *argv[])
   {
-    CSGeometry * geometry = dynamic_cast<CSGeometry*> (ng_geometry.Ptr());
+    CSGeometry * geometry = dynamic_cast<CSGeometry*> (ng_geometry.get());
     if (!geometry)
       {
 	Tcl_SetResult (interp, err_needscsgeometry, TCL_STATIC);
@@ -306,7 +306,7 @@ namespace netgen
 			 Tcl_Interp * interp,
 			 int argc, tcl_const char *argv[])
   {
-    CSGeometry * geometry = dynamic_cast<CSGeometry*> (ng_geometry.Ptr());
+    CSGeometry * geometry = dynamic_cast<CSGeometry*> (ng_geometry.get());
     if (!geometry)
       {
 	Tcl_SetResult (interp, err_needscsgeometry, TCL_STATIC);
@@ -337,7 +337,7 @@ namespace netgen
 		       Tcl_Interp * interp,
 		       int argc, tcl_const char *argv[])
   {
-    CSGeometry * geometry = dynamic_cast<CSGeometry*> (ng_geometry.Ptr());
+    CSGeometry * geometry = dynamic_cast<CSGeometry*> (ng_geometry.get());
     if (!geometry)
       {
 	Tcl_SetResult (interp, err_needscsgeometry, TCL_STATIC);
@@ -368,7 +368,7 @@ namespace netgen
 		   Tcl_Interp * interp,
 		   int argc, tcl_const char *argv[])
   {
-    CSGeometry * geometry = dynamic_cast<CSGeometry*> (ng_geometry.Ptr());
+    CSGeometry * geometry = dynamic_cast<CSGeometry*> (ng_geometry.get());
     if (!geometry)
       {
 	Tcl_SetResult (interp, err_needscsgeometry, TCL_STATIC);
@@ -493,14 +493,14 @@ namespace netgen
 			 Tcl_Interp * interp,
 			 int argc, tcl_const char *argv[])
   {
-    CSGeometry * geometry = dynamic_cast<CSGeometry*> (ng_geometry.Ptr());
+    CSGeometry * geometry = dynamic_cast<CSGeometry*> (ng_geometry.get());
     if (!geometry)
       {
 	Tcl_SetResult (interp, err_needscsgeometry, TCL_STATIC);
 	return TCL_ERROR;
       }
 
-    if (!mesh.Ptr())
+    if (!mesh)
       {
 	Tcl_SetResult (interp, err_needsmesh, TCL_STATIC);
 	return TCL_ERROR;
@@ -511,9 +511,9 @@ namespace netgen
 	return TCL_ERROR;
       }
 
-    double globh = mparam.maxh;
+    // double globh = mparam.maxh;
     for (int i = 1; i <= geometry->singedges.Size(); i++)
-      geometry->singedges.Get(i)->SetMeshSize (*mesh, globh);
+      geometry->singedges.Get(i)->SetMeshSize (*mesh, 1e99 /* globh*/);
     return TCL_OK;
   }
 
@@ -522,16 +522,16 @@ namespace netgen
 			  Tcl_Interp * interp,
 			  int argc, tcl_const char *argv[])
   {
-    CSGeometry * geometry = dynamic_cast<CSGeometry*> (ng_geometry.Ptr());
+    CSGeometry * geometry = dynamic_cast<CSGeometry*> (ng_geometry.get());
     if (!geometry)
       {
 	Tcl_SetResult (interp, err_needscsgeometry, TCL_STATIC);
 	return TCL_ERROR;
       }
 
-    double globh = mparam.maxh;
+    // double globh = mparam.maxh;
     for (int i = 1; i <= geometry->singpoints.Size(); i++)
-      geometry->singpoints.Get(i)->SetMeshSize (*mesh, globh);
+      geometry->singpoints.Get(i)->SetMeshSize (*mesh, 1e99 /* globh */ );
     return TCL_OK;
   }
 
@@ -611,7 +611,7 @@ namespace netgen
 
   VisualScene * CSGeometryRegister :: GetVisualScene (const NetgenGeometry * geom) const
   {
-    CSGeometry * geometry = dynamic_cast<CSGeometry*> (ng_geometry.Ptr());
+    CSGeometry * geometry = dynamic_cast<CSGeometry*> (ng_geometry.get());
     if (geometry)
       {
 	vsgeom.SetGeometry (geometry);
