@@ -10,8 +10,9 @@ namespace netgen
 
   EdgeCalculation :: 
   EdgeCalculation (const CSGeometry & ageometry,
-		   Array<SpecialPoint> & aspecpoints)
-    : geometry(ageometry), specpoints(aspecpoints)
+		   Array<SpecialPoint> & aspecpoints,
+                   MeshingParameters & amparam)
+    : geometry(ageometry), specpoints(aspecpoints), mparam(amparam)
   {
     Box<3> bbox = geometry.BoundingBox();
 
@@ -40,8 +41,8 @@ namespace netgen
     PrintMessage (1, "Find edges");
     PushStatus ("Find edges");
 
-    for (int i = 1; i <= mesh.GetNP(); i++)
-      meshpoint_tree->Insert (mesh.Point(i), i);
+    for (PointIndex pi : mesh.Points().Range())    
+      meshpoint_tree->Insert (mesh[pi], pi);
 
 
     // add all special points before edge points (important for periodic identification)
@@ -62,6 +63,7 @@ namespace netgen
               meshpoint_tree -> Insert (p, pi); 
             }
         }
+           
 
     /*
       // slow version
@@ -453,7 +455,6 @@ namespace netgen
 		refedges[i].surfnr2 = geometry.GetSurfaceClassRepresentant(refedges[i].surfnr2);
 	      }
 
-
 	    /*
 	      for (int i = oldnseg+1; i <= mesh.GetNSeg(); i++)
 	      for (int j = 1; j <= oldnseg; j++)
@@ -484,7 +485,26 @@ namespace netgen
 		      layer,
 		      mesh);
 	  }
-	
+	for(int i=0; i<refedges.Size(); i++)
+	  {
+	    auto splinesurface = dynamic_cast<const SplineSurface*>(geometry.GetSurface(refedges[i].surfnr1));
+	    if(splinesurface)
+	      {
+		auto name = splinesurface->GetBCNameOf(specpoints[startpoints.Get(refedges[i].edgenr)].p,specpoints[endpoints.Get(refedges[i].edgenr)].p);
+		mesh.SetCD2Name(refedges[i].edgenr,name);
+	      }
+	    else
+	      {
+		auto splinesurface2 = dynamic_cast<const SplineSurface*>(geometry.GetSurface(refedges[i].surfnr2));
+	    if(splinesurface2)
+	      {
+		auto name = splinesurface2->GetBCNameOf(specpoints[startpoints.Get(refedges[i].edgenr)].p,specpoints[endpoints.Get(refedges[i].edgenr)].p);
+		mesh.SetCD2Name(refedges[i].edgenr,name);
+	      }
+		
+	      }
+	    
+	  }
 
 	/*
 	  // not available ...
@@ -597,7 +617,9 @@ namespace netgen
 			(*testout) << "Point on edge" << endl
 				   << "seg = " << i2 << ", p = " << pi << endl
 				   << "pos = " << p << ", projected = " << hp << endl
-				   << "seg is = " << mesh.Point(i2.I1()) << " - " << mesh.Point(i2.I2()) << endl;
+                                   << "seg is = "
+                                   << mesh.Point(PointIndex(i2.I1())) << " - "
+                                   << mesh.Point(PointIndex(i2.I2())) << endl;
 		      }
 		  }
 	      }
@@ -634,9 +656,10 @@ namespace netgen
       }
     
 
-    for (int i = 1; i <= nseg; i++)
+    // for (int i = 1; i <= nseg; i++)
+    for (Segment & seg : mesh.LineSegments())
       {
-	Segment & seg = mesh.LineSegment (i);
+	// Segment & seg = mesh.LineSegment (i);
 	if (seg.edgenr >= 1 && seg.edgenr <= cntedge)
 	  {
 	    if (osedges.Get(seg.edgenr) != -1)
@@ -698,8 +721,8 @@ namespace netgen
     // (*testout) << "geometry.GetSurface(s1) -> LocH (p, 3, 1, h) " << geometry.GetSurface(s1) -> LocH (p, 3, 1, h)
     // << " geometry.GetSurface(s2) -> LocH (p, 3, 1, h) " << geometry.GetSurface(s2) -> LocH (p, 3, 1, h) << endl;
 
-    loch = min2 (geometry.GetSurface(s1) -> LocH (p, 3, 1, h), 
-		 geometry.GetSurface(s2) -> LocH (p, 3, 1, h));
+    loch = min2 (geometry.GetSurface(s1) -> LocH (p, 3, 1, mparam, h), 
+		 geometry.GetSurface(s2) -> LocH (p, 3, 1, mparam, h));
   
   
   
@@ -848,18 +871,16 @@ namespace netgen
 	  }
 	*/
 
-	loch = min2 (geometry.GetSurface(s1_rep) -> LocH (np, 3, 1, h), 
-		     geometry.GetSurface(s2_rep) -> LocH (np, 3, 1, h));
+	loch = min2 (geometry.GetSurface(s1_rep) -> LocH (np, 3, 1, mparam, h), 
+		     geometry.GetSurface(s2_rep) -> LocH (np, 3, 1, mparam, h));
         loch = max2 (loch, mparam.minh);
 
 	if (uselocalh)
 	  {
 	    double lh = mesh.GetH(np);
-	    if (lh < loch)
-	      loch = lh;
+	    if (lh < loch) loch = lh;
 	  }
-      
-      
+        
 	len += Dist (p, np) / loch;
 	edgepoints.Append (np);
 	curvelength.Append (len);
@@ -1138,7 +1159,8 @@ namespace netgen
 			//seg.surfnr2 = s2_rep;
 			seg.surfnr1 = s1;
 			seg.surfnr2 = s2;
-			hi = refedges.Append (seg);
+                        refedges.Append (seg);
+                        hi = refedges.Size();
 			refedgesinv.Append (edgeinv);
 			edges_priority.Append((pre_ok[k-1]) ? 1 : 0);
 		      }
@@ -1158,7 +1180,23 @@ namespace netgen
 			  refedges.Elem(hi).domout = i;
 		      }
 		    else
+		      {
 		      refedges.Elem(hi).tlosurf = i;
+		      for(int kk = 0; kk < geometry.GetNTopLevelObjects(); kk++)
+			{
+			  auto othersolid = geometry.GetTopLevelObject(kk)->GetSolid();
+			  auto othersurf = geometry.GetTopLevelObject(kk)->GetSurface();
+			  if(!othersurf)
+			    {
+			      if(othersolid->IsIn(edgepoints[0])  &&
+				 othersolid->IsIn(edgepoints[edgepoints.Size()-1]))
+				{
+				  refedges.Elem(hi).domin = kk;
+				  refedges.Elem(hi).domout = kk;
+				}
+			    }
+			}
+		      }
 
 		    if(pre_ok[k-1])
 		      edges_priority[hi-1] = 1;
@@ -1736,7 +1774,7 @@ namespace netgen
 	    Vec<3> nv = s -> GetNormalVector (p1);
 		    
 	    double hloc = 
-	      min2 (s->LocH (p1, 3, 1, h), mesh.GetH(p1));
+	      min2 (s->LocH (p1, 3, 1, mparam, h), mesh.GetH(p1));
 
 	  
 		    
@@ -1777,15 +1815,15 @@ namespace netgen
 			  {
 			    seg1.domin = j;
 			    seg2.domin = j;
-			    seg1.tlosurf = j;
-			    seg2.tlosurf = j;
+			    seg1.tlosurf = -1;
+			    seg2.tlosurf = -1;
 			  }
 			else
 			  {
 			    seg1.domout = j;
 			    seg2.domout = j;
-			    seg1.tlosurf = j;
-			    seg2.tlosurf = j;
+			    seg1.tlosurf = -1;
+			    seg2.tlosurf = -1;
 			  }
 			//        seg.s2 = i;
 			//        seg.invs1 = surfaces[i] -> Inverse();
