@@ -25,11 +25,14 @@ namespace netgen
     const char * savetask = multithread.task;
     multithread.task = "Find points";
 
+    mesh.pointelements.SetSize(0);
     for (int i = 0; i < geom.GetNUserPoints(); i++)
       {
-	mesh.AddPoint(geom.GetUserPoint (i));
+        auto up = geom.GetUserPoint(i);
+	auto pnum = mesh.AddPoint(up);
 	mesh.Points().Last().Singularity (geom.GetUserPointRefFactor(i));
 	mesh.AddLockedPoint (PointIndex (i+1));
+        mesh.pointelements.Append (Element0d(pnum, up.GetIndex()));
       }
 
     SpecialPointCalculation spc;
@@ -60,9 +63,10 @@ namespace netgen
 
 
 
-  static void FindEdges (CSGeometry & geom, Mesh & mesh, const bool setmeshsize = false)
+  static void FindEdges (CSGeometry & geom, Mesh & mesh, MeshingParameters & mparam,
+                         const bool setmeshsize = false)
   {
-    EdgeCalculation ec (geom, specpoints);
+    EdgeCalculation ec (geom, specpoints, mparam);
     ec.SetIdEps(geom.GetIdEps());
     ec.Calc (mparam.maxh, mesh);
 
@@ -97,6 +101,12 @@ namespace netgen
 	//(*testout) << " to " << mesh.LineSegment(i).si << endl;
       }
 
+    for(int k = 1; k<=mesh.GetNFD(); k++)
+      {
+	*testout << "face: " << k << endl
+		 << "FD: " << mesh.GetFaceDescriptor(k) << endl;
+      }
+
     if (geom.identifications.Size())
       {
 	PrintMessage (3, "Find Identifications");
@@ -117,7 +127,7 @@ namespace netgen
     
     Point3d pmin, pmax;
     mesh.GetBox (pmin, pmax);
-    Box3dTree segtree (pmin, pmax);
+    BoxTree<3> segtree (pmin, pmax);
     
     for (SegmentIndex si = 0; si < mesh.GetNSeg(); si++)
       {
@@ -215,11 +225,11 @@ namespace netgen
   }
 
 
+  
+  
 
 
-
-
-  static void MeshSurface (CSGeometry & geom, Mesh & mesh)
+  static void MeshSurface (CSGeometry & geom, Mesh & mesh, MeshingParameters & mparam)
   {
     const char * savetask = multithread.task;
     multithread.task = "Surface meshing";
@@ -337,7 +347,6 @@ namespace netgen
 	FaceDescriptor & fd = mesh.GetFaceDescriptor(k);
 	fd.SetBCName ( mesh.GetBCNamePtr ( fd.BCProperty() - 1 ) );
       }
-    
 
     //!!
     
@@ -413,7 +422,7 @@ namespace netgen
 	  geom.GetSurface((mesh.GetFaceDescriptor(k).SurfNr()));
 
 
-	Meshing2Surfaces meshing(*surf, geom.BoundingBox());
+	Meshing2Surfaces meshing(*surf, mparam, geom.BoundingBox());
 	meshing.SetStartTime (starttime);
 
         double eps = 1e-8 * geom.MaxSize();
@@ -482,7 +491,7 @@ namespace netgen
 	mparam.checkoverlap = 0;
 
 	MESHING2_RESULT res =
-	  meshing.GenerateMesh (mesh, maxh, k);
+	  meshing.GenerateMesh (mesh, mparam, maxh, k);
 
 	if (res != MESHING2_OK)
 	  {
@@ -531,7 +540,7 @@ namespace netgen
 		  meshopt.SetMetricWeight (mparam.elsizeweight);
 		  meshopt.SetWriteStatus (0);
 
-		  meshopt.ImproveMesh (mesh);
+		  meshopt.ImproveMesh (mesh, mparam);
 		}
 		
 		{
@@ -553,7 +562,7 @@ namespace netgen
 		  meshopt.SetMetricWeight (mparam.elsizeweight);
 		  meshopt.SetWriteStatus (0);
 
-		  meshopt.ImproveMesh (mesh);
+		  meshopt.ImproveMesh (mesh, mparam);
 		}
 	      }
 	  }
@@ -561,8 +570,7 @@ namespace netgen
 
 	PrintMessage (3, (mesh.GetNSE() - oldnf), " elements, ", mesh.GetNP(), " points");
 
-	extern void Render();
-	Render();
+	mparam.Render();
       }
     
     mesh.Compress();
@@ -639,8 +647,7 @@ namespace netgen
 	    PrintMessage (3, (mesh.GetNSE() - oldnf), " elements, ", mesh.GetNP(), " points");
 	  }
       
-	extern void Render();
-	Render();
+        mparam.Render();
       }
     while (changed);
 
@@ -653,28 +660,22 @@ namespace netgen
 
 
 
-
-
-
-
-  int CSGGenerateMesh (CSGeometry & geom,
-		       Mesh *& mesh,
-		       int perfstepsstart, int perfstepsend,
-		       const char * optstr)
+  int CSGGenerateMesh (CSGeometry & geom, 
+		       shared_ptr<Mesh> & mesh, MeshingParameters & mparam)
   {
     if (mesh && mesh->GetNSE() &&
 	!geom.GetNSolids())
       {
-	if (perfstepsstart < MESHCONST_MESHVOLUME)
-	  perfstepsstart = MESHCONST_MESHVOLUME;
+	if (mparam.perfstepsstart < MESHCONST_MESHVOLUME)
+	  mparam.perfstepsstart = MESHCONST_MESHVOLUME;
       }
 
-    if (perfstepsstart <= MESHCONST_ANALYSE)
+    if (mparam.perfstepsstart <= MESHCONST_ANALYSE)
       {
         if (mesh)
           mesh -> DeleteMesh();
         else
-          mesh = new Mesh();
+          mesh = make_shared<Mesh>();
 
 	mesh->SetGlobalH (mparam.maxh);
 	mesh->SetMinimalH (mparam.minh);
@@ -693,6 +694,8 @@ namespace netgen
 			     mparam.grading);
 
 	    mesh -> LoadLocalMeshSize (mparam.meshsizefilename);
+            for (auto mspnt : mparam.meshsize_points)
+              mesh -> RestrictLocalH (mspnt.pnt, mspnt.h);
 	  }
 
 	spoints.SetSize(0);
@@ -708,13 +711,13 @@ namespace netgen
       }
 
 
-    if (multithread.terminate || perfstepsend <= MESHCONST_ANALYSE) 
+    if (multithread.terminate || mparam.perfstepsend <= MESHCONST_ANALYSE) 
       return TCL_OK;
 
 
-    if (perfstepsstart <= MESHCONST_MESHEDGES)
+    if (mparam.perfstepsstart <= MESHCONST_MESHEDGES)
       {
-	FindEdges (geom, *mesh, true);
+	FindEdges (geom, *mesh, mparam, true);
 	if (multithread.terminate) return TCL_OK;
 #ifdef LOG_STREAM      
 	(*logout) << "Edges meshed" << endl
@@ -728,30 +731,30 @@ namespace netgen
   
 	if (mparam.uselocalh)
 	  {
-	    mesh->CalcLocalH();
+	    mesh->CalcLocalH(mparam.grading);
 	    mesh->DeleteMesh();
 	    
 	    FindPoints (geom, *mesh);
 	    if (multithread.terminate) return TCL_OK;
-	    FindEdges (geom, *mesh, true);
+	    FindEdges (geom, *mesh, mparam, true);
 	    if (multithread.terminate) return TCL_OK;
 	    
 	    mesh->DeleteMesh();
 	  
 	    FindPoints (geom, *mesh);
 	    if (multithread.terminate) return TCL_OK;
-	    FindEdges (geom, *mesh);
+	    FindEdges (geom, *mesh, mparam);
 	    if (multithread.terminate) return TCL_OK;
 	  }
       }
   
-    if (multithread.terminate || perfstepsend <= MESHCONST_MESHEDGES)
+    if (multithread.terminate || mparam.perfstepsend <= MESHCONST_MESHEDGES)
       return TCL_OK;
 
 
-    if (perfstepsstart <= MESHCONST_MESHSURFACE)
+    if (mparam.perfstepsstart <= MESHCONST_MESHSURFACE)
       {
-	MeshSurface (geom, *mesh);  
+	MeshSurface (geom, *mesh, mparam);  
 	if (multithread.terminate) return TCL_OK;
       
 #ifdef LOG_STREAM
@@ -759,20 +762,22 @@ namespace netgen
 		  << "time = " << GetTime() << " sec" << endl
 		  << "points: " << mesh->GetNP() << endl;
 #endif      
-      
-	if (mparam.uselocalh && 0)
+
+        /*
+	if (mparam.uselocalh)
 	  {
-	    mesh->CalcLocalH();      
+	    mesh->CalcLocalH(mparam.grading);      
 	    mesh->DeleteMesh();
 
 	    FindPoints (geom, *mesh);
 	    if (multithread.terminate) return TCL_OK;
-	    FindEdges (geom, *mesh);
+	    FindEdges (geom, *mesh, mparam);
 	    if (multithread.terminate) return TCL_OK;
 
-	    MeshSurface (geom, *mesh);  
+	    MeshSurface (geom, *mesh, mparam);  
 	    if (multithread.terminate) return TCL_OK;
 	  }
+        */
 
 #ifdef LOG_STREAM      
 	(*logout) << "Surfaces remeshed" << endl
@@ -790,11 +795,11 @@ namespace netgen
 	mesh->CalcSurfacesOfNode();
       }
   
-    if (multithread.terminate || perfstepsend <= MESHCONST_OPTSURFACE)
+    if (multithread.terminate || mparam.perfstepsend <= MESHCONST_OPTSURFACE)
       return TCL_OK;
 
 
-    if (perfstepsstart <= MESHCONST_MESHVOLUME)
+    if (mparam.perfstepsstart <= MESHCONST_MESHVOLUME)
       {
 	multithread.task = "Volume meshing";
 
@@ -825,11 +830,11 @@ namespace netgen
 #endif
       }
 
-    if (multithread.terminate || perfstepsend <= MESHCONST_MESHVOLUME)
+    if (multithread.terminate || mparam.perfstepsend <= MESHCONST_MESHVOLUME)
       return TCL_OK;
 
 
-    if (perfstepsstart <= MESHCONST_OPTVOLUME)
+    if (mparam.perfstepsstart <= MESHCONST_OPTVOLUME)
       {
 	multithread.task = "Volume optimization";
       
@@ -849,6 +854,7 @@ namespace netgen
 #endif
       }
 
+    mesh -> OrderElements();
     return TCL_OK;
   }
 }
