@@ -15,8 +15,9 @@ Meshing2Surfaces :: Meshing2Surfaces (const Surface & asurface)
 }
   */
 Meshing2Surfaces :: Meshing2Surfaces (const Surface & asurf,
+				      const MeshingParameters & mp,
 				      const Box<3> & abb)
-  : Meshing2(abb), surface(asurf)
+  : Meshing2(mp, abb), surface(asurf), mparam (mp)
 {
   ;
 }
@@ -57,7 +58,7 @@ int Meshing2Surfaces :: TransformFromPlain (Point2d & planepoint,
 
 double Meshing2Surfaces :: CalcLocalH (const Point3d & p, double gh) const
 {
-  return surface.LocH (p, 3, 1, gh);
+  return surface.LocH (p, 3, 1, mparam, gh);
   /*
     double loch = mesh.lochfunc->GetH(p);
     if (gh < loch) loch = gh;
@@ -119,7 +120,7 @@ RefinementSurfaces :: RefinementSurfaces (const CSGeometry & ageometry)
 {
   if(geometry.GetNSurf() == 0)
     *testout << endl 
-             << "WARNING: Intializing 2D refinement with 0-surface geometry" << endl
+             << "WARNING: Initializing 2D refinement with 0-surface geometry" << endl
              << "==========================================================" << endl
              << endl << endl;
 }
@@ -134,7 +135,7 @@ PointBetween (const Point<3> & p1, const Point<3> & p2, double secpoint,
 	      int surfi, 
 	      const PointGeomInfo & gi1, 
 	      const PointGeomInfo & gi2,
-	      Point<3> & newp, PointGeomInfo & newgi)
+	      Point<3> & newp, PointGeomInfo & newgi) const
 {
   Point<3> hnewp;
   hnewp = p1+secpoint*(p2-p1);
@@ -152,7 +153,7 @@ PointBetween (const Point<3> & p1, const Point<3> & p2, double secpoint,
 	      int surfi1, int surfi2, 
 	      const EdgePointGeomInfo & ap1, 
 	      const EdgePointGeomInfo & ap2,
-	      Point<3> & newp, EdgePointGeomInfo & newgi)
+	      Point<3> & newp, EdgePointGeomInfo & newgi) const
 {
   Point<3> hnewp = p1+secpoint*(p2-p1);
   //(*testout) << "hnewp " << hnewp << " s1 " << surfi1 << " s2 " << surfi2 << endl;
@@ -192,7 +193,7 @@ Vec<3> RefinementSurfaces :: GetNormal (const Point<3> & p, int surfi1,
 
 
 
-void RefinementSurfaces :: ProjectToSurface (Point<3> & p, int surfi)
+void RefinementSurfaces :: ProjectToSurface (Point<3> & p, int surfi) const
 {
   if (surfi != -1)
     geometry.GetSurface (surfi) -> Project (p);
