@@ -31,7 +31,8 @@ namespace netgen
     prim = aprim;
     s1 = s2 = NULL;
     maxh = 1e10;
-    name = NULL;   
+    name = NULL;
+    num_surfs = prim->GetNSurfaces();
   }
 
   Solid :: Solid (optyp aop, Solid * as1, Solid * as2)
@@ -42,6 +43,9 @@ namespace netgen
     prim = NULL;
     name = NULL;
     maxh = 1e10;
+    num_surfs = 0;
+    if (s1) num_surfs += s1->num_surfs;
+    if (s2) num_surfs += s2->num_surfs;
   }
 
   Solid :: ~Solid ()
@@ -877,11 +881,7 @@ namespace netgen
 	  INSOLID_TYPE ist = prim->PointInSolid(p, eps);
 
 	  if (ist == DOES_INTERSECT)
-	    {
-	      //(*testout) << "Calling VecInSolid3..." << endl;
-	      ist = prim->VecInSolid3 (p, t, t2, eps);
-	      //(*testout) << "...done" << endl;
-	    }
+	    ist = prim->VecInSolid3 (p, t, t2, eps);
 	  in = (ist == IS_INSIDE || ist == DOES_INTERSECT);
 	  strin = (ist == IS_INSIDE);
 
@@ -1235,6 +1235,87 @@ namespace netgen
 
   Solid * Solid :: RecGetReducedSolid (const BoxSphere<3> & box, INSOLID_TYPE & in) const
   {
+    if (num_surfs <= 2)
+    {
+      // checking special case for degenerated plane - cylinder, Dec 2014
+      int cnt_plane = 0, cnt_cyl = 0;
+      bool inv_plane, inv_cyl;
+      Plane * plane;
+      Cylinder * cyl;
+
+      ForEachSurface ( [&] (Surface * surf, bool inv)
+                       {
+                         if (dynamic_cast<Plane*>(surf))
+                           {
+                             cnt_plane++;
+                             plane = dynamic_cast<Plane*>(surf);
+                             inv_plane = inv;
+                           }
+                         if (dynamic_cast<Cylinder*>(surf))
+                           {
+                             cnt_cyl++;
+                             cyl = dynamic_cast<Cylinder*>(surf);
+                             inv_cyl = inv;
+                           }
+                       });
+
+      if (cnt_plane == 1 && cnt_cyl == 1)
+        {
+          double scala = (cyl->A()-plane->P()) * plane->N();
+          double scalb = (cyl->B()-plane->P()) * plane->N();
+          double scal = plane->N() * plane->N();
+          if ( ( fabs (scala*scala - cyl->R()*cyl->R()*scal) < 1e-10*cyl->R()*cyl->R() ) &&
+               ( fabs (scalb*scalb - cyl->R()*cyl->R()*scal) < 1e-10*cyl->R()*cyl->R() ) )
+            {
+              // intersection edge in box ?
+              Point<3> p0 = cyl->A() - (scala/scal) * plane->N();
+              Vec<3> vedge = cyl->B() - cyl->A();
+              Vec<3> ve_center = box.Center()-p0;
+
+              // dist(lam) = \| ve_center \|^2 - 2 lam (vedge, ve_center) + lam^2 \| vedge \|^2
+
+              double num = vedge*ve_center;
+              double den = vedge*vedge;
+
+              double dist_edge_center2 = ve_center*ve_center  - num * num /den;
+              
+                
+              bool edge_in_box =  dist_edge_center2 < sqr (box.Diam());
+
+                
+              if (!edge_in_box)
+                {
+                  if (op == SECTION)
+                    {
+                      // cout << "solid = " << *this << endl;
+                      if (!inv_cyl && !inv_plane && scala < 0) 
+                        {
+                          // cout << "fix for degenerated cyl-plane edge: just the cylinder" << endl;
+                          Solid * sol = new Solid (cyl);
+                          sol -> op = TERM_REF;
+                          return sol;
+                        }
+                    }
+
+                  if (op == UNION)
+                    {
+                      // cout << "solid = " << *this << ", inv_plane = " << inv_plane << " inv_cyl = " << inv_cyl << " scalb " << scalb << endl;
+                      if (!inv_plane && !inv_cyl && (scala < 0))
+                        {
+                          // cout << "fix for degenerated cyl-plane edge: just the plane" << endl;
+                          // return new Solid (plane);
+                          Solid * sol = new Solid (plane);
+                          sol -> op = TERM_REF;
+                          return sol;
+                        }
+                    }
+                  ; // *testout << "have 1 plane and 1 cyl, degenerated" << endl;
+                }
+            }
+        }
+    }
+
+
     Solid * redsol = NULL;
 
     switch (op)
@@ -1420,7 +1501,36 @@ namespace netgen
 	}
       }
   }
-
+  void Solid :: ForEachSurface (const std::function<void(Surface*,bool)> & lambda, bool inv) const
+  {
+    switch (op)
+      {
+      case TERM: case TERM_REF:
+	{
+	  for (int j = 0; j < prim->GetNSurfaces(); j++)
+	    if (prim->SurfaceActive (j))
+              lambda (&prim->GetSurface(j), inv);
+	  break;
+	}
+      case UNION:
+      case SECTION:
+	{
+	  s1 -> ForEachSurface (lambda, inv);
+	  s2 -> ForEachSurface (lambda, inv);
+	  break;
+	}
+      case SUB:
+        {
+	  s1 -> ForEachSurface (lambda, !inv);
+	  break;
+        }
+      case ROOT:
+	{
+	  s1 -> ForEachSurface (lambda, inv);
+	  break;
+	}
+      }
+  }
 
 
   void Solid :: GetTangentialSurfaceIndices (const Point<3> & p, Array<int> & surfind, double eps) const
@@ -1539,7 +1649,6 @@ namespace netgen
 		  prim->GetSurface(j).CalcGradient (p, grad);
 		  if (sqr (grad * v) < 1e-6 * v.Length2() * grad.Length2())
 		    {
-		      //  (*testout) << "p2" << endl;
 		      Mat<3> hesse;
 		      prim->GetSurface(j).CalcHesse (p, hesse);
 		      double hv2 = v2 * grad + v * (hesse * v);
@@ -1549,6 +1658,19 @@ namespace netgen
 			  if (!surfind.Contains (prim->GetSurfaceId(j)))
 			    surfind.Append (prim->GetSurfaceId(j));
 			}
+		      /*
+		      else
+			{
+			  *testout << "QUAD NOT OK" << endl;
+			  *testout << "v = " << v << ", v2 = " << v2 << endl;
+			  *testout << "v * grad = " << v*grad << endl;
+			  *testout << "v2 * grad = " << v2*grad << endl;
+			  *testout << "v H v = " << v*(hesse*v) << endl;
+			  *testout << "grad = " << grad << endl;
+			  *testout << "hesse = " << hesse << endl;
+			  *testout << "hv2 = " << v2 * grad + v * (hesse * v) << endl;
+			}
+		      */
 		    }
 		}
 	    }
