@@ -16,7 +16,6 @@
 */
 
 
-// #undef DEVELOP
 // #define DEVELOP
 
 
@@ -27,7 +26,7 @@ namespace netgen
 
   void ProjectToEdge (const Surface * f1, const Surface * f2, Point<3> & hp);
 
-
+  enum { check_crosspoint = 5 };
 
   SpecialPoint :: SpecialPoint (const SpecialPoint & sp)
   {
@@ -200,15 +199,22 @@ namespace netgen
 	throw NgException (err.c_str());
       }
 
+    if (level == 40 || level == 41 || level == 45)
+      {
+        *testout << "level = " << level << " cp = " << calccp << " ep = " << calcep << ", box = " << box << ", solid = " << *sol << endl;
+      }
+
+
 
     bool decision;
     bool possiblecrossp, possibleexp;  // possible cross or extremalpoint
     bool surecrossp = 0, sureexp = 0;          // sure ...
   
-    static Array<int> locsurf;  // attention: array is static
+    // static Array<int> locsurf;  // attention: array is static
+    ArrayMem<int,100> locsurf; 
 
-    static int cntbox = 0;
-    cntbox++;
+    // static int cntbox = 0;
+    // cntbox++;
 
     if (level <= boxesinlevel.Size())
       boxesinlevel.Elem(level)++;
@@ -233,7 +239,7 @@ namespace netgen
 
 
     // explicit solution for planes only and at most one quadratic
-    if (numprim <= 5)
+    if (numprim <= check_crosspoint)
       {
 	int nplane = 0, nquad = 0, quadi = -1, nsphere = 0;
 	const QuadraticSurface *qsurf = 0, *qsurfi;
@@ -279,7 +285,7 @@ namespace netgen
 					  dynamic_cast<const Plane*> (geometry->GetSurface(locsurf[k2])),
 					  dynamic_cast<const Plane*> (geometry->GetSurface(locsurf[k3])),
 					  pts);
-		     
+
 		      for (int j = 0; j < pts.Size(); j++)
 			if (Dist (pts[j], box.Center()) < box.Diam()/2)
 			  {
@@ -464,7 +470,7 @@ namespace netgen
     possiblecrossp = (numprim >= 3) && calccp;
     surecrossp = 0;
 
-    if (possiblecrossp && (locsurf.Size() <= 5 || level > 50))
+    if (possiblecrossp && (locsurf.Size() <= check_crosspoint || level > 50))
       {
 	decision = 1;
 	surecrossp = 0;
@@ -484,6 +490,11 @@ namespace netgen
 		   geometry->GetSurface(locsurf.Get(k2)), 
 		   geometry->GetSurface(locsurf.Get(k3)), box );
 	      
+#ifdef DEVELOP
+		(*testout) << "k1,2,3 = " << k1 << "," << k2 << "," << k3 << ", nc = " << nc << ", deg = " << deg << endl;
+#endif
+
+
 		if (!nc && !deg) decision = 0;
 		if (nc) surecrossp = 1;
 	      }
@@ -565,7 +576,7 @@ namespace netgen
     possibleexp = (numprim >= 2) && calcep;
 
     // (*testout) << "l = " << level << "locsize = " << locsurf.Size() << " possexp = " << possibleexp << "\n";
-    if (possibleexp && (numprim <= 5 || level >= 50))
+    if (possibleexp && (numprim <= check_crosspoint || level >= 50))
       {
 	decision = 1;
 	sureexp = 0;
@@ -615,8 +626,10 @@ namespace netgen
 		      Point<3> pp;
 		      if (IsEdgeExtremalPoint (surf1, surf2, p, pp, box.Diam()/2))
 			{
-			  (*testout) << "extremalpoint (nearly) found:" << pp << endl;
-
+			  (*testout) << "extremalpoint (nearly) found:" << pp 
+                                     << "box.diam = " << box.Diam() << ", dist = " << Dist(pp,box.Center())
+                                     << endl;
+                          
 			  if (Dist (pp, box.Center()) < box.Diam()/2 &&
 			      sol -> IsIn (pp, 1e-6*size) && !sol->IsStrictIn (pp, 1e-6*size) )
 			    {
@@ -640,7 +653,7 @@ namespace netgen
 	  {
 	    box.GetSubBox (i, sbox);
 	    sbox.Increase (1e-4 * sbox.Diam());
-
+	    sbox.CalcDiamCenter();
 	    Solid * redsol = sol -> GetReducedSolid (sbox);
 
 	    if (redsol)
@@ -713,6 +726,7 @@ namespace netgen
         *testout << "check Newton: " << "beta = " << beta << ", gamma = " << gamma << ", eta = " << eta << endl;
         double rad = 1.0 / (beta * gamma);
         *testout << "rad = " << rad << endl;
+	*testout << "rs = " << rs << endl;
 #endif
         
 	return (beta * gamma * eta < 0.1) && (2 > box.Diam()*beta*gamma);
@@ -856,7 +870,7 @@ namespace netgen
 		   const BoxSphere<3> & box) const
   {
     // perform newton steps. normals parallel ?
-    // if not decideable: return 0 
+    // if not decidable: return 0 
   
     Point<3> p = box.Center();
     Vec<3> g1, g2, sol;
@@ -2013,7 +2027,8 @@ namespace netgen
 		      
 		      if (spi == -1)
 			{
-			  spi = specpoints.Append (SpecialPoint()) - 1;
+			  specpoints.Append (SpecialPoint());
+                          spi = specpoints.Size()-1;
 			  specpoint2point.Append (i);
 			  specpoints.Last().unconditional = 0;
 			  searchtree.Insert (apoints[i], spi);
