@@ -176,7 +176,9 @@ namespace netgen
 	\[ h  \times \kappa (x) \leq c \qquad \mbox{in} B(x, h), \]
 	where kappa(x) is the curvature in x. */
     virtual double LocH (const Point<3> & p, double x, 
-			 double c, double hmax) const;
+			 double c, 
+                         const MeshingParameters & mparam,
+                         double hmax) const;
 
     /**
        Gets Approximation by triangles,
@@ -209,10 +211,10 @@ namespace netgen
 
   class DummySurface : public Surface
   {
-    virtual double CalcFunctionValue (const Point<3> & point) const 
+    virtual double CalcFunctionValue (const Point<3> & /* point */) const
     { return 0; }
 
-    virtual void CalcGradient (const Point<3> & point, Vec<3> & grad) const 
+    virtual void CalcGradient (const Point<3> & /* point */, Vec<3> & grad) const
     { grad = Vec<3> (0,0,0); }
   
     virtual Point<3> GetSurfacePoint () const
@@ -221,7 +223,7 @@ namespace netgen
     virtual double HesseNorm () const
     { return 0; }
 
-    virtual void Project (Point<3> & p) const
+    virtual void Project (Point<3> & /* p */) const
     { ; }
 
     virtual void Print (ostream & ost) const
@@ -360,7 +362,7 @@ namespace netgen
 
   /**
      Projects point to edge.
-     The point hp is projected to the edge descibed by f1 and f2.
+     The point hp is projected to the edge described by f1 and f2.
      It is assumed that the edge is non-degenerated, and the
      (generalized) Newton method converges.
   */
