@@ -16,35 +16,29 @@ namespace netgen
   
   void MyMPI_SendCmd (const char * cmd)
   {
-    char buf[100];
-    strcpy (buf, cmd);
-    // MPI_Bcast (&buf, 100, MPI_CHAR, 0, MPI_COMM_WORLD);
+    int ntasks;
+    MPI_Comm_size(MPI_COMM_WORLD, &ntasks);
 
+    if(ntasks==1)
+      return;
+    
     for (int dest = 1; dest < ntasks; dest++)
-      MPI_Bsend( &buf, 100, MPI_CHAR, dest, MPI_TAG_CMD, MPI_COMM_WORLD);
+      MPI_Send( (void*)cmd, (strlen(cmd)+1), MPI_CHAR, dest, MPI_TAG_CMD, MPI_COMM_WORLD);
   }
 
   string MyMPI_RecvCmd ()
   {
-    char buf[100];
-    // MPI_Bcast (&buf, 100, MPI_CHAR, 0, MPI_COMM_WORLD);
-
-    // VT_OFF();
     MPI_Status status;
     int flag;
-    do
-      {
-	MPI_Iprobe (0, MPI_TAG_CMD, MPI_COMM_WORLD, &flag, &status);
-	if (!flag) 
-	  {
-	    VT_TRACER ("sleep");
-	    usleep (1000);
-	  }
-      }
-    while (!flag);
-    // VT_ON();
+    int size_of_msg = -1;
 
-    MPI_Recv( &buf, 100, MPI_CHAR, 0, MPI_TAG_CMD, MPI_COMM_WORLD, &status);
+    MPI_Probe(0, MPI_TAG_CMD, MPI_COMM_WORLD, &status);
+    MPI_Get_count(&status, MPI_CHAR, &size_of_msg);
+
+    //char* buf = (char*)malloc(size_of_msg*sizeof(char));
+    char buf[100000]; //1MB should be enough...
+
+    MPI_Recv( &buf, size_of_msg, MPI_CHAR, 0, MPI_TAG_CMD, MPI_COMM_WORLD, &status);
     
     return string(buf);
   }
