@@ -21,6 +21,7 @@
 #include "parthreads.hpp"
 // #include "moveablemem.hpp"
 #include "dynamicmem.hpp"
+#include "archive_base.hpp"
 
 #include "template.hpp"
 #include "array.hpp"
@@ -45,4 +46,7 @@
 #include "netgenout.hpp"
 #include "gzstream.h"
 
+#include "ngsimd.hpp"
+
+
 #endif
