@@ -11,22 +11,23 @@ namespace netgen
 {
 
 /// Base class for all ng exceptions
-class NgException 
+class NgException : public std::exception
 {
   /// verbal description of exception
-  string what;
+  string m_what;
 public:
   ///
-  NgException (const string & s);
+  DLL_HEADER NgException (const string & s);
   ///
-  virtual ~NgException ();
+  DLL_HEADER virtual ~NgException ();
 
   /// append string to description
-  void Append (const string & s);
+  DLL_HEADER void Append (const string & s);
   //  void Append (const char * s);
   
   /// verbal description of exception
-  const string & What() const { return what; }
+  const string & What() const { return m_what; }
+  virtual const char* what() const noexcept override { return m_what.c_str(); }
 };
 }
 
