@@ -3,7 +3,7 @@
 
 namespace netgen
 {
-  extern DLL_HEADER MeshingParameters mparam;
+  // extern DLL_HEADER MeshingParameters mparam;
 
   extern void Optimize2d (Mesh & mesh, MeshingParameters & mp);
 
@@ -30,7 +30,7 @@ namespace netgen
       }
 
     // limit slope
-    double gradh = 1/elto0;
+    double gradh = min(1/elto0,mp.grading);
     for (int i = 0; i < n-1; i++)
       {
 	double hnext = hi[i] + gradh * (xi[i+1]-xi[i]).Length();
@@ -238,6 +238,29 @@ namespace netgen
 	  }
       }
 
+    // first add all vertices (for compatible orientation on periodic bnds)
+    {
+      double diam2 = Dist2(pmin, pmax);
+      for (int i = 0; i < splines.Size(); i++)
+        for (int j : { 1, 2 })
+          {
+            Point<2> hnewp = (j == 1) ? splines[i]->StartPI() : splines[i]->EndPI();
+            Point<3> newp(hnewp(0), hnewp(1), 0);
+            int layer = GetSpline(i).layer;
+            int npi = -1;
+            for (PointIndex pi = PointIndex::BASE; 
+                 pi < mesh2d.GetNP()+PointIndex::BASE; pi++)
+              if (Dist2 (mesh2d.Point(pi), newp) < 1e-12 * diam2 && mesh2d.Point(pi).GetLayer() == layer)
+                npi = pi;
+            
+	    if (npi == -1)
+	      {
+		npi = mesh2d.AddPoint (newp, layer);
+		searchtree.Insert (newp, npi);
+	      }
+          }
+    }
+    
     for (int i = 0; i < splines.Size(); i++)
       if (GetSpline(i).copyfrom == -1)
 	{
@@ -350,33 +373,32 @@ namespace netgen
 
 
   void MeshFromSpline2D (SplineGeometry2d & geometry,
-			 Mesh *& mesh, 
+			 shared_ptr<Mesh> & mesh, 
 			 MeshingParameters & mp)
   {
     PrintMessage (1, "Generate Mesh from spline geometry");
 
-    double h = mp.maxh;
-
     Box<2> bbox = geometry.GetBoundingBox ();
 
-    if (bbox.Diam() < h) 
-      {
-	h = bbox.Diam();
-	mp.maxh = h;
-      }
+    if (bbox.Diam() < mp.maxh) 
+      mp.maxh = bbox.Diam();
 
-    mesh = new Mesh;
+    // double h = mp.maxh;
+
+    // mesh = make_shared<Mesh>();
     mesh->SetDimension (2);
 
     Point3d pmin(bbox.PMin()(0), bbox.PMin()(1), -bbox.Diam());
     Point3d pmax(bbox.PMax()(0), bbox.PMax()(1), bbox.Diam());
 
-    mesh->SetLocalH (pmin, pmax, mparam.grading);
-    mesh->SetGlobalH (h);
+    mesh->SetLocalH (pmin, pmax, mp.grading);
+    mesh->SetGlobalH (mp.maxh);
 
     
 
-    geometry.PartitionBoundary (mp, h, *mesh);
+    geometry.PartitionBoundary (mp, mp.maxh, *mesh);
+    
+    PrintMessage (3, "Boundary mesh done, np = ", mesh->GetNP());
 
 
     // marks mesh points for hp-refinement
@@ -394,7 +416,7 @@ namespace netgen
 		mpi = pi;
 		mindist = Dist2(gp3, (*mesh)[pi]);
 	      }
-	  (*mesh)[mpi].Singularity(1.);
+	  (*mesh)[mpi].Singularity(geometry.GetPoint(i).hpref);
 	}
 
 
@@ -413,24 +435,22 @@ namespace netgen
     // number of bcnames
     int maxsegmentindex = 0;
     for (SegmentIndex si = 0; si < mesh->GetNSeg(); si++)
-      {
-	if ( (*mesh)[si].si > maxsegmentindex) maxsegmentindex = (*mesh)[si].si;
-      }
+      if ( (*mesh)[si].si > maxsegmentindex) maxsegmentindex = (*mesh)[si].si;
 
-    mesh->SetNBCNames(maxsegmentindex);
+    mesh->SetNBCNames(maxsegmentindex+1);
 
-    for ( int sindex = 0; sindex < maxsegmentindex; sindex++ )
+    for ( int sindex = 0; sindex <= maxsegmentindex; sindex++ )
       mesh->SetBCName ( sindex, geometry.GetBCName( sindex+1 ) );
 
     for (SegmentIndex si = 0; si < mesh->GetNSeg(); si++)
       (*mesh)[si].SetBCName ( (*mesh).GetBCNamePtr( (*mesh)[si].si-1 ) );
-
   
-    mesh->CalcLocalH(mparam.grading);
+    mesh->CalcLocalH(mp.grading);
 
     int bnp = mesh->GetNP(); // boundary points
+    auto BndPntRange = mesh->Points().Range();
 
-    int hquad = mparam.quad;
+    int hquad = mp.quad;
 
 
     for (int domnr = 1; domnr <= maxdomnr; domnr++)
@@ -517,7 +537,8 @@ namespace netgen
     for (int domnr = 1; domnr <= maxdomnr; domnr++)
       {
         if (geometry.GetDomainTensorMeshing (domnr)) continue;
-
+        
+        double h = mp.maxh;
 	if ( geometry.GetDomainMaxh ( domnr ) > 0 )
 	  h = geometry.GetDomainMaxh(domnr);
 
@@ -526,17 +547,17 @@ namespace netgen
 
 	int oldnf = mesh->GetNSE();
 
-        mparam.quad = hquad || geometry.GetDomainQuadMeshing (domnr);
+        mp.quad = hquad || geometry.GetDomainQuadMeshing (domnr);
 
-	Meshing2 meshing (mparam, Box<3> (pmin, pmax));
+	Meshing2 meshing (mp, Box<3> (pmin, pmax));
 
 	Array<int, PointIndex::BASE> compress(bnp);
 	compress = -1;
 	int cnt = 0;
-	for (PointIndex pi = PointIndex::BASE; pi < bnp+PointIndex::BASE; pi++)
+        for (PointIndex pi : BndPntRange)
 	  if ( (*mesh)[pi].GetLayer() == geometry.GetDomainLayer(domnr))
 	    {
-	      meshing.AddPoint ( (*mesh)[pi], pi);
+	      meshing.AddPoint ((*mesh)[pi], pi);
 	      cnt++;
 	      compress[pi] = cnt;
 	    }
@@ -547,38 +568,32 @@ namespace netgen
 	  {
 	    if ( (*mesh)[si].domin == domnr)
 	      {
-		meshing.AddBoundaryElement ( compress[(*mesh)[si][0]], 
-					     compress[(*mesh)[si][1]], gi, gi);
+		meshing.AddBoundaryElement (compress[(*mesh)[si][0]], 
+                                            compress[(*mesh)[si][1]], gi, gi);
 	      }
 	    if ( (*mesh)[si].domout == domnr)
 	      {
-		meshing.AddBoundaryElement ( compress[(*mesh)[si][1]],
-					     compress[(*mesh)[si][0]], gi, gi);
-		
+		meshing.AddBoundaryElement (compress[(*mesh)[si][1]],
+                                            compress[(*mesh)[si][0]], gi, gi);
 	      }
-
 	  }
 
-
-	mparam.checkoverlap = 0;
-
-	meshing.GenerateMesh (*mesh, mparam, h, domnr);
+        // not complete, use at own risk ...
+        // meshing.Delaunay(*mesh, domnr, mp);
+        mp.checkoverlap = 0;
+        meshing.GenerateMesh (*mesh, mp, h, domnr);
 
 	for (SurfaceElementIndex sei = oldnf; sei < mesh->GetNSE(); sei++)
 	  (*mesh)[sei].SetIndex (domnr);
 
-
 	// astrid
 	char * material;
-	geometry.GetMaterial( domnr, material );
-	if ( material )
-	  {
-	    (*mesh).SetMaterial ( domnr,  material );
-	  }
-
+	geometry.GetMaterial (domnr, material);
+	if (material)
+          mesh->SetMaterial (domnr, material);
       }
 
-    mparam.quad = hquad;
+    mp.quad = hquad;
 
 
 
@@ -595,11 +610,10 @@ namespace netgen
     mp.optsteps2d = hsteps;
 
     mesh->Compress();
+    mesh->OrderElements();
     mesh -> SetNextMajorTimeStamp();
 
-
-    extern DLL_HEADER void Render();
-    Render();
+    mp.Render();
   }
 
 
