@@ -161,7 +161,7 @@ namespace netgen
 	// cout << "add spline " << i << ", left = " << leftdom << ", right = " << rightdom << endl;
       
 	infile >> buf;
-	// type of spline segement
+	// type of spline segment
 	if (strcmp (buf, "2") == 0)
 	  { // a line
 	    infile >> hi1 >> hi2;
@@ -332,7 +332,7 @@ namespace netgen
 	// cout << "add spline " << i << ", left = " << leftdom << endl;
 
 	infile >> buf;
-	// type of spline segement
+	// type of spline segment
 	if (strcmp (buf, "2") == 0)
 	  { // a line
 	    infile >> hi1 >> hi2;
@@ -629,7 +629,7 @@ namespace netgen
 
 	      
 		infile >> buf;
-		// type of spline segement
+		// type of spline segment
 		if (strcmp (buf, "2") == 0)
 		  { // a line
 		    infile >> hi1 >> hi2;
@@ -887,28 +887,51 @@ namespace netgen
 
 
 
-
-  string SplineGeometry2d :: GetBCName( const int  bcnr ) const
+  void SplineGeometry2d :: SetBCName (int bcnr, string name) 
   {
-    if ( bcnames.Size() >= bcnr)
+    if (bcnr < 1)
+      throw NgException ("Illegal nr in SetBCName");
+    int new_to_add = bcnr - bcnames.Size();
+    for (int i = 0; i < new_to_add; i++)
+      bcnames.Append (new string("default"));
+    delete bcnames[bcnr-1];
+    bcnames[bcnr-1] = new string(name);
+  }
+  
+  string SplineGeometry2d :: GetBCName( int  bcnr ) const
+  {
+    if (bcnames.Size() >= bcnr)
       if (bcnames[bcnr-1] )
 	return *bcnames[bcnr-1];
     return "default";
   }
 
-  string * SplineGeometry2d :: BCNamePtr( const int bcnr ) 
+  string * SplineGeometry2d :: BCNamePtr( int bcnr ) 
   {
     if ( bcnr > bcnames.Size() )
-      return 0;
+      return nullptr;
     else
       return bcnames[bcnr-1];
   }
 
 
+  int SplineGeometry2d :: GetBCNumber (string name) const
+  {
+    for (int i = 0; i < bcnames.Size(); i++)
+      if (*bcnames[i] == name)
+        return i+1;
+    return 0;
+  }
+
+  int SplineGeometry2d :: AddBCName (string name)
+  {
+    bcnames.Append (new string(name));
+    return bcnames.Size();
+  }
 
 
 
-  void SplineGeometry2d :: GetMaterial( const int  domnr, char* & material )
+  void SplineGeometry2d :: GetMaterial (int  domnr, char* & material )
   {
     if ( materials.Size() >= domnr)
       material =  materials[domnr-1];
@@ -916,8 +939,26 @@ namespace netgen
       material = 0;
   }
 
+  void SplineGeometry2d :: SetMaterial (int  domnr, const string & material)
+  {
+    int oldsize = materials.Size();
+    if (domnr > materials.Size()) materials.SetSize (domnr);
+    for (int i = oldsize; i < domnr; i++)
+      materials[i] = nullptr;
+    
+    if (domnr >= 1) //  && domnr <= materials.Size())
+      {
+        delete materials[domnr-1];
+        materials[domnr-1] = new char[material.size()+1];
+        strcpy(materials[domnr-1], material.c_str());
+      }
+    else
+      throw NgException ("material index out of range");
+  }
 
-  double SplineGeometry2d :: GetDomainMaxh( const int  domnr )
+  
+
+  double SplineGeometry2d :: GetDomainMaxh (const int domnr )
   {
     if ( maxh.Size() >= domnr  && domnr > 0)
       return maxh[domnr-1];
@@ -925,15 +966,27 @@ namespace netgen
       return -1;
   }
 
+  void SplineGeometry2d :: SetDomainMaxh (int domnr, double h)
+  {
+    int oldsize = maxh.Size();
+    if (domnr > maxh.Size()) maxh.SetSize (domnr);
+    for (int i = oldsize; i < domnr; i++)
+      maxh[i] = 1e99;
+    
+    if (domnr >= 1) 
+      maxh[domnr-1] = h;
+    else
+      throw NgException ("material index out of range");
+  }
 
+  
 
   extern void MeshFromSpline2D (SplineGeometry2d & geometry,
-				Mesh *& mesh, 
+				shared_ptr<Mesh> & mesh, 
 				MeshingParameters & mp);
 
 
-  int SplineGeometry2d :: GenerateMesh (Mesh*& mesh, MeshingParameters & mparam,
-					int perfstepsstart, int perfstepsend)
+  int SplineGeometry2d :: GenerateMesh (shared_ptr<Mesh> & mesh, MeshingParameters & mparam)
   {
     MeshFromSpline2D (*this, mesh, mparam);
     return 0;
