@@ -451,7 +451,7 @@ namespace netgen
   void ADTree3 :: PrintRec (ostream & ost, const ADTreeNode3 * node) const
   {
   
-    if (node->data)
+    // if (node->data)  // true anyway
       {
 	ost << node->pi << ": ";
 	ost << node->nchilds << " childs, ";
@@ -1525,7 +1525,7 @@ namespace netgen
 
 
 #endif
-
+  
 
 
 
@@ -1751,8 +1751,8 @@ namespace netgen
 
   void ADTree6 :: PrintRec (ostream & ost, const ADTreeNode6 * node) const
   {
-  
-    if (node->data)
+    
+    // if (node->data)     // true anyway
       {
 	ost << node->pi << ": ";
 	ost << node->nchilds << " childs, ";
@@ -1794,6 +1794,241 @@ namespace netgen
 
 
 
+  
+  template <int dim, typename T>
+  T_ADTree<dim,T> :: T_ADTree (Point<dim> acmin, Point<dim> acmax)
+  // : ela(0)
+  {
+    cmin = acmin;
+    cmax = acmax;
+    
+    root = new T_ADTreeNode<dim,T>;
+    root->sep = (cmin[0] + cmax[0]) / 2;
+  }
+
+  template <int dim, typename T>  
+  T_ADTree<dim,T> :: ~T_ADTree ()
+  {
+    root->DeleteChilds();
+    delete root;
+  }
+
+  template <int dim, typename T>
+  void T_ADTree<dim,T> :: Insert (Point<dim> p, T pi)
+  {
+    T_ADTreeNode<dim,T> *node(NULL);
+    T_ADTreeNode<dim,T> *next;
+    int dir;
+    int lr(0);
+
+    Point<dim> bmin = cmin;
+    Point<dim> bmax = cmax;
+  
+    next = root;
+    dir = 0;
+    while (next)
+      {
+	node = next;
+
+	if (IsInvalid(node->pi))
+	  {    
+	    // memcpy (node->data, p, dim * sizeof(float));
+            node->data = p;
+	    node->pi = pi;
+
+	    // if (ela.Size() < pi+1)
+            // ela.SetSize (pi+1);
+	    ela[pi] = node;
+
+	    return;
+	  }
+
+	if (node->sep > p[dir])
+	  {
+	    next = node->left;
+	    bmax(dir) = node->sep;
+	    lr = 0;
+	  }
+	else
+	  {
+	    next = node->right;
+	    bmin(dir) = node->sep;
+	    lr = 1;
+	  }
+
+	dir++;
+	if (dir == dim) dir = 0;
+      }
+
+
+    next = new T_ADTreeNode<dim,T>;
+    // memcpy (next->data, p, dim * sizeof(float));
+    next->data = p;
+    next->pi = pi;
+    next->sep = (bmin[dir] + bmax[dir]) / 2;
+
+    // if (ela.Size() < pi+1)
+    // ela.SetSize (pi+1);
+    ela[pi] = next;
+
+    if (lr)
+      node->right = next;
+    else
+      node->left = next;
+    next -> father = node;
+
+    while (node)
+      {
+	node->nchilds++;
+	node = node->father;
+      }
+  }
+
+  template <int dim, typename T>
+  void T_ADTree<dim,T> :: DeleteElement (T pi)
+  {
+    T_ADTreeNode<dim,T> * node = ela[pi];
+    ela.Delete(pi);
+    
+    SetInvalid(node->pi); //  = -1;
+
+    node = node->father;
+    while (node)
+      {
+	node->nchilds--;
+	node = node->father;
+      }
+  }
+
+  template <int dim, typename T>
+  void T_ADTree<dim,T> :: PrintMemInfo (ostream & ost) const
+  {
+    ost << Elements() << " elements a " << sizeof(ADTreeNode6) 
+	<< " Bytes = "
+	<< Elements() * sizeof(T_ADTreeNode<dim,T>) << endl;
+    ost << "maxind = " << ela.Size() << " = " << sizeof(T_ADTreeNode<dim,T>*) * ela.Size() << " Bytes" << endl;
+  }
+
+
+  template <int dim, typename T>
+  class inttn {
+  public:
+    int dir;
+    T_ADTreeNode<dim,T> * node;
+  };
+
+
+  template <int dim, typename T>
+  void T_ADTree<dim,T> :: GetIntersecting (Point<dim> bmin, Point<dim> bmax,
+                                           Array<T> & pis) const
+  {
+    // static Array<inttn6> stack(10000);
+    // stack.SetSize (10000);
+    ArrayMem<inttn<dim,T>,10000> stack(10000);
+    pis.SetSize(0);
+
+    stack[0].node = root;
+    stack[0].dir = 0;
+    int stacks = 0;
+
+    while (stacks >= 0)
+      {
+	T_ADTreeNode<dim,T> * node = stack[stacks].node;
+	int dir = stack[stacks].dir; 
+
+	stacks--;
+	if (!IsInvalid(node->pi)) //  != -1)
+	  {
+            bool found = true;
+            for (int i = 0; i < dim/2; i++)
+              if (node->data[i] > bmax[i])
+                found = false;
+            for (int i = dim/2; i < dim; i++)
+              if (node->data[i] < bmin[i])
+                found = false;
+            if (found)
+              pis.Append (node->pi);            
+            /*
+	    if (node->data[0] > bmax[0] || 
+		node->data[1] > bmax[1] || 
+		node->data[2] > bmax[2] || 
+		node->data[3] < bmin[3] || 
+		node->data[4] < bmin[4] || 
+		node->data[5] < bmin[5])
+	      ;
+	    else
+              {
+                pis.Append (node->pi);
+              }
+            */
+	  }
+
+	int ndir = (dir+1) % dim;
+
+	if (node->left && bmin[dir] <= node->sep)
+	  {
+	    stacks++;
+	    stack[stacks].node = node->left;
+	    stack[stacks].dir = ndir;
+	  }
+	if (node->right && bmax[dir] >= node->sep)
+	  {
+	    stacks++;
+	    stack[stacks].node = node->right;
+	    stack[stacks].dir = ndir;
+	  }
+      }
+  }
+
+  template <int dim, typename T>
+  void T_ADTree<dim,T> :: PrintRec (ostream & ost, const T_ADTreeNode<dim,T> * node) const
+  {
+    
+    // if (node->data)     // true anyway
+      {
+	ost << node->pi << ": ";
+	ost << node->nchilds << " childs, ";
+	for (int i = 0; i < dim; i++)
+	  ost << node->data[i] << " ";
+	ost << endl;
+      }
+    if (node->left)
+      PrintRec (ost, node->left);
+    if (node->right)
+      PrintRec (ost, node->right);
+  }
+
+  template <int dim, typename T>
+  int T_ADTree<dim,T> :: DepthRec (const T_ADTreeNode<dim,T> * node) const
+  {
+    int ldepth = 0;
+    int rdepth = 0;
+
+    if (node->left)
+      ldepth = DepthRec(node->left);
+    if (node->right)
+      rdepth = DepthRec(node->right);
+    return 1 + max2 (ldepth, rdepth);
+  }
+
+  template <int dim, typename T>
+  int T_ADTree<dim,T> :: ElementsRec (const T_ADTreeNode<dim,T> * node) const
+  {
+    int els = 1;
+    if (node->left)
+      els += ElementsRec(node->left);
+    if (node->right)
+      els += ElementsRec(node->right);
+    return els;
+  }
+
+
+
+
+  
+
+
+
 #ifdef ABC
 
   /* ******************************* ADTree6F ******************************* */
@@ -2112,67 +2347,83 @@ namespace netgen
 
 
 
-
-  Box3dTree :: Box3dTree (const Box<3> & abox)
+  template <int dim, typename T>
+  BoxTree<dim,T> :: BoxTree (const Box<dim> & abox)
   {
     boxpmin = abox.PMin();
     boxpmax = abox.PMax();
-    float tpmin[6], tpmax[6];
-    for (int i = 0; i < 3; i++)
+    Point<2*dim> tpmin, tpmax;
+    for (int i = 0; i < dim; i++)
       {
-	tpmin[i] = tpmin[i+3] = boxpmin(i);
-	tpmax[i] = tpmax[i+3] = boxpmax(i);
+	tpmin(i) = tpmin(i+dim) = boxpmin(i);
+	tpmax(i) = tpmax(i+dim) = boxpmax(i);
       }
-    tree = new ADTree6 (tpmin, tpmax);
+    tree = new T_ADTree<2*dim,T> (tpmin, tpmax);
   }
 
-  Box3dTree :: Box3dTree (const Point<3> & apmin, const Point<3> & apmax)
+  template <int dim, typename T>
+  BoxTree<dim,T> :: BoxTree (const Point<dim> & apmin, const Point<dim> & apmax)
   {
     boxpmin = apmin;
     boxpmax = apmax;
-    float tpmin[6], tpmax[6];
-    for (int i = 0; i < 3; i++)
+    Point<2*dim> tpmin, tpmax;
+    for (int i = 0; i < dim; i++)
       {
-	tpmin[i] = tpmin[i+3] = boxpmin(i);
-	tpmax[i] = tpmax[i+3] = boxpmax(i);
+	tpmin(i) = tpmin(i+dim) = boxpmin(i);
+	tpmax(i) = tpmax(i+dim) = boxpmax(i);
       }
-    tree = new ADTree6 (tpmin, tpmax);
+    tree = new T_ADTree<2*dim,T> (tpmin, tpmax);
   }
 
-  Box3dTree :: ~Box3dTree ()
+  template <int dim, typename T>
+  BoxTree<dim,T> :: ~BoxTree ()
   {
     delete tree;
   }
 
-  void Box3dTree :: Insert (const Point<3> & bmin, const Point<3> & bmax, int pi)
+  template <int dim, typename T>
+  void BoxTree<dim,T> :: Insert (const Point<dim> & bmin, const Point<dim> & bmax, T pi)
   {
-    float tp[6];
+    Point<2*dim> tp;
 
-    for (int i = 0; i < 3; i++)
+    for (size_t i = 0; i < dim; i++)
       {
-	tp[i] = bmin(i);
-	tp[i+3] = bmax(i);
+	tp(i) = bmin(i);
+	tp(i+dim) = bmax(i);
       }
 
     tree->Insert (tp, pi);
   }
 
-  void Box3dTree ::GetIntersecting (const Point<3> & pmin, const Point<3> & pmax, 
-				    Array<int> & pis) const
+  template <int dim, typename T>
+  void BoxTree<dim,T> ::GetIntersecting (const Point<dim> & pmin, const Point<dim> & pmax, 
+                                         Array<T> & pis) const
   {
-    float tpmin[6];
-    float tpmax[6];
+    Point<2*dim> tpmin, tpmax;
 
-    for (int i = 0; i < 3; i++)
+    for (size_t i = 0; i < dim; i++)
       {
-	tpmin[i] = boxpmin(i);
-	tpmax[i] = pmax(i);
-      
-	tpmin[i+3] = pmin(i);
-	tpmax[i+3] = boxpmax(i);
+	tpmin(i) = boxpmin(i);
+	tpmax(i) = pmax(i);
+        
+	tpmin(i+dim) = pmin(i);
+	tpmax(i+dim) = boxpmax(i);
       }
 
     tree->GetIntersecting (tpmin, tpmax, pis);
   }
 
+
+  template<> BlockAllocator T_ADTreeNode<4,INDEX> :: ball(sizeof (T_ADTreeNode<4,INDEX>));
+  template class T_ADTree<4,INDEX>;
+  template class BoxTree<2,INDEX>;
+
+  template<> BlockAllocator T_ADTreeNode<4,INDEX_2> :: ball(sizeof (T_ADTreeNode<4,INDEX_2>));
+  template class T_ADTree<4,INDEX_2>;
+  template class BoxTree<2,INDEX_2>;
+  
+  
+  template<> BlockAllocator T_ADTreeNode<6,INDEX> :: ball(sizeof (T_ADTreeNode<6,INDEX>));
+  template class T_ADTree<6,INDEX>;
+  template class BoxTree<3,INDEX>;
 }
