@@ -9,60 +9,17 @@
 #endif
 
 #include "nginterface.h"
-#include "../visualization/soldata.hpp"
+// #include "../visualization/soldata.hpp"
+// #include <visual.hpp>
 
 
-#ifdef _MSC_VER
-// Philippose - 30/01/2009
-// MSVC Express Edition Support
-#ifdef MSVC_EXPRESS
 
-// #include <pthread.h>
-
-static pthread_t meshingthread;
-void RunParallel ( void * (*fun)(void *), void * in)
+namespace netgen
 {
-  if (netgen::mparam.parthread) //  && (ntasks == 1) )
-    {
-      pthread_attr_t attr;
-      pthread_attr_init (&attr);
-      // the following call can be removed if not available:
-      pthread_attr_setstacksize(&attr, 1000000);
-      //pthread_create (&meshingthread, &attr, fun, NULL);
-      pthread_create (&meshingthread, &attr, fun, in);
-    }
-  else
-    fun (in);
+  DLL_HEADER MeshingParameters mparam;
 }
 
-#else // Using MS VC++ Standard / Enterprise / Professional edition
-
-// Afx - Threads need different return - value:
-
-static void* (*sfun)(void *);
-unsigned int fun2 (void * val)
-{
-  sfun (val);
-  return 0;
-}
-
-void RunParallel ( void* (*fun)(void *), void * in)
-{
-  sfun = fun;
-  if (netgen::mparam.parthread)
-    AfxBeginThread (fun2, in);
-  //AfxBeginThread (fun2, NULL);
-  else
-    fun (in);
-}
-
-#endif // #ifdef MSVC_EXPRESS
-
-#else  // For #ifdef _MSC_VER
-
-// #include <pthread.h>
- 
-static pthread_t meshingthread;
+static std::thread meshingthread;
 void RunParallel ( void * (*fun)(void *), void * in)
 {
   bool parthread = netgen::mparam.parthread;
@@ -77,42 +34,24 @@ void RunParallel ( void * (*fun)(void *), void * in)
 
   if (parthread)
     {
-      pthread_attr_t attr;
-      pthread_attr_init (&attr);
-      // the following call can be removed if not available:
-      pthread_attr_setstacksize(&attr, 1000000);
-      //pthread_create (&meshingthread, &attr, fun, NULL);
-      pthread_create (&meshingthread, &attr, fun, in);
+      meshingthread = std::thread(fun, in);
+      meshingthread.detach();
     }
   else
     fun (in);
 }
 
-#endif // #ifdef _MSC_VER
-
-
 
 
 
+#include "writeuser.hpp"
 
 namespace netgen
 {
-#include "writeuser.hpp"
+  extern shared_ptr<Mesh> mesh;
+  extern shared_ptr<NetgenGeometry> ng_geometry;
 
-  MeshingParameters mparam;
-
-  // global variable mesh (should not be used in libraries)
-  AutoPtr<Mesh> mesh;
-  // NetgenGeometry * ng_geometry = NULL; // new NetgenGeometry;
-  AutoPtr<NetgenGeometry> ng_geometry;
-
-  // extern NetgenGeometry * ng_geometry;
-  // extern AutoPtr<Mesh> mesh;
-
-#ifndef NOTCL
   extern Tcl_Interp * tcl_interp;
-#endif
-
 
 #ifdef SOCKETS
   extern AutoPtr<ClientSocket> clientsocket;
@@ -133,7 +72,7 @@ void Ng_LoadGeometry (const char * filename)
   // can be used to reset geometry
   if (!filename || strcmp(filename,"")==0) 
     {
-      ng_geometry.Reset (new NetgenGeometry());
+      ng_geometry.reset (new NetgenGeometry());
       return;
     }
 
@@ -142,8 +81,8 @@ void Ng_LoadGeometry (const char * filename)
       NetgenGeometry * hgeom = geometryregister[i]->Load (filename);
       if (hgeom)
 	{
-          ng_geometry.Reset (hgeom);
-	  mesh.Reset();
+          ng_geometry.reset (hgeom);
+	  mesh.reset();
 	  return;
 	}
     }
@@ -156,18 +95,22 @@ void Ng_LoadGeometry (const char * filename)
 
 void Ng_LoadMeshFromStream ( istream & input )
 {
-  mesh.Reset (new Mesh());
+  mesh.reset (new Mesh());
   mesh -> Load(input);
-  
+
+  SetGlobalMesh (mesh);
   for (int i = 0; i < geometryregister.Size(); i++)
     {
       NetgenGeometry * hgeom = geometryregister[i]->LoadFromMeshFile (input);
       if (hgeom)
 	{
-          ng_geometry.Reset (hgeom);
+          ng_geometry.reset (hgeom);
 	  break;
 	}
     }
+  if (!ng_geometry)
+    ng_geometry = make_shared<NetgenGeometry>();
+  mesh->SetGeometry (ng_geometry);
 }
 
 
@@ -175,6 +118,11 @@ void Ng_LoadMeshFromStream ( istream & input )
 
 void Ng_LoadMesh (const char * filename)
 {
+  {
+      ifstream infile(filename);
+      if(!infile.good())
+          throw NgException(string("Error opening file ") + filename);
+  }
 #ifdef PARALLEL
   MPI_Comm_size(MPI_COMM_WORLD, &ntasks);
   MPI_Comm_rank(MPI_COMM_WORLD, &id);
@@ -188,7 +136,7 @@ void Ng_LoadMesh (const char * filename)
 	   strcmp (filename + (strlen (filename)-4), ".vol") != 0 )
         */
 	{
-	  mesh.Reset (new Mesh());
+	  mesh.reset (new Mesh());
 	  ReadFile(*mesh,filename);
 	  
 	  //mesh->SetGlobalH (mparam.maxh);
@@ -273,7 +221,10 @@ void Ng_LoadMesh (const char * filename)
     }
   else
     {
-      mesh.Reset (new Mesh());
+      mesh.reset (new Mesh());
+//       vssolution.SetMesh(mesh);
+//       vsmesh.SetMesh(mesh);
+      SetGlobalMesh (mesh);
       mesh->SendRecvMesh();
     }
 #endif
@@ -452,16 +403,17 @@ void Ng_SetElementIndex(const int ei, const int index)
   mesh->VolumeElement(ei).SetIndex(index);
 }
 
-char * Ng_GetElementMaterial (int ei)
+const char * Ng_GetElementMaterial (int ei)
 {
   static char empty[] = "";
   if (mesh->GetDimension() == 3)
     {
       int ind = mesh->VolumeElement(ei).GetIndex();
       // cout << "ind = " << ind << endl;
-      const char * mat = mesh->GetMaterial (ind);
+      const string * mat = mesh->GetMaterialPtr (ind);
       if (mat)
-	return const_cast<char*> (mat);
+	// return const_cast<char*> (mat);
+        return mat->c_str();
       else 
 	return empty;
     }
@@ -470,24 +422,24 @@ char * Ng_GetElementMaterial (int ei)
     {
       int ind = mesh->SurfaceElement(ei).GetIndex();
       ind = mesh->GetFaceDescriptor(ind).BCProperty();
-      const char * mat = mesh->GetMaterial ( ind );
+      const string * mat = mesh->GetMaterialPtr ( ind );
       if (mat)
-	return const_cast<char*> (mat);
+	return mat->c_str();
       else
 	return empty;
     }
   return 0;
 }
 
-char * Ng_GetDomainMaterial (int dom)
+const char * Ng_GetDomainMaterial (int dom)
 {
   static char empty[] = "";
   // astrid
   if ( 1 ) // mesh->GetDimension() == 3)
     {
-      const char * mat = mesh->GetMaterial(dom);
+      const string * mat = mesh->GetMaterialPtr(dom);
       if (mat)
-	return const_cast<char*> (mat);
+	return mat->c_str();
       else 
 	return empty;      
     }
@@ -596,6 +548,11 @@ char * Ng_GetBCNumBCName (int bcnr)
   return const_cast<char *>(mesh->GetBCName(bcnr).c_str());
 }
 
+char * Ng_GetCD2NumCD2Name (int cd2nr)
+{
+  return const_cast<char *>(mesh->GetCD2Name(cd2nr).c_str());
+}
+
 
 // Inefficient (but maybe safer) version:
 //void Ng_GetBCNumBCName (int bcnr, char * name)
@@ -603,6 +560,7 @@ char * Ng_GetBCNumBCName (int bcnr)
 //    strcpy(name,mesh->GetBCName(bcnr).c_str());
 //}
 
+/*
 void Ng_GetNormalVector (int sei, int locpi, double * nv)
 {
   nv[0] = 0; 
@@ -629,7 +587,7 @@ void Ng_GetNormalVector (int sei, int locpi, double * nv)
 	  nv[2] = n(2);
 	}
 #endif
-      CSGeometry * geometry = dynamic_cast<CSGeometry*> (ng_geometry.Ptr());
+      CSGeometry * geometry = dynamic_cast<CSGeometry*> (ng_geometry.get());
       if (geometry)
 	{
 	  n = geometry->GetSurface (surfi) -> GetNormalVector(p);
@@ -639,7 +597,7 @@ void Ng_GetNormalVector (int sei, int locpi, double * nv)
 	}
     }
 }
-
+*/
 
 
 void Ng_SetPointSearchStartElement(const int el)
@@ -1009,7 +967,7 @@ void Ng_Refine (NG_REFINEMENT_TYPE reftype)
   if (reftype == NG_REFINE_HP)
     biopt.refine_hp = 1;
 
-  const Refinement & ref = ng_geometry->GetRefinement();
+  const Refinement & ref = mesh->GetGeometry()->GetRefinement();
 
   // Refinement * ref;
   MeshOptimize2d * opt = NULL;
@@ -1055,7 +1013,7 @@ void Ng_Refine (NG_REFINEMENT_TYPE reftype)
 
 void Ng_SecondOrder ()
 {
-  const_cast<Refinement&> (ng_geometry->GetRefinement()).MakeSecondOrder(*mesh);
+  const_cast<Refinement&> (mesh->GetGeometry()->GetRefinement()).MakeSecondOrder(*mesh);
   /*
     if (stlgeometry)
     {
@@ -1122,7 +1080,7 @@ void Ng_HPRefinement (int levels, double parameter, bool setorders,
                       bool ref_level)
 {
   NgLock meshlock (mesh->MajorMutex(), true);
-  Refinement & ref = const_cast<Refinement&> (ng_geometry -> GetRefinement());
+  Refinement & ref = const_cast<Refinement&> (mesh->GetGeometry()->GetRefinement());
   HPRefinement (*mesh, &ref, levels, parameter, setorders, ref_level);
   /*
     Refinement * ref;
@@ -1142,12 +1100,20 @@ void Ng_HPRefinement (int levels, double parameter, bool setorders,
 void Ng_HighOrder (int order, bool rational)
 {
   NgLock meshlock (mesh->MajorMutex(), true);
-
+  /*
   mesh -> GetCurvedElements().BuildCurvedElements 
     (&const_cast<Refinement&> (ng_geometry -> GetRefinement()),
      order, rational);
-
+  */
+  /*
+  if (!mesh->GetGeometry())
+    throw NgException ("don't have a geometry for mesh curving");
+  mesh->BuildCurvedElements 
+    (&const_cast<Refinement&> (mesh->GetGeometry()->GetRefinement()),
+     order, rational);
   mesh -> SetNextMajorTimeStamp();
+  */
+  mesh->BuildCurvedElements(order);
 }
 
 
@@ -1498,8 +1464,8 @@ void Ng_UpdateTopology()
 
 Ng_Mesh Ng_SelectMesh (Ng_Mesh newmesh)
 {
-  Mesh * hmesh = mesh.Ptr();
-  mesh.Ptr() = (Mesh*)newmesh;
+  Mesh * hmesh = mesh.get();
+  mesh.reset((Mesh*)newmesh);
   return hmesh;
 }
 
@@ -1620,12 +1586,15 @@ int Ng_GetNVertexElements (int vnr)
     case 2:
       return mesh->GetTopology().GetVertexSurfaceElements(vnr).Size();
     case 1:
+      return mesh->GetTopology().GetVertexSegments(vnr).Size();
+      /*
       {
         int cnt = 0;
         for (SegmentIndex i = 0; i < mesh->GetNSeg(); i++)
           if ( ((*mesh)[i][0] == vnr) || ((*mesh)[i][1] == vnr) ) cnt++;
         return cnt;
       }
+      */
     default:
       cerr << "error: mesh->GetDimension() gives " 
            << mesh->GetDimension() << endl;
@@ -1645,17 +1614,22 @@ void Ng_GetVertexElements (int vnr, int * els)
       }
     case 2:
       {
-        FlatArray<int> ia = mesh->GetTopology().GetVertexSurfaceElements(vnr);
-        for (int i = 0; i < ia.Size(); i++) els[i] = ia[i];
+        FlatArray<SurfaceElementIndex> ia = mesh->GetTopology().GetVertexSurfaceElements(vnr);
+        for (int i = 0; i < ia.Size(); i++) els[i] = ia[i]+1;
         break;
       }
     case 1:
       {
+        FlatArray<SegmentIndex> ia = mesh->GetTopology().GetVertexSegments(vnr);
+        for (int i = 0; i < ia.Size(); i++) els[i] = ia[i]+1;
+        break;
+        /*
         int cnt = 0;
         for (SegmentIndex i = 0; i < mesh->GetNSeg(); i++)
           if ( ((*mesh)[i][0] == vnr) || ((*mesh)[i][1] == vnr) ) 
             els[cnt++] = i+1;
         break;
+        */
       }
     }
 }
@@ -1941,11 +1915,11 @@ int Ng_GetVertex_SurfaceElements( int vnr, int* elems )
     case 3:
       {
         const MeshTopology& topology = mesh->GetTopology();
-        ArrayMem<int,4> indexArray;
+        ArrayMem<SurfaceElementIndex,4> indexArray;
         topology.GetVertexSurfaceElements( vnr, indexArray );
         
         for( int i=0; i<indexArray.Size(); i++ )
-          elems[i] = indexArray[i];
+          elems[i] = indexArray[i]+1;
         
         return indexArray.Size();
       }
@@ -1957,6 +1931,14 @@ int Ng_GetVertex_SurfaceElements( int vnr, int* elems )
             elems[cnt++] = i+1;
         return cnt;
       }
+    case 1:
+      {
+        int cnt = 0;
+        for (int i = 0; i < mesh->pointelements.Size(); i++)
+          if (mesh->pointelements[i].pnum == vnr)
+            elems[cnt++] = i+1;
+        return cnt;
+      }
     }
   return 0;
 }
@@ -1979,7 +1961,7 @@ int Ng_GetVertex_NSurfaceElements( int vnr )
     case 3:
       {
         const MeshTopology& topology = mesh->GetTopology();
-        ArrayMem<int,4> indexArray;
+        ArrayMem<SurfaceElementIndex,4> indexArray;
         topology.GetVertexSurfaceElements( vnr, indexArray );
         return indexArray.Size();
       }
@@ -2093,7 +2075,7 @@ int Ng_Bisect_WithInfo ( const char * refinementfile, double ** qualityloss, int
   biopt.femcode = "fepp";
   biopt.refinementfilename = refinementfile;
   
-  Refinement * ref = const_cast<Refinement*> (&ng_geometry -> GetRefinement());
+  Refinement * ref = const_cast<Refinement*> (&mesh->GetGeometry()->GetRefinement());
   MeshOptimize2d * opt = NULL;
   /*
     if (stlgeometry)
@@ -2128,12 +2110,15 @@ int Ng_Bisect_WithInfo ( const char * refinementfile, double ** qualityloss, int
 #endif
     {
       // ref = new RefinementSurfaces(*geometry);
-      CSGeometry * geometry = dynamic_cast<CSGeometry*> (ng_geometry.Ptr());
+      /*
+        // joachim, oct 2014
+      CSGeometry * geometry = dynamic_cast<CSGeometry*> (ng_geometry.get());
       if (geometry)
 	{
 	  opt = new MeshOptimize2dSurfaces(*geometry);
 	  ref->Set2dOptimizer(opt);
 	}
+      */
     }
 
   if(!mesh->LocalHFunctionGenerated())
@@ -2337,3 +2322,18 @@ void Ng_GetArgs (int & argc, char ** &argv)
   argc = h_argc;
   argv = h_argv;
 }
+
+
+
+void LinkFunction ()
+{
+  Ng_Redraw();
+}
+
+
+
+void Ng_TclCmd(string cmd)
+{
+  lock_guard<mutex> guard(tcl_todo_mutex);
+  *(multithread.tcl_todo) += cmd;
+}
