@@ -7,18 +7,19 @@
 /* Date:    10. Dec. 97                                                   */
 /**************************************************************************/
 
+namespace netgen {
 
-extern
+DLL_HEADER extern
 void WriteFile (int typ,
                 const Mesh & mesh,
-                const CSGeometry & geom,
+                const NetgenGeometry & geom,
                 const char * filename,
                 const char * geomfile = NULL,
                 double h = 0);
 
 
 
-extern
+DLL_HEADER extern
 void ReadFile (Mesh & mesh,
                const string & filename);
 
@@ -29,7 +30,7 @@ void ReadFile (Mesh & mesh,
 
 extern
 void WriteNeutralFormat (const Mesh & mesh,
-                         const CSGeometry & geom,
+                         const NetgenGeometry & geom,
                          const string & filename);
 
 extern
@@ -57,19 +58,19 @@ void WriteVRMLFormat (const Mesh & mesh,
 
 extern
 void WriteFEPPFormat (const Mesh & mesh,
-                      const CSGeometry & geom,
+                      const NetgenGeometry & geom,
                       const string & filename);
 
 extern
 void WriteGmshFormat (const Mesh & mesh,
-                      const CSGeometry & geom,
+                      const NetgenGeometry & geom,
                       const string & filename);
 
 
 // Philippose - 29/01/2009
 // Added GMSH v2.xx Mesh Export support
 void WriteGmsh2Format (const Mesh & mesh,
-                       const CSGeometry & geom,
+                       const NetgenGeometry & geom,
                        const string & filename);
 
 
@@ -78,7 +79,7 @@ void WriteGmsh2Format (const Mesh & mesh,
 extern 
 void WriteOpenFOAM15xFormat (const Mesh & mesh, 
                              const string & casename,
-							 const bool compressed);
+                             const bool compressed);
 
 
 extern
@@ -87,13 +88,13 @@ void WriteUserChemnitz (const Mesh & mesh,
 
 extern
 void WriteJCMFormat (const Mesh & mesh,
-                     const CSGeometry & geom,
+                     const NetgenGeometry & geom,
                      const string & filename);
 
 
 extern
 void WriteDiffPackFormat (const Mesh & mesh,
-                          const CSGeometry & geom,
+                          const NetgenGeometry & geom,
                           const string & filename);
 
 extern
@@ -102,7 +103,7 @@ void WriteTochnogFormat (const Mesh & mesh,
 
 extern
 void WriteTecPlotFormat (const Mesh & mesh,
-                         const CSGeometry & geom,
+                         const NetgenGeometry & geom,
                          const string & filename);
 
 extern
@@ -128,7 +129,7 @@ void WriteElmerFormat (const Mesh & mesh,
 
 extern
 void WriteEdgeElementFormat (const Mesh & mesh,
-                             const CSGeometry & geom,
+                             const NetgenGeometry & geom,
                              const string & filename);
 
 
@@ -153,14 +154,16 @@ void WriteDolfinFormat (const Mesh & mesh,
                         const string & filename);
 
 
-extern void RegisterUserFormats (Array<const char*> & names,
+extern void DLL_HEADER RegisterUserFormats (Array<const char*> & names,
                                  Array<const char*> & extensions);
 
 
-extern bool WriteUserFormat (const string & format,
-                             const Mesh & mesh,
-                             const NetgenGeometry & geom,
-                             const string & filename);
+extern bool DLL_HEADER WriteUserFormat (const string & format,
+                                        const Mesh & mesh,
+                                        // const NetgenGeometry & geom,
+                                        const string & filename);
+
+}
 
 #endif
 
