@@ -15,14 +15,15 @@ namespace netgen
   DenseMatrix :: DenseMatrix (int h, int w)
   {
     if (!w) w = h;
-    width = w;
+    width = w; 
     height = h;
-    if (h*w)
-      data = new double[h*w];
+    int hw = h*w;
+    if (hw)
+      data = new double[hw];
     else 
       data = 0;
 
-    for (int i = 0 ; i < (h * w); i++)
+    for (int i = 0 ; i < (hw); i++)
       data[i] = 0;
   }
 
@@ -160,39 +161,12 @@ namespace netgen
     return *this;
   }
 
-
-
-
-  /*
-    double & DenseMatrix :: operator() (int i, int j)
-    {
-    if (i >= 1 && j >= 1 && i <= height && j <= width)
-    return Elem(i,j);
-    else (*myerr) << "DenseMatrix: index (" << i << "," << j << ") out of range (1.."
-    << height << ",1.." << width << ")\n";
-    static double dummy = 0;
-    return dummy;
-    }
-
-    double DenseMatrix :: operator() (int i, int j) const
-    {
-    if (i >= 1 && j >= 1 && i <= height && j <= width)
-    return Get(i,j);
-    else (*myerr) << "DenseMatrix: index (" << i << "," << j << ") out of range (1.."
-    << height << ",1.." << width << ")\n";
-
-    static double dummy = 0;
-    return dummy;
-    }
-  */
-
   DenseMatrix & DenseMatrix :: operator= (double v)
   {
-    int i;
     double * p = data;
 
     if (data)
-      for (i = width*height; i > 0; i--, p++)
+      for (int i = width*height; i > 0; i--, p++)
         *p = v;
 
     return *this;
@@ -202,11 +176,10 @@ namespace netgen
 
   DenseMatrix & DenseMatrix :: operator*= (double v)
   {
-    int i;
     double * p = data;
 
     if (data)
-      for (i = width*height; i > 0; i--, p++)
+      for (int i = width*height; i > 0; i--, p++)
         *p *= v;
 
     return *this;
@@ -244,12 +217,12 @@ namespace netgen
   {
     double det;
 
-    if (m1.width != m1.height)
+    if (m1.Width() != m1.Height())
       {
         (*myerr) << "CalcInverse: matrix not symmetric" << endl;
         return;
       }
-    if (m1.width != m2.width || m1.height != m2.height)
+    if (m1.Width() != m2.Width() || m1.Height() != m2.Height())
       {
         (*myerr) << "CalcInverse: dim(m2) != dim(m1)" << endl;
         return;
@@ -262,11 +235,12 @@ namespace netgen
         if (det == 0)
           {
             (*myerr) << "CalcInverse: Matrix singular" << endl;
+            (*testout) << "CalcInverse: Matrix singular" << endl;
             return;
           }
 
         det = 1.0 / det;
-        switch (m1.width)
+        switch (m1.Width())
           {
           case 1:
             {
@@ -488,6 +462,7 @@ namespace netgen
             if (max < 1e-20)
               {
                 cerr << "Inverse matrix: matrix singular" << endl;
+                *testout << "Inverse matrix: matrix singular" << endl;
                 return;
               }
 	  
