@@ -88,7 +88,8 @@ namespace netgen
       }
     else
       {
-	pi = points.Append (FrontPoint2 (p, globind, mgi, pointonsurface)) - 1;
+	points.Append (FrontPoint2 (p, globind, mgi, pointonsurface));
+        pi = points.Size()-1;
       }
 
     if (mgi)
@@ -128,7 +129,8 @@ namespace netgen
       }
     else
       {
-	li = lines.Append(FrontLine (INDEX_2(pi1, pi2))) - 1;
+	lines.Append(FrontLine (INDEX_2(pi1, pi2)));
+        li = lines.Size()-1;
       }
 
   
@@ -332,7 +334,8 @@ namespace netgen
 	      {
 		pindex.Append (pi);
 		invpindex[pi] = pindex.Size();
-		loclines[i][j] = locpoints.Append (points[pi].P());
+                locpoints.Append (points[pi].P());
+		loclines[i][j] = locpoints.Size();
 	      }
 	    else
 	      loclines[i][j] = invpindex[pi];
@@ -349,7 +352,8 @@ namespace netgen
 	    // Dist2 (points.Get(i).P(), p0) <= xh2 &&
 	    invpindex[i] <= 0)
 	  {
-	    invpindex[i] = locpoints.Append (points[i].P());
+            locpoints.Append (points[i].P());
+	    invpindex[i] = locpoints.Size();
 	    pindex.Append(i);
 	  }
       }
@@ -492,4 +496,72 @@ namespace netgen
     
     return ((cnt % 2) != 0);
   }
+
+  bool AdFront2 :: SameSide (const Point<2> & lp1, const Point<2> & lp2, 
+                             const Array<int> * testfaces) const
+  {
+    int cnt = 0;
+
+    if (testfaces)
+      {
+        for (int ii = 0; ii < testfaces->Size(); ii++)
+          if (lines[(*testfaces)[ii]].Valid())
+            {
+              int i = (*testfaces)[ii];
+              const Point<3> & p13d = points[lines[i].L().I1()].P();
+              const Point<3> & p23d = points[lines[i].L().I2()].P();
+              
+              Point<2> p1(p13d(0), p13d(1));
+              Point<2> p2(p23d(0), p23d(1));
+              
+              // p1 + alpha v = lp1 + beta vl
+              Vec<2> v = p2-p1;
+              Vec<2> vl = lp2 - lp1;
+              Mat<2,2> mat, inv;
+              Vec<2> rhs, sol;
+              mat(0,0) = v(0);
+              mat(1,0) = v(1);
+              mat(0,1) = -vl(0);
+              mat(1,1) = -vl(1);
+              rhs = lp1-p1;
+              
+              if (Det(mat) == 0) continue;
+              CalcInverse (mat, inv);
+              sol = inv * rhs;
+              if ( (sol(0) >= 0) && (sol(0) <= 1) && (sol(1) >= 0) && (sol(1) <= 1))
+                { cnt++; }
+            }
+
+      }
+    else
+      {
+        for (int i = 0; i < lines.Size(); i++)
+          if (lines[i].Valid())
+            {
+              const Point<3> & p13d = points[lines[i].L().I1()].P();
+              const Point<3> & p23d = points[lines[i].L().I2()].P();
+              
+              Point<2> p1(p13d(0), p13d(1));
+              Point<2> p2(p23d(0), p23d(1));
+              
+              // p1 + alpha v = lp1 + beta vl
+              Vec<2> v = p2-p1;
+              Vec<2> vl = lp2 - lp1;
+              Mat<2,2> mat, inv;
+              Vec<2> rhs, sol;
+              mat(0,0) = v(0);
+              mat(1,0) = v(1);
+              mat(0,1) = -vl(0);
+              mat(1,1) = -vl(1);
+              rhs = lp1-p1;
+              
+              if (Det(mat) == 0) continue;
+              CalcInverse (mat, inv);
+              sol = inv * rhs;
+              if ((sol(0) >= 0) && (sol(0) <= 1) && (sol(1) >= 0) && (sol(1) <= 1))
+                { cnt++; }
+            }
+      }
+    return ((cnt % 2) == 0);
+  }
 }
