@@ -9,7 +9,7 @@ namespace netgen
 
 FrontPoint3 :: FrontPoint3 () 
 { 
-  globalindex = -1;
+  globalindex.Invalidate(); //  = -1;
   nfacetopoint = 0; 
   frontnr = 1000; 
   cluster = 0;
@@ -159,8 +159,9 @@ INDEX AdFront3 :: AddFace (const MiniElement2d & aface)
 
   for (i = 1; i <= aface.GetNP(); i++)
     points[aface.PNum(i)].DecFrontNr (minfn+1);
-
-  int nfn = faces.Append(FrontFace (aface));
+  
+  faces.Append(FrontFace (aface));
+  int nfn = faces.Size();
   faces.Elem(nfn).cluster = cluster;
 
   if (hashon && hashcreated) 
@@ -244,7 +245,7 @@ void AdFront3 :: CreateTrees ()
   pmin = pmin + 0.5 * (pmin - pmax);
 
   delete facetree;
-  facetree = new Box3dTree (pmin, pmax);
+  facetree = new BoxTree<3> (pmin, pmax);
   
   for (i = 1; i <= GetNF(); i++)
     {
@@ -484,17 +485,17 @@ int AdFront3 :: SelectBaseElement ()
 
 
 int AdFront3 :: GetLocals (int fstind,
-			   Array<Point3d > & locpoints,
+			   Array<Point3d, PointIndex::BASE> & locpoints,
 			   Array<MiniElement2d> & locfaces,   // local index
-			   Array<PointIndex> & pindex,
+			   Array<PointIndex, PointIndex::BASE> & pindex,
 			   Array<INDEX> & findex,
 			   INDEX_2_HASHTABLE<int> & getconnectedpairs,
 			   float xh,
 			   float relh,
 			   INDEX& facesplit)
 {
-  static int timer = NgProfiler::CreateTimer ("AdFront3::GetLocals");
-  NgProfiler::RegionTimer reg (timer);
+  // static int timer = NgProfiler::CreateTimer ("AdFront3::GetLocals");
+  // NgProfiler::RegionTimer reg (timer);
 
 
   if (hashon && faces.Size() < 500) { hashon=0; }
@@ -600,12 +601,13 @@ int AdFront3 :: GetLocals (int fstind,
 	  if (invpindex[pi] == -1)
 	    {
 	      pindex.Append (pi);
-	      invpindex[pi] = pindex.Size();  // -1+PointIndex::BASE;
-	      locfaces.Elem(i).PNum(j) = locpoints.Append (points[pi].P());
-	    }
-	  else
-	    locfaces.Elem(i).PNum(j) = invpindex[pi];
-
+              locpoints.Append (points[pi].P());
+	      invpindex[pi] = pindex.Size()-1+PointIndex::BASE;
+            }
+          // locfaces.Elem(i).PNum(j) = locpoints.Append (points[pi].P());
+          // }
+	  // else
+          locfaces.Elem(i).PNum(j) = invpindex[pi];
 	}
     }
 
@@ -655,24 +657,25 @@ int AdFront3 :: GetLocals (int fstind,
 
 // returns all points connected with fi
 void AdFront3 :: GetGroup (int fi,
-			   Array<MeshPoint> & grouppoints,
+			   Array<MeshPoint, PointIndex::BASE> & grouppoints,
 			   Array<MiniElement2d> & groupelements,
-			   Array<PointIndex> & pindex,
+			   Array<PointIndex, PointIndex::BASE> & pindex,
 			   Array<INDEX> & findex) 
 {
   // static Array<char> pingroup;
-  int i, j, changed;
+  int changed;
 
   pingroup.SetSize(points.Size());
 
   pingroup = 0;
-  for (j = 1; j <= 3; j++)
+  for (int j = 1; j <= 3; j++)
     pingroup.Elem (faces.Get(fi).Face().PNum(j)) = 1;
 
   do
     {
       changed = 0;
 
+      /*
       for (i = 1; i <= faces.Size(); i++)
 	if (faces.Get(i).Valid())
 	  {
@@ -691,28 +694,45 @@ void AdFront3 :: GetGroup (int fi,
 		    changed = 1;
 		  }
 	  }
+      */
+      for (auto & f : faces)
+	if (f.Valid())
+	  {
+	    const MiniElement2d & face = f.Face();
+
+	    int fused = 0;
+	    for (int j = 1; j <= 3; j++)
+	      if (pingroup.Elem(face.PNum(j))) 
+		fused++;
+            
+	    if (fused >= 2)
+	      for (int j = 1; j <= 3; j++)
+		if (!pingroup.Elem(face.PNum(j)))
+		  {
+		    pingroup.Elem(face.PNum(j)) = 1;
+		    changed = 1;
+		  }
+	  }
 
     }
   while (changed);
 
-
-  //  static Array<int> invpindex;
   invpindex.SetSize (points.Size());
-  
 
-  for (PointIndex pi = points.Begin(); pi < points.End(); pi++)
-    if (points.Get(i).Valid())
+  // for (PointIndex pi = points.Begin(); pi < points.End(); pi++)
+  for (PointIndex pi : points.Range())
+    if (points[pi].Valid())
       {
 	grouppoints.Append (points[pi].P());
 	pindex.Append (pi);
 	invpindex[pi] = pindex.Size();
       }
 
-  for (i = 1; i <= faces.Size(); i++)
+  for (int i = 1; i <= faces.Size(); i++)
     if (faces.Get(i).Valid())
       {
 	int fused = 0;
-	for (j = 1; j <= 3; j++)
+	for (int j = 1; j <= 3; j++)
 	  if (pingroup.Get(faces.Get(i).Face().PNum(j)))
 	    fused++;
 
@@ -722,37 +742,28 @@ void AdFront3 :: GetGroup (int fi,
 	    findex.Append (i);
 	  }
       }
-      
-  for (i = 1; i <= groupelements.Size(); i++)
-    for (j = 1; j <= 3; j++)
+
+  /*
+  for (int i = 1; i <= groupelements.Size(); i++)
+    for (int j = 1; j <= 3; j++)
       {
 	groupelements.Elem(i).PNum(j) =
 	  invpindex.Get(groupelements.Elem(i).PNum(j));
       }
-
-  /*
-  for (i = 1; i <= groupelements.Size(); i++)
-    for (j = 1; j <= 3; j++)
-      for (k = 1; k <= grouppoints.Size(); k++)
-        if (pindex.Get(k) == groupelements.Get(i).PNum(j))
-          {
-	    groupelements.Elem(i).PNum(j) = k;
-	    break;
-          }
-  */          
+  */
+  for (auto & e : groupelements)
+    for (int j = 1; j <= 3; j++)
+      e.PNum(j) = invpindex.Get(e.PNum(j));
 }
 
 
 void AdFront3 :: SetStartFront (int /* baseelnp */)
 {
-  INDEX i;
-  int j;
-
-  for (i = 1; i <= faces.Size(); i++)
+  for (INDEX i = 1; i <= faces.Size(); i++)
     if (faces.Get(i).Valid())
       {
 	const MiniElement2d & face = faces.Get(i).Face();
-	for (j = 1; j <= 3; j++)
+	for (int j = 1; j <= 3; j++)
 	  points[face.PNum(j)].DecFrontNr(0);
       }
 
