@@ -24,15 +24,16 @@ namespace netgen
   {
   public:
     typedef ::netgen::T_POINTS T_POINTS;
-    typedef Array<Element> T_VOLELEMENTS;
-    typedef Array<Element2d> T_SURFELEMENTS;
+    typedef Array<Element, 0, size_t> T_VOLELEMENTS;
+    // typedef Array<Element2d, 0, SurfaceElementIndex> T_SURFELEMENTS;
+    typedef Array<Element2d, 0, size_t> T_SURFELEMENTS;
 
   private:
     /// point coordinates
     T_POINTS points;
 
     /// line-segments at edges
-    Array<Segment> segments;
+    Array<Segment, 0, size_t> segments;
     /// surface elements, 2d-inner elements
     T_SURFELEMENTS surfelements;
     /// volume elements
@@ -42,7 +43,7 @@ namespace netgen
 
 
     /// surface indices at boundary nodes
-    TABLE<int,PointIndex::BASE> surfacesonnode;
+    // TABLE<int,PointIndex::BASE> surfacesonnode;
     /// boundary edges  (1..normal bedge, 2..segment)
     INDEX_2_CLOSED_HASHTABLE<int> * boundaryedges;
     ///
@@ -82,11 +83,14 @@ namespace netgen
     Array<EdgeDescriptor> edgedecoding;
 
     /// sub-domain materials 
-    Array<char*> materials;
+    Array<string*> materials;
 
     /// labels for boundary conditions
     Array<string*> bcnames;
 
+    /// labels for co dim 2 bboundary conditions
+    Array<string*> cd2names;
+
     /// Periodic surface, close surface, etc. identifications
     Identifications * ident;
 
@@ -95,12 +99,12 @@ namespace netgen
     int numvertices;
 
     /// geometric search tree for interval intersection search
-    Box3dTree * elementsearchtree;
+    BoxTree<3> * elementsearchtree;
     /// time stamp for tree
     mutable int elementsearchtreets;
 
     /// element -> face, element -> edge etc ...
-    class MeshTopology * topology;
+    MeshTopology topology;
     /// methods for high order elements
     class CurvedElements * curvedelems;
 
@@ -139,6 +143,8 @@ namespace netgen
 
 #endif
 
+    
+    shared_ptr<NetgenGeometry> geometry;
 
   private:
     void BuildBoundaryEdges(void);
@@ -165,7 +171,7 @@ namespace netgen
     /// number of refinement levels
     int mglevels;
     /// refinement hierarchy
-    Array<INDEX_2,PointIndex::BASE> mlbetweennodes;
+    Array<PointIndices<2>,PointIndex::BASE> mlbetweennodes;
     /// parent element of volume element
     Array<int> mlparentelement;
     /// parent element of surface element
@@ -187,32 +193,34 @@ namespace netgen
     void ClearSurfaceElements();
 
     ///
-    void ClearVolumeElements()
+    DLL_HEADER void ClearVolumeElements()
     {
       volelements.SetSize(0); 
       timestamp = NextTimeStamp();
     }
 
     ///
-    void ClearSegments()
+    DLL_HEADER void ClearSegments()
     { 
       segments.SetSize(0); 
       timestamp = NextTimeStamp();
     }
-  
+    
     ///
     bool TestOk () const;
 
     void SetAllocSize(int nnodes, int nsegs, int nsel, int nel);
-
+    
 
     DLL_HEADER PointIndex AddPoint (const Point3d & p, int layer = 1);
     DLL_HEADER PointIndex AddPoint (const Point3d & p, int layer, POINTTYPE type);
 
     int GetNP () const { return points.Size(); }
 
+    // [[deprecated("Use Point(PointIndex) instead of int !")]]        
     MeshPoint & Point(int i) { return points.Elem(i); }
     MeshPoint & Point(PointIndex pi) { return points[pi]; }
+    // [[deprecated("Use Point(PointIndex) instead of int !")]]            
     const MeshPoint & Point(int i) const { return points.Get(i); }
     const MeshPoint & Point(PointIndex pi) const { return points[pi]; }
 
@@ -226,16 +234,20 @@ namespace netgen
     DLL_HEADER SegmentIndex AddSegment (const Segment & s);
     void DeleteSegment (int segnr)
     {
-      segments.Elem(segnr)[0] = PointIndex::BASE-1;
-      segments.Elem(segnr)[1] = PointIndex::BASE-1;
+      segments.Elem(segnr)[0].Invalidate();
+      segments.Elem(segnr)[1].Invalidate();
     }
+    /*
     void FullDeleteSegment (int segnr)  // von wem ist das ???
     {
       segments.Delete(segnr-PointIndex::BASE);
     }
+    */
 
     int GetNSeg () const { return segments.Size(); }
+    // [[deprecated("Use LineSegment(SegmentIndex) instead of int !")]]                
     Segment & LineSegment(int i) { return segments.Elem(i); }
+    // [[deprecated("Use LineSegment(SegmentIndex) instead of int !")]]                    
     const Segment & LineSegment(int i) const { return segments.Get(i); }
 
     Segment & LineSegment(SegmentIndex si) { return segments[si]; }
@@ -243,26 +255,41 @@ namespace netgen
     const Segment & operator[] (SegmentIndex si) const { return segments[si]; }
     Segment & operator[] (SegmentIndex si) { return segments[si]; }
 
-
-
+    /*
+    const Array<Segment> & LineSegments() const { return segments; }
+    Array<Segment> & LineSegments() { return segments; }
+    */
+    const auto & LineSegments() const { return segments; }
+    auto & LineSegments() { return segments; }
+    
+    Array<Element0d> pointelements;  // only via python interface
 
     DLL_HEADER SurfaceElementIndex AddSurfaceElement (const Element2d & el);
+    // write to pre-allocated container, thread-safe
+    DLL_HEADER void SetSurfaceElement (SurfaceElementIndex sei, const Element2d & el);
+    
+    // [[deprecated("Use DeleteSurfaceElement(SurfaceElementIndex) instead of int !")]]
     void DeleteSurfaceElement (int eli)
     { 
       surfelements.Elem(eli).Delete();
-      surfelements.Elem(eli).PNum(1) = -1; 
-      surfelements.Elem(eli).PNum(2) = -1; 
-      surfelements.Elem(eli).PNum(3) = -1; 
+      surfelements.Elem(eli).PNum(1).Invalidate();
+      surfelements.Elem(eli).PNum(2).Invalidate();
+      surfelements.Elem(eli).PNum(3).Invalidate();
       timestamp = NextTimeStamp();
     }
 
     void DeleteSurfaceElement (SurfaceElementIndex eli)
     {
-      DeleteSurfaceElement (int(eli)+1);
+      for (auto & p : surfelements[eli].PNums()) p.Invalidate();
+      surfelements[eli].Delete();
+      timestamp = NextTimeStamp();
     }
 
     int GetNSE () const { return surfelements.Size(); }
+
+    // [[deprecated("Use SurfaceElement(SurfaceElementIndex) instead of int !")]]    
     Element2d & SurfaceElement(int i) { return surfelements.Elem(i); }
+    // [[deprecated("Use SurfaceElement(SurfaceElementIndex) instead of int !")]]        
     const Element2d & SurfaceElement(int i) const { return surfelements.Get(i); }
     Element2d & SurfaceElement(SurfaceElementIndex i) { return surfelements[i]; }
     const Element2d & SurfaceElement(SurfaceElementIndex i) const { return surfelements[i]; }
@@ -272,15 +299,22 @@ namespace netgen
     Element2d & operator[] (SurfaceElementIndex ei)
     { return surfelements[ei]; }
 
+    const T_SURFELEMENTS & SurfaceElements() const { return surfelements; }
+    T_SURFELEMENTS & SurfaceElements() { return surfelements; }
+
   
     DLL_HEADER void RebuildSurfaceElementLists ();
     DLL_HEADER void GetSurfaceElementsOfFace (int facenr, Array<SurfaceElementIndex> & sei) const;
 
     DLL_HEADER ElementIndex AddVolumeElement (const Element & el);
+    // write to pre-allocated container, thread-safe
+    DLL_HEADER void SetVolumeElement (ElementIndex sei, const Element & el);
 
     int GetNE () const { return volelements.Size(); }
 
+    // [[deprecated("Use VolumeElement(ElementIndex) instead of int !")]]    
     Element & VolumeElement(int i) { return volelements.Elem(i); }
+    // [[deprecated("Use VolumeElement(ElementIndex) instead of int !")]]        
     const Element & VolumeElement(int i) const { return volelements.Get(i); }
     Element & VolumeElement(ElementIndex i) { return volelements[i]; }
     const Element & VolumeElement(ElementIndex i) const { return volelements[i]; }
@@ -295,9 +329,8 @@ namespace netgen
     ELEMENTTYPE ElementType (ElementIndex i) const 
     { return (volelements[i].flags.fixed) ? FIXEDELEMENT : FREEELEMENT; }
 
-    const T_VOLELEMENTS & VolumeElements() const { return volelements; }
-    T_VOLELEMENTS & VolumeElements() { return volelements; }
-
+    const auto & VolumeElements() const { return volelements; }
+    auto & VolumeElements() { return volelements; }
 
     ///
     DLL_HEADER double ElementError (int eli, const MeshingParameters & mp) const;
@@ -307,20 +340,16 @@ namespace netgen
     ///
     void ClearLockedPoints ();
 
-    const Array<PointIndex> & LockedPoints() const
-    { return lockedpoints; }
+    const auto & LockedPoints() const { return lockedpoints; }
 
     /// Returns number of domains
-    int GetNDomains() const;
-
+    DLL_HEADER int GetNDomains() const;
     ///
-    int GetDimension() const 
-    { return dimension; }
-    void SetDimension(int dim)
-    { dimension = dim; }
+    int GetDimension() const { return dimension; }
+    void SetDimension (int dim) { dimension = dim; }
 
     /// sets internal tables
-    void CalcSurfacesOfNode ();
+    DLL_HEADER void CalcSurfacesOfNode ();
 
     /// additional (temporarily) fix points 
     void FixPoints (const BitArray & fixpoints);
@@ -373,7 +402,7 @@ namespace netgen
     /// Calculates localh 
     DLL_HEADER void CalcLocalH (double grading);
     ///
-    DLL_HEADER void SetLocalH (const Point3d & pmin, const Point3d & pmax, double grading);
+    DLL_HEADER void SetLocalH (netgen::Point<3> pmin, netgen::Point<3> pmax, double grading);
     ///
     DLL_HEADER void RestrictLocalH (const Point3d & p, double hloc);
     ///
@@ -386,20 +415,22 @@ namespace netgen
     ///
     DLL_HEADER void RestrictLocalH (resthtype rht, int nr, double loch);
     ///
-    DLL_HEADER void LoadLocalMeshSize (const char * meshsizefilename);
+    DLL_HEADER void LoadLocalMeshSize (const string & meshsizefilename);
     ///
     DLL_HEADER void SetGlobalH (double h);
     ///
-    void SetMinimalH (double h);
+	DLL_HEADER void SetMinimalH (double h);
     ///
-    double MaxHDomain (int dom) const;
+	DLL_HEADER double MaxHDomain (int dom) const;
     ///
-    void SetMaxHDomain (const Array<double> & mhd);
+	DLL_HEADER void SetMaxHDomain (const Array<double> & mhd);
     ///
-    double GetH (const Point3d & p) const;
+    DLL_HEADER double GetH (const Point3d & p) const;
     ///
     double GetMinH (const Point3d & pmin, const Point3d & pmax);
     ///
+    bool HasLocalHFunction () { return lochfunc != nullptr; }
+    ///
     LocalH & LocalHFunction () { return * lochfunc; }
     ///
     bool LocalHFunctionGenerated(void) const { return (lochfunc != NULL); }
@@ -422,15 +453,15 @@ namespace netgen
     bool HasOpenQuads () const;
 
     /// split into connected pieces
-    void SplitIntoParts ();
+	DLL_HEADER void SplitIntoParts ();
 
     /// 
-    void SplitSeparatedFaces ();
+	DLL_HEADER void SplitSeparatedFaces ();
 
     /// Refines mesh and projects points to true surface
     // void Refine (int levels, const CSGeometry * geom);
   
-
+    
     bool BoundaryEdge (PointIndex pi1, PointIndex pi2) const
     {
       if(!boundaryedges)
@@ -461,22 +492,26 @@ namespace netgen
     */
     DLL_HEADER void Compress ();
 
-    ///
-    void Save (ostream & outfile) const;
-    ///
-    void Load (istream & infile);
-    ///
-    void Merge (istream & infile, const int surfindex_offset = 0);
-    ///
-    void Save (const string & filename) const;
-    ///
-    void Load (const string & filename);
-    ///
-    void Merge (const string & filename, const int surfindex_offset = 0);
-
+    /// first vertex has lowest index
+    void OrderElements(); 
 
     ///
-    void ImproveMesh (const MeshingParameters & mp, OPTIMIZEGOAL goal = OPT_QUALITY);
+	DLL_HEADER void Save (ostream & outfile) const;
+    ///
+	DLL_HEADER void Load (istream & infile);
+    ///
+	DLL_HEADER void Merge (istream & infile, const int surfindex_offset = 0);
+    ///
+	DLL_HEADER void Save (const string & filename) const;
+    ///
+	DLL_HEADER void Load (const string & filename);
+    ///
+	DLL_HEADER void Merge (const string & filename, const int surfindex_offset = 0);
+
+
+    DLL_HEADER void DoArchive (ngstd::Archive & archive);
+    ///
+	DLL_HEADER void ImproveMesh (const MeshingParameters & mp, OPTIMIZEGOAL goal = OPT_QUALITY);
 
     ///
     void ImproveMeshJacobian (const MeshingParameters & mp, OPTIMIZEGOAL goal = OPT_QUALITY, const BitArray * usepoint = NULL);
@@ -510,19 +545,19 @@ namespace netgen
        triangle angles min/max, tetangles min/max
        if null, output results on cout
     */
-    void CalcMinMaxAngle (double badellimit, double * retvalues = NULL);
+	DLL_HEADER void CalcMinMaxAngle (double badellimit, double * retvalues = NULL);
 
     /*
       Marks elements which are dangerous to refine
       return: number of illegal elements
     */
-    int MarkIllegalElements ();
+	DLL_HEADER int MarkIllegalElements ();
 
     /// orient surface mesh, for one sub-domain only
-    void SurfaceMeshOrientation ();
+	DLL_HEADER void SurfaceMeshOrientation ();
 
     /// convert mixed element mesh to tet-mesh
-    void Split2Tets();
+	DLL_HEADER void Split2Tets();
 
 
     /// build box-search tree
@@ -558,24 +593,42 @@ namespace netgen
 
     ///
     int AddFaceDescriptor(const FaceDescriptor& fd)
-    { return facedecoding.Append(fd); }
+    { facedecoding.Append(fd); return facedecoding.Size(); }
 
     int AddEdgeDescriptor(const EdgeDescriptor & fd)
-    { return edgedecoding.Append(fd) - 1; }
+    { edgedecoding.Append(fd); return edgedecoding.Size() - 1; }
 
     ///
-    DLL_HEADER void SetMaterial (int domnr, const char * mat);
+    DLL_HEADER void SetMaterial (int domnr, const string & mat);
     ///
-    const char * GetMaterial (int domnr) const;
+    DLL_HEADER const string & GetMaterial (int domnr) const;
+    DLL_HEADER static string defaultmat;
+    const string * GetMaterialPtr (int domnr) const // 1-based
+    {
+      return domnr <= materials.Size() ? materials.Get(domnr) : &defaultmat;
+    }
     
     DLL_HEADER void SetNBCNames ( int nbcn );
 
     DLL_HEADER void SetBCName ( int bcnr, const string & abcname );
 
-    const string & GetBCName ( int bcnr ) const;
+    DLL_HEADER const string & GetBCName ( int bcnr ) const;
 
-    string * GetBCNamePtr ( int bcnr )
-    { return bcnames[bcnr]; }
+    DLL_HEADER void SetNCD2Names (int ncd2n);
+    DLL_HEADER void SetCD2Name (int cd2nr, const string & abcname);
+
+    DLL_HEADER const string & GetCD2Name (int cd2nr ) const;
+    DLL_HEADER static string cd2_default_name;
+    string * GetCD2NamePtr (int cd2nr ) const
+    {
+      if (cd2nr < cd2names.Size() && cd2names[cd2nr]) return cd2names[cd2nr];
+      return &cd2_default_name;
+    }
+    size_t GetNCD2Names() const { return cd2names.Size(); }
+
+    DLL_HEADER static string default_bc;
+    string * GetBCNamePtr (int bcnr) const
+    { return (bcnr < bcnames.Size() && bcnames[bcnr]) ? bcnames[bcnr] : &default_bc; }
 
     ///
     void ClearFaceDescriptors()
@@ -626,7 +679,8 @@ namespace netgen
     Identifications & GetIdentifications () { return *ident; }
     /// return periodic, close surface etc. identifications
     const Identifications & GetIdentifications () const { return *ident; }
-
+    ///
+    bool HasIdentifications() const { return ident != nullptr; }
 
     void InitPointCurve(double red = 1, double green = 0, double blue = 0) const;
     void AddPointCurvePoint(const Point3d & pt) const;
@@ -648,17 +702,21 @@ namespace netgen
   
 
 
-    bool PureTrigMesh (int faceindex = 0) const;
-    bool PureTetMesh () const;
+    DLL_HEADER bool PureTrigMesh (int faceindex = 0) const;
+    DLL_HEADER bool PureTetMesh () const;
 
 
-    const class MeshTopology & GetTopology () const
-    { return *topology; }
+    const MeshTopology & GetTopology () const
+    { return topology; }
 
-    void UpdateTopology();
+    DLL_HEADER void UpdateTopology (TaskManager tm = &DummyTaskManager,
+                                    Tracer tracer = &DummyTracer);
   
     class CurvedElements & GetCurvedElements () const
     { return *curvedelems; }
+    
+    DLL_HEADER void BuildCurvedElements  (const class Refinement * ref, int aorder, bool arational = false);
+    DLL_HEADER void BuildCurvedElements  (int aorder);
 
     const class AnisotropicClusters & GetClusters () const
     { return *clusters; }
@@ -679,10 +737,8 @@ namespace netgen
 	  area += Cross ( mesh[sel[1]]-mesh[sel[0]],
 			  mesh[sel[2]]-mesh[sel[0]] ).Length() / 2;
 	else
-	  area += Cross (Vec3d (mesh.Point (sel.PNum(1)),
-				mesh.Point (sel.PNum(3))),
-			 Vec3d (mesh.Point (sel.PNum(1)),
-				mesh.Point (sel.PNum(4)))).Length() / 2;;
+	  area += Cross (Vec3d (mesh[sel.PNum(1)], mesh[sel.PNum(3)]),
+			 Vec3d (mesh[sel.PNum(1)], mesh[sel.PNum(4)])).Length() / 2;;
       }
       void ReCalc ()
       {
@@ -716,6 +772,15 @@ namespace netgen
     NgMutex & MajorMutex ()   { return majormutex; }
 
 
+    shared_ptr<NetgenGeometry> GetGeometry() const
+    { 
+      return geometry; 
+    }
+    void SetGeometry (shared_ptr<NetgenGeometry> geom) 
+    {
+      geometry = geom;
+    }
+
     ///
     void SetUserData(const char * id, Array<int> & data);
     ///
