@@ -8,6 +8,7 @@ namespace netgen
   extern const char * prismrules2[];
   extern const char * pyramidrules[];
   extern const char * pyramidrules2[];
+  extern const char * hexrules[];
 
 
   // extern double teterrpow; 
@@ -17,8 +18,8 @@ namespace netgen
      int meshed;
 
      Array<INDEX_2> connectednodes;
-
-     if (&mesh3d.LocalHFunction() == NULL) mesh3d.CalcLocalH(mp.grading);
+     
+     if (!mesh3d.HasLocalHFunction()) mesh3d.CalcLocalH(mp.grading);
 
      mesh3d.Compress();
 
@@ -31,6 +32,8 @@ namespace netgen
      int nonconsist = 0;
      for (int k = 1; k <= mesh3d.GetNDomains(); k++)
      {
+       if(mp.only3D_domain_nr && mp.only3D_domain_nr !=k)
+	 continue;
         PrintMessage (3, "Check subdomain ", k, " / ", mesh3d.GetNDomains());
 
         mesh3d.FindOpenElements(k);
@@ -62,6 +65,8 @@ namespace netgen
 
      for (int k = 1; k <= mesh3d.GetNDomains(); k++)
        {
+	 if(mp.only3D_domain_nr && mp.only3D_domain_nr !=k)
+	   continue;
 	 if (multithread.terminate)
            break;
 	 
@@ -70,7 +75,7 @@ namespace netgen
 	 (*testout) << "Meshing subdomain " << k << endl;
 	 
 	 mp.maxh = min2 (globmaxh, mesh3d.MaxHDomain(k));
-	 
+
 	 mesh3d.CalcSurfacesOfNode();
 	 mesh3d.FindOpenElements(k);
 	 
@@ -85,7 +90,7 @@ namespace netgen
 	   {
 	     const Element2d & el = mesh3d[sei];
 	     if (el.IsDeleted() ) continue;
-	     
+
 	     if (mesh3d.GetFaceDescriptor(el.GetIndex()).DomainIn() == k ||
 		 mesh3d.GetFaceDescriptor(el.GetIndex()).DomainOut() == k)
 	       
@@ -94,9 +99,12 @@ namespace netgen
 	   }
 	 domain_bbox.Increase (0.01 * domain_bbox.Diam());
 	 
-	
-        for (int qstep = 1; qstep <= 3; qstep++)
+
+         for (int qstep = 0; qstep <= 3; qstep++)
+           // for (int qstep = 0; qstep <= 0; qstep++)  // for hex-filling
 	  {
+            if (qstep == 0 && !mp.try_hexes) continue;
+            
 	    // cout << "openquads = " << mesh3d.HasOpenQuads() << endl;
 	    if (mesh3d.HasOpenQuads())
 	      {
@@ -105,6 +113,10 @@ namespace netgen
 		const char ** rulep = NULL;
 		switch (qstep)
 		  {
+		  case 0:
+                    rulefile = "/Users/joachim/gitlab/netgen/rules/hexa.rls";
+                    rulep = hexrules;
+		    break;
 		  case 1:
 		    rulefile += "/rules/prisms2.rls";
 		    rulep = prismrules2;
@@ -119,8 +131,8 @@ namespace netgen
 		    break;
 		  }
 		
-		//		Meshing3 meshing(rulefile);
-		Meshing3 meshing(rulep); 
+                // Meshing3 meshing(rulefile);
+                Meshing3 meshing(rulep); 
 		
 		MeshingParameters mpquad = mp;
 		
@@ -172,9 +184,12 @@ namespace netgen
 
            mesh3d.FindOpenElements(k);
 
-
+           /*
            for (PointIndex pi = mesh3d.Points().Begin(); pi < mesh3d.Points().End(); pi++)
               meshing.AddPoint (mesh3d[pi], pi);
+           */
+           for (PointIndex pi : mesh3d.Points().Range())
+              meshing.AddPoint (mesh3d[pi], pi);
 
            for (int i = 1; i <= mesh3d.GetNOpenElements(); i++)
               meshing.AddBoundaryElement (mesh3d.OpenElement(i));
@@ -193,11 +208,11 @@ namespace netgen
 
         int cntsteps = 0;
         if (mesh3d.GetNOpenElements())
-           do
-           {
+          do
+            {
               if (multithread.terminate)
-                 break;
-
+                break;
+              
               mesh3d.FindOpenElements(k);
               PrintMessage (5, mesh3d.GetNOpenElements(), " open faces");
               cntsteps++;
@@ -233,10 +248,10 @@ namespace netgen
               mp.giveuptol = 15 + 10 * cntsteps; 
               mp.sloppy = 5;
               meshing.GenerateMesh (mesh3d, mp);
-
+              
               for (ElementIndex ei = oldne; ei < mesh3d.GetNE(); ei++)
                  mesh3d[ei].SetIndex (k);
-
+              
 
               mesh3d.CalcSurfacesOfNode();
               mesh3d.FindOpenElements(k);
@@ -273,17 +288,17 @@ namespace netgen
                  mesh3d.FindOpenElements(k);
               }
               else
-              {
+                {
                  meshed = 1;
                  PrintMessage (1, "Success !");
-              }
-           }
-           while (!meshed);
-
-           PrintMessage (1, mesh3d.GetNP(), " points, ",
-              mesh3d.GetNE(), " elements");
-     }
-
+                }
+            }
+          while (!meshed);
+        
+        PrintMessage (1, mesh3d.GetNP(), " points, ",
+                      mesh3d.GetNE(), " elements");
+       }
+     
      mp.maxh = globmaxh;
 
      MeshQuality3d (mesh3d);
@@ -569,7 +584,7 @@ namespace netgen
     if (res)
       {
 	mesh3d.FindOpenElements();
-	PrintSysError (1, "Open elemetns: ", mesh3d.GetNOpenElements());
+	PrintSysError (1, "Open elements: ", mesh3d.GetNOpenElements());
 	exit (1);
       }
 
@@ -639,7 +654,8 @@ namespace netgen
 	MeshOptimize3d optmesh(mp);
 
 	// teterrpow = mp.opterrpow;
-	for (size_t j = 1; j <= strlen(mp.optimize3d); j++)
+	// for (size_t j = 1; j <= strlen(mp.optimize3d); j++)
+        for (size_t j = 1; j <= mp.optimize3d.length(); j++)
 	  {
 	    if (multithread.terminate)
 	      break;
@@ -694,7 +710,7 @@ namespace netgen
 	  break;
 
 	PrintMessage (5, nillegal, " illegal tets");
-	optmesh.SplitImprove (mesh3d, OPT_LEGAL);
+        optmesh.SplitImprove (mesh3d, OPT_LEGAL);
 
 	mesh3d.MarkIllegalElements();  // test
 	optmesh.SwapImprove (mesh3d, OPT_LEGAL);
