@@ -57,6 +57,7 @@ namespace netgen
 
 
   Segment :: Segment() 
+    : is_curved(false)
   {
     pnums[0] = -1;
     pnums[1] = -1; 
@@ -86,11 +87,11 @@ namespace netgen
       epgeominfo[1].dist = 0;
     */
 
-    bcname = 0;
+    bcname = nullptr;
   }    
 
   Segment::Segment (const Segment & other)
-    :
+    : 
     edgenr(other.edgenr),
     singedge_left(other.singedge_left),
     singedge_right(other.singedge_right),
@@ -104,6 +105,7 @@ namespace netgen
     surfnr2(other.surfnr2),
     epgeominfo(),
     meshdocval(other.meshdocval),
+    is_curved(other.is_curved),
     hp_elnr(other.hp_elnr)
   {
     for (int j = 0; j < 3; j++)
@@ -140,10 +142,20 @@ namespace netgen
 	meshdocval = other.meshdocval;
 	hp_elnr = other.hp_elnr;
 	bcname = other.bcname;
+        is_curved = other.is_curved;
       }
     
     return *this;
   }
+  
+  ngstd::Archive & Segment :: DoArchive (ngstd::Archive & ar)
+  {
+    return ar & pnums[0] & pnums[1] & pnums[2]
+      & edgenr & singedge_left & singedge_right
+      & si & cd2i & domin & domout & tlosurf
+      & surfnr1 & surfnr2
+      & bcname;
+  }
 
 
   ostream & operator<<(ostream  & s, const Segment & seg)
@@ -154,10 +166,9 @@ namespace netgen
       << " si = " << seg.si << ", edgenr = " << seg.edgenr;
     return s;
   }
-
-
+  /*
   Element2d :: Element2d ()
-  { 
+  {
     for (int i = 0; i < ELEMENT2D_MAXPOINTS; i++)
       {
 	pnum[i] = 0;
@@ -172,9 +183,9 @@ namespace netgen
     orderx = ordery = 1;
     refflag = 1;
     strongrefflag = false;
+    is_curved = false;
   } 
-
-
+  */
   Element2d :: Element2d (int anp)
   { 
     for (int i = 0; i < ELEMENT2D_MAXPOINTS; i++)
@@ -197,6 +208,7 @@ namespace netgen
     orderx = ordery = 1;
     refflag = 1;
     strongrefflag = false;
+    is_curved = (np >= 4); // false;
   } 
 
   Element2d :: Element2d (ELEMENT_TYPE atyp)
@@ -216,6 +228,7 @@ namespace netgen
     orderx = ordery = 1;
     refflag = 1;
     strongrefflag = false;
+    is_curved = (np >= 4); // false;
   } 
 
 
@@ -240,6 +253,7 @@ namespace netgen
     deleted = 0;
     visible = 1;
     orderx = ordery = 1;
+    is_curved = false;
   }
 
   Element2d :: Element2d (int pi1, int pi2, int pi3, int pi4)
@@ -263,6 +277,7 @@ namespace netgen
     deleted = 0;
     visible = 1;
     orderx = ordery = 1;
+    is_curved = true;
   }
 
 
@@ -420,7 +435,7 @@ namespace netgen
       case TRIG: pp = &eltriqp[0][0]; break;
       case QUAD: pp = &elquadqp[ip-1][0]; break;
       default:
-        PrintSysError ("Element2d::GetIntegrationPoint, illegal type ", typ);
+        PrintSysError ("Element2d::GetIntegrationPoint, illegal type ", int(typ));
       }
 
     p.X() = pp[0];
@@ -475,7 +490,7 @@ namespace netgen
       case TRIG: dshapep = &ipdtrig.Get(ip)->dshape; break;
       case QUAD: dshapep = &ipdquad.Get(ip)->dshape; break;
       default:
-        PrintSysError ("Element2d::GetTransformation, illegal type ", typ);
+        PrintSysError ("Element2d::GetTransformation, illegal type ", int(typ));
       }
   
     CalcABt (pmat, *dshapep, trans);
@@ -504,7 +519,7 @@ namespace netgen
         shape(3) = (1-p.X()) * p.Y();
         break;
       default:
-        PrintSysError ("Element2d::GetShape, illegal type ", typ);
+        PrintSysError ("Element2d::GetShape, illegal type ", int(typ));
       }
   }
 
@@ -530,6 +545,35 @@ namespace netgen
           shape(3) = (1-p(0))*   p(1) ;
           break;
         }
+
+      default:
+        throw NgException ("illegal element type in GetShapeNew");
+      }
+  }
+
+  template <typename T>
+  void Element2d :: GetShapeNew (const Point<2,T> & p, TFlatVector<T> shape) const
+  {
+    switch (typ)
+      {
+      case TRIG:
+        {
+          shape(0) = p(0);
+          shape(1) = p(1);
+          shape(2) = 1-p(0)-p(1);
+          break;
+        }
+
+      case QUAD:
+        {
+          shape(0) = (1-p(0))*(1-p(1));
+          shape(1) =    p(0) *(1-p(1));
+          shape(2) =    p(0) *   p(1) ;
+          shape(3) = (1-p(0))*   p(1) ;
+          break;
+        }
+      default:
+        throw NgException ("illegal element type in GetShapeNew");
       }
   }
 
@@ -574,20 +618,20 @@ namespace netgen
         break;
 
       default:
-        PrintSysError ("Element2d::GetDShape, illegal type ", typ);
+        PrintSysError ("Element2d::GetDShape, illegal type ", int(typ));
       }
   }
 
 
-
+  template <typename T>
   void Element2d :: 
-  GetDShapeNew (const Point<2> & p, MatrixFixWidth<2> & dshape) const
+  GetDShapeNew (const Point<2,T> & p, MatrixFixWidth<2,T> & dshape) const
   {
     switch (typ)
       {
       case TRIG:
         {
-          dshape = 0;
+          dshape = T(0.0);
           dshape(0,0) = 1;
           dshape(1,1) = 1;
           dshape(2,0) = -1;
@@ -609,6 +653,8 @@ namespace netgen
           dshape(3,1) = (1-p(0));
           break;
         }
+      default:
+        throw NgException ("illegal element type in GetDShapeNew");
       }
   }
 
@@ -906,6 +952,11 @@ namespace netgen
 
 
 
+  ostream & operator<<(ostream  & s, const Element0d & el)
+  {
+    s << el.pnum << ", index = " << el.index;
+    return s;
+  }
 
 
   ostream & operator<<(ostream  & s, const Element2d & el)
@@ -925,7 +976,7 @@ namespace netgen
     return s;
   }
 
-
+  /*
   Element :: Element ()
   {
     typ = TET;
@@ -944,13 +995,12 @@ namespace netgen
     flags.deleted = 0;
     flags.fixed = 0;
     orderx = ordery = orderz = 1;
-
+    is_curved = false;
 #ifdef PARALLEL
     partitionNumber = -1;
 #endif
-
   }
-
+  */
 
   Element :: Element (int anp)
   {
@@ -980,6 +1030,7 @@ namespace netgen
       default: cerr << "Element::Element: unknown element with " << np << " points" << endl;
       }
     orderx = ordery = orderz = 1;
+    is_curved = typ != TET; // false;
   }
 
   void Element :: SetOrder (const int aorder) 
@@ -1017,12 +1068,16 @@ namespace netgen
     flags.deleted = 0;
     flags.fixed = 0;
     orderx = ordery = orderz = 1;
+    is_curved =  typ != TET; // false;
+#ifdef PARALLEL
+    partitionNumber = -1;
+#endif
   }
 
 
 
 
-
+  /*
   Element & Element :: operator= (const Element & el2)
   {
     typ = el2.typ;
@@ -1036,9 +1091,10 @@ namespace netgen
     orderz = el2.orderz;
     hp_elnr = el2.hp_elnr;
     flags = el2.flags;
+    is_curved = el2.is_curved;
     return *this;
   }
-
+  */
 
 
   void Element :: SetNP (int anp)
@@ -1149,7 +1205,7 @@ namespace netgen
         { 3, 2, 3, 5, 0 },
         { 3, 3, 4, 5, 0 },
         { 3, 4, 1, 5, 0 } };
-
+    
     static const int prismfaces[][5] =
       {
         { 3, 1, 3, 2, 0 },
@@ -1158,6 +1214,17 @@ namespace netgen
         { 4, 2, 3, 6, 5 },
         { 4, 3, 1, 4, 6 }
       };
+    
+    static const int hexfaces[][5] =
+      {
+        { 4, 4, 3, 2, 1 },
+        { 4, 3, 7, 6, 2 },
+        { 4, 7, 8, 5, 6 },
+        { 4, 8, 4, 1, 5 },
+        { 4, 1, 2, 6, 5 },
+        { 4, 3, 4, 8, 7 }
+      };
+    
 
     switch (np)
       {
@@ -1193,6 +1260,13 @@ namespace netgen
             face.PNum(j) = PNum(prismfaces[i-1][j]);
           break;
         }
+      case 8:
+        {
+          face.SetType(QUAD);
+          for (int j = 1; j <= 4; j++)
+            face.PNum(j) = PNum(hexfaces[i-1][j]);
+          break;
+        }
       }
   }
 
@@ -1645,8 +1719,8 @@ namespace netgen
 
 
 
-  Array< AutoPtr < IntegrationPointData > > ipdtet;
-  Array< AutoPtr < IntegrationPointData > > ipdtet10;
+  Array< shared_ptr < IntegrationPointData > > ipdtet;
+  Array< shared_ptr < IntegrationPointData > > ipdtet10;
 
 
 
@@ -1681,12 +1755,14 @@ namespace netgen
         { 0, 0, 1, 1 },
         { 0, 0, 0, 1 },
       };
-  
+    
     double * pp = NULL;
     switch (typ)
       {
       case TET: pp = &eltetqp[0][0]; break;
       case TET10: pp = &eltet10qp[ip-1][0]; break;
+      default:
+        throw NgException ("illegal element shape in GetIntegrationPoint");
       }
 
     p(0) = pp[0];
@@ -1812,12 +1888,14 @@ namespace netgen
           shape(7) = (1-hp(0))*(  hp(1))*(  hp(2));
           break;
         }
+      default:
+        throw NgException("Element :: GetShape not implemented for that element");
       }
   }
 
 
-
-  void Element :: GetShapeNew (const Point<3> & p, FlatVector & shape) const
+  template <typename T>
+  void Element :: GetShapeNew (const Point<3,T> & p, TFlatVector<T> shape) const
   {
     /*
       if (shape.Size() < GetNP())
@@ -1840,10 +1918,10 @@ namespace netgen
 
       case TET10:
         {
-          double lam1 = p(0);
-          double lam2 = p(1);
-          double lam3 = p(2);
-          double lam4 = 1-p(0)-p(1)-p(2);
+          T lam1 = p(0);
+          T lam2 = p(1);
+          T lam3 = p(2);
+          T lam4 = 1-p(0)-p(1)-p(2);
 	
           shape(0) = 2 * lam1 * (lam1-0.5);
           shape(1) = 2 * lam2 * (lam2-0.5);
@@ -1863,11 +1941,12 @@ namespace netgen
 
       case PYRAMID:
         {
-          double noz = 1-p(2);
-          if (noz == 0.0) noz = 1e-10;
+          T noz = 1-p(2);
+          // if (noz == 0.0) noz = 1e-10;
+          noz += T(1e-12);
 
-          double xi  = p(0) / noz;
-          double eta = p(1) / noz;
+          T xi  = p(0) / noz;
+          T eta = p(1) / noz;
           shape(0) = (1-xi)*(1-eta) * (noz);
           shape(1) = (  xi)*(1-eta) * (noz);
           shape(2) = (  xi)*(  eta) * (noz);
@@ -1898,6 +1977,8 @@ namespace netgen
           shape(7) = (1-p(0))*(  p(1))*(  p(2));
           break;
         }
+      default:
+        throw NgException("Element :: GetNewShape not implemented for that element");
       }
   }
 
@@ -1931,15 +2012,15 @@ namespace netgen
       }
   }
 
-
+  template <typename T>
   void Element :: 
-  GetDShapeNew (const Point<3> & p, MatrixFixWidth<3> & dshape) const
+  GetDShapeNew (const Point<3,T> & p, MatrixFixWidth<3,T> & dshape) const
   {
     switch (typ)
       {
       case TET:
         {
-          dshape = 0;
+          dshape = T(0.0);
           dshape(0,0) = 1;
           dshape(1,1) = 1;
           dshape(2,2) = 1;
@@ -1950,7 +2031,7 @@ namespace netgen
         }
       case PRISM:
         {
-          dshape = 0;
+          dshape = T(0.0);
           dshape(0,0) = 1-p(2);
           dshape(0,2) = -p(0);
           dshape(1,1) = 1-p(2);
@@ -1973,23 +2054,40 @@ namespace netgen
         {
           int np = GetNP();
           double eps = 1e-6;
-          Vector shaper(np), shapel(np);
+          ArrayMem<T,100> mem(2*np);
+          TFlatVector<T> shaper(np, &mem[0]);
+          TFlatVector<T> shapel(np, &mem[np]);
+          // Vector shaper(np), shapel(np);
 	
-          for (int i = 1; i <= 3; i++)
+          for (int i = 0; i < 3; i++)
             {
-              Point3d pr(p), pl(p);
-              pr.X(i) += eps;
-              pl.X(i) -= eps;
+              Point<3,T> pr(p), pl(p);
+              pr(i) += eps;
+              pl(i) -= eps;
 	    
               GetShapeNew (pr, shaper);
               GetShapeNew (pl, shapel);
               for (int j = 0; j < np; j++)
-                dshape(j, i-1) = (shaper(j) - shapel(j)) / (2 * eps);
+                dshape(j, i) = (shaper(j) - shapel(j)) / (2 * eps);
             }
         }
       }
   }
 
+  template void Element2d :: GetShapeNew (const Point<2,double> & p, TFlatVector<double> shape) const;
+  template void Element2d :: GetShapeNew (const Point<2,SIMD<double>> & p, TFlatVector<SIMD<double>> shape) const;
+
+  template void Element2d::GetDShapeNew<double> (const Point<2> &, MatrixFixWidth<2> &) const;
+  template void Element2d::GetDShapeNew<SIMD<double>> (const Point<2,SIMD<double>> &, MatrixFixWidth<2,SIMD<double>> &) const;
+
+
+  template void Element :: GetShapeNew (const Point<3,double> & p, TFlatVector<double> shape) const;
+  template void Element :: GetShapeNew (const Point<3,SIMD<double>> & p, TFlatVector<SIMD<double>> shape) const;
+  
+  template void Element::GetDShapeNew<double> (const Point<3> &, MatrixFixWidth<3> &) const;
+  template void Element::GetDShapeNew<SIMD<double>> (const Point<3,SIMD<double>> &, MatrixFixWidth<3,SIMD<double>> &) const;
+
+
   void Element :: 
   GetPointMatrix (const T_POINTS & points,
                   DenseMatrix & pmat) const
@@ -2235,8 +2333,8 @@ namespace netgen
 
         switch (GetType())
           {
-          case TET: ipdtet.Elem(i).Reset(ipd); break;
-          case TET10: ipdtet10.Elem(i).Reset(ipd); break;
+          case TET: ipdtet.Elem(i).reset(ipd); break;
+          case TET10: ipdtet10.Elem(i).reset(ipd); break;
           default:
             PrintSysError ("Element::ComputeIntegrationPoint(2), illegal type ", int(typ));
           }
@@ -2257,7 +2355,7 @@ namespace netgen
     // Initialise surface colour
     surfcolour = Vec3d(0.0,1.0,0.0);
     tlosurf = -1; 
-    bcname = 0;
+    // bcname = 0;
     firstelement = -1;
   }
 
@@ -2282,7 +2380,7 @@ namespace netgen
     tlosurf = tlosurfi; 
     bcprop = surfnri;
     domin_singular = domout_singular = 0.;
-    bcname = 0;
+    // bcname = 0;
     firstelement = -1;
   }
 
@@ -2297,7 +2395,7 @@ namespace netgen
     tlosurf = seg.tlosurf+1;
     bcprop = 0;
     domin_singular = domout_singular = 0.;
-    bcname = 0;
+    // bcname = 0;
     firstelement = -1;
   }
 
@@ -2310,22 +2408,35 @@ namespace netgen
       tlosurf == seg.tlosurf+1;
   }
 
-
+  string FaceDescriptor :: default_bcname = "default";
+  /*
   const string & FaceDescriptor :: GetBCName () const
   {
     static string defaultstring = "default";
     if (bcname) return *bcname;
     return defaultstring;
   }
-
-  /*
-    void FaceDescriptor :: SetBCName (string * bcn)
-    {
-    bcname = bcn;
-    }
   */
 
+  void FaceDescriptor :: SetBCName (string * bcn)
+  {
+    if (bcn)
+      bcname = bcn;
+    else
+      bcn = &default_bcname;
+  }
+  
+  ngstd::Archive & FaceDescriptor :: DoArchive (ngstd::Archive & ar)
+  {
+    return ar & surfnr & domin & domout & tlosurf & bcprop
+      & surfcolour.X() & surfcolour.Y() & surfcolour.Z()
+      & bcname   
+      & domin_singular & domout_singular ;
+      // don't need:  firstelement
+  }
+  
 
+  
   ostream & operator<<(ostream  & s, const FaceDescriptor & fd)
   {
     s << "surfnr = " << fd.SurfNr() 
@@ -2333,6 +2444,7 @@ namespace netgen
       << ", domout = " << fd.DomainOut()
       << ", tlosurf = " << fd.TLOSurface()
       << ", bcprop = " << fd.BCProperty()
+      << ", bcname = " << fd.GetBCName()
       << ", domin_sing = " << fd.DomainInSingular()
       << ", domout_sing = " << fd.DomainOutSingular()
       << ", colour = " << fd.SurfColour();
@@ -2340,41 +2452,69 @@ namespace netgen
   }
 
 
-
-
-
-
   Identifications :: Identifications (Mesh & amesh)
-    : mesh(amesh)
+    : mesh(amesh), identifiedpoints(100), identifiedpoints_nr(100)
   {
-    identifiedpoints = new INDEX_2_HASHTABLE<int>(100);
-    identifiedpoints_nr = new INDEX_3_HASHTABLE<int>(100);
+    // identifiedpoints = new INDEX_2_HASHTABLE<int>(100);
+    // identifiedpoints_nr = new INDEX_3_HASHTABLE<int>(100);
     maxidentnr = 0;
   }
 
   Identifications :: ~Identifications ()
   {
-    delete identifiedpoints;
-    delete identifiedpoints_nr;
+    ;
+    // delete identifiedpoints;
+    // delete identifiedpoints_nr;
   }
 
   void Identifications :: Delete ()
   {
+    identifiedpoints.DeleteData();
+    identifiedpoints_nr.DeleteData();
+
+    /*
     delete identifiedpoints;
     identifiedpoints = new INDEX_2_HASHTABLE<int>(100);
     delete identifiedpoints_nr;
     identifiedpoints_nr = new INDEX_3_HASHTABLE<int>(100);
+    */
     maxidentnr = 0;
   }
 
+    ngstd::Archive & Identifications :: DoArchive (ngstd::Archive & ar)
+    {
+      ar & maxidentnr;
+      ar & identifiedpoints & identifiedpoints_nr;
+
+      ar & idpoints_table;
+      if (ar.Output())
+        {
+          size_t s = type.Size();
+          ar & s;
+          for (auto & t : type)
+            ar & (unsigned char&)(t);
+        }
+      else
+        {
+          size_t s;
+          ar & s;
+          type.SetSize(s);
+          for (auto & t : type)
+            ar & (unsigned char&)(t);
+        }
+      return ar;
+    }    
+
+  
+
   void Identifications :: Add (PointIndex pi1, PointIndex pi2, int identnr)
   {
     //  (*testout) << "Identification::Add, pi1 = " << pi1 << ", pi2 = " << pi2 << ", identnr = " << identnr << endl;
     INDEX_2 pair (pi1, pi2);
-    identifiedpoints->Set (pair, identnr);
+    identifiedpoints.Set (pair, identnr);
 
     INDEX_3 tripl (pi1, pi2, identnr);
-    identifiedpoints_nr->Set (tripl, 1);
+    identifiedpoints_nr.Set (tripl, 1);
 
     if (identnr > maxidentnr) maxidentnr = identnr;
 
@@ -2388,8 +2528,8 @@ namespace netgen
   int Identifications :: Get (PointIndex pi1, PointIndex pi2) const
   {
     INDEX_2 pair(pi1, pi2);
-    if (identifiedpoints->Used (pair))
-      return identifiedpoints->Get(pair);
+    if (identifiedpoints.Used (pair))
+      return identifiedpoints.Get(pair);
     else
       return 0;
   }
@@ -2397,7 +2537,7 @@ namespace netgen
   bool Identifications :: Get (PointIndex pi1, PointIndex pi2, int nr) const
   {
     INDEX_3 tripl(pi1, pi2, nr);
-    if (identifiedpoints_nr->Used (tripl))
+    if (identifiedpoints_nr.Used (tripl))
       return 1;
     else
       return 0;
@@ -2408,12 +2548,12 @@ namespace netgen
   int Identifications :: GetSymmetric (PointIndex pi1, PointIndex pi2) const
   {
     INDEX_2 pair(pi1, pi2);
-    if (identifiedpoints->Used (pair))
-      return identifiedpoints->Get(pair);
+    if (identifiedpoints.Used (pair))
+      return identifiedpoints.Get(pair);
 
     pair = INDEX_2 (pi2, pi1);
-    if (identifiedpoints->Used (pair))
-      return identifiedpoints->Get(pair);
+    if (identifiedpoints.Used (pair))
+      return identifiedpoints.Get(pair);
 
     return 0;
   }
@@ -2437,12 +2577,12 @@ namespace netgen
       {
         cout << "getmap, identnr = " << identnr << endl;
 
-        for (int i = 1; i <= identifiedpoints_nr->GetNBags(); i++)
-          for (int j = 1; j <= identifiedpoints_nr->GetBagSize(i); j++)
+        for (int i = 1; i <= identifiedpoints_nr.GetNBags(); i++)
+          for (int j = 1; j <= identifiedpoints_nr.GetBagSize(i); j++)
             {
               INDEX_3 i3;
               int dummy;
-              identifiedpoints_nr->GetData (i, j, i3, dummy);
+              identifiedpoints_nr.GetData (i, j, i3, dummy);
 	    
               if (i3.I3() == identnr || !identnr)
                 {
@@ -2462,21 +2602,21 @@ namespace netgen
     identpairs.SetSize(0);
   
     if (identnr == 0)
-      for (int i = 1; i <= identifiedpoints->GetNBags(); i++)
-        for (int j = 1; j <= identifiedpoints->GetBagSize(i); j++)
+      for (int i = 1; i <= identifiedpoints.GetNBags(); i++)
+        for (int j = 1; j <= identifiedpoints.GetBagSize(i); j++)
           {
             INDEX_2 i2;
             int nr;
-            identifiedpoints->GetData (i, j, i2, nr);
+            identifiedpoints.GetData (i, j, i2, nr);
             identpairs.Append (i2);
           }  
     else
-      for (int i = 1; i <= identifiedpoints_nr->GetNBags(); i++)
-        for (int j = 1; j <= identifiedpoints_nr->GetBagSize(i); j++)
+      for (int i = 1; i <= identifiedpoints_nr.GetNBags(); i++)
+        for (int j = 1; j <= identifiedpoints_nr.GetBagSize(i); j++)
           {
             INDEX_3 i3;
             int dummy;
-            identifiedpoints_nr->GetData (i, j, i3 , dummy);
+            identifiedpoints_nr.GetData (i, j, i3 , dummy);
 	  
             if (i3.I3() == identnr)
               identpairs.Append (INDEX_2(i3.I1(), i3.I2()));
@@ -2486,17 +2626,17 @@ namespace netgen
 
   void Identifications :: SetMaxPointNr (int maxpnum)
   {
-    for (int i = 1; i <= identifiedpoints->GetNBags(); i++)
-      for (int j = 1; j <= identifiedpoints->GetBagSize(i); j++)
+    for (int i = 1; i <= identifiedpoints.GetNBags(); i++)
+      for (int j = 1; j <= identifiedpoints.GetBagSize(i); j++)
         {
           INDEX_2 i2;
           int nr;
-          identifiedpoints->GetData (i, j, i2, nr);
+          identifiedpoints.GetData (i, j, i2, nr);
 	
           if (i2.I1() > maxpnum || i2.I2() > maxpnum)
             {
               i2.I1() = i2.I2() = -1;
-              identifiedpoints->SetData (i, j, i2, -1);	    
+              identifiedpoints.SetData (i, j, i2, -1);	    
             }
         }
   }
@@ -2505,49 +2645,49 @@ namespace netgen
   void Identifications :: Print (ostream & ost) const
   {
     ost << "Identifications:" << endl;
-    ost << "pairs: " << endl << *identifiedpoints << endl;
-    ost << "pairs and nr: " << endl << *identifiedpoints_nr << endl;
+    ost << "pairs: " << endl << identifiedpoints << endl;
+    ost << "pairs and nr: " << endl << identifiedpoints_nr << endl;
     ost << "table: " << endl << idpoints_table << endl;
   }
 
 
   MeshingParameters :: MeshingParameters ()
   {
-    optimize3d = "cmdmustm";
+    // optimize3d = "cmdmustm";
     //optimize3d = "cmdmstm";
-    optsteps3d = 3;
-    optimize2d = "smsmsmSmSmSm";
-    optsteps2d = 3;
-    opterrpow = 2;
-    blockfill = 1;
-    filldist = 0.1;
-    safety = 5;
-    relinnersafety = 3;
-    uselocalh = 1;
-    grading = 0.3;
-    delaunay = 1;
-    maxh = 1e10;
-    minh = 0;
-    meshsizefilename = NULL;
-    startinsurface = 0;
-    checkoverlap = 1;
-    checkoverlappingboundary = 1;
-    checkchartboundary = 1;
-    curvaturesafety = 2;
-    segmentsperedge = 1;
-    parthread = 0;
+    // optsteps3d = 3;
+    // optimize2d = "smsmsmSmSmSm";
+    // optsteps2d = 3;
+    // opterrpow = 2;
+    // blockfill = 1;
+    // filldist = 0.1;
+    // safety = 5;
+    // relinnersafety = 3;
+    // uselocalh = 1;
+    // grading = 0.3;
+    // delaunay = 1;
+    // maxh = 1e10;
+    // minh = 0;
+    // meshsizefilename = NULL;
+    // startinsurface = 0;
+    // checkoverlap = 1;
+    // checkoverlappingboundary = 1;
+    // checkchartboundary = 1;
+    // curvaturesafety = 2;
+    // segmentsperedge = 1;
+    // parthread = 0;
 
-    elsizeweight = 0.2;
-    giveuptol2d = 200;
-    giveuptol = 10;
-    maxoutersteps = 10;
-    starshapeclass = 5;
-    baseelnp = 0;
-    sloppy = 1;
+    // elsizeweight = 0.2;
+    // giveuptol2d = 200;
+    // giveuptol = 10;
+    // maxoutersteps = 10;
+    // starshapeclass = 5;
+    // baseelnp = 0;
+    // sloppy = 1;
 
-    badellimit = 175;
-    check_impossible = 0;
-    secondorder = 0;
+    // badellimit = 175;
+    // check_impossible = 0;
+    // secondorder = 0;
   }
 
   void MeshingParameters :: Print (ostream & ost) const
@@ -2565,12 +2705,9 @@ namespace netgen
         << " uselocalh = " <<  uselocalh << endl
         << " grading = " <<  grading << endl
         << " delaunay = " <<  delaunay << endl
-        << " maxh = " <<  maxh << endl;
-    if(meshsizefilename)
-      ost << " meshsizefilename = " <<  meshsizefilename << endl;
-    else
-      ost << " meshsizefilename = NULL" << endl;
-    ost << " startinsurface = " <<  startinsurface << endl
+        << " maxh = " <<  maxh << endl
+        << " meshsizefilename = " <<  meshsizefilename << endl
+        << " startinsurface = " <<  startinsurface << endl
         << " checkoverlap = " <<  checkoverlap << endl
         << " checkchartboundary = " <<  checkchartboundary << endl
         << " curvaturesafety = " <<  curvaturesafety << endl
@@ -2591,6 +2728,7 @@ namespace netgen
         << " inverttrigs = " <<  inverttrigs << endl;
   }
 
+  /*
   void MeshingParameters :: CopyFrom(const MeshingParameters & other)
   {
     //strcpy(optimize3d,other.optimize3d); 
@@ -2610,6 +2748,7 @@ namespace netgen
     maxh = other.maxh;
     //strcpy(const_cast<char*>(meshsizefilename), other.meshsizefilename);
     //const_cast<char*>(meshsizefilename) = other.meshsizefilename; //???
+    meshsizefilename = other.meshsizefilename;
     startinsurface = other.startinsurface;
     checkoverlap = other.checkoverlap;
     checkoverlappingboundary = other.checkoverlappingboundary;
@@ -2631,7 +2770,7 @@ namespace netgen
     inverttets = other.inverttets;
     inverttrigs = other.inverttrigs;
   }
-
+  */
 
   DebugParameters :: DebugParameters ()
   {
