@@ -17,7 +17,7 @@ namespace netgen
 
 
 
-  enum ELEMENT_TYPE { 
+  enum ELEMENT_TYPE : unsigned char { 
     SEGMENT = 1, SEGMENT3 = 2,
     TRIG = 10, QUAD=11, TRIG6 = 12, QUAD6 = 13, QUAD8 = 14,
     TET = 20, TET10 = 21, 
@@ -25,9 +25,25 @@ namespace netgen
     HEX = 25
   };
 
+  /*
   typedef int ELEMENT_EDGE[2];      // initial point, end point
   typedef int ELEMENT_FACE[4];      // points, last one is -1 for trig
+  */
 
+  struct ELEMENT_EDGE
+  {
+    int vals[2];
+    int & operator[] (size_t i) { return vals[i]; }
+    int operator[] (size_t i) const { return vals[i]; }
+  };
+  
+  struct ELEMENT_FACE
+  {
+    int vals[4];
+    int & operator[] (size_t i) { return vals[i]; }
+    int operator[] (size_t i) const { return vals[i]; }
+  };
+  
 
 #define ELEMENT_MAXPOINTS 12
 #define ELEMENT2D_MAXPOINTS 8
@@ -38,18 +54,35 @@ namespace netgen
   enum OPTIMIZEGOAL { OPT_QUALITY, OPT_CONFORM, OPT_REST, OPT_WORSTCASE, OPT_LEGAL };
 
 
+  extern DLL_HEADER size_t timestamp;
+  inline size_t GetTimeStamp() 
+  { 
+    return timestamp; 
+  }
 
+  inline size_t NextTimeStamp()
+  {
+    timestamp++;
+    return timestamp;
+  }
+  
+  /*
   extern DLL_HEADER int GetTimeStamp();
   extern DLL_HEADER int NextTimeStamp();
-
+  */
   class PointGeomInfo
   {
   public:
     int trignum;   // for STL Meshing
     double u, v;   // for OCC Meshing
 
-    PointGeomInfo () 
-      : trignum(-1), u(0), v(0) { ; }
+    PointGeomInfo () = default;
+    // : trignum(-1), u(0), v(0) { ; }
+    PointGeomInfo (const PointGeomInfo&) = default;
+    PointGeomInfo (PointGeomInfo &&) = default;
+    PointGeomInfo & operator= (const PointGeomInfo&) = default;
+    PointGeomInfo & operator= (PointGeomInfo&&) = default;
+    
   };
 
   inline ostream & operator<< (ostream & ost, const PointGeomInfo & gi)
@@ -117,30 +150,37 @@ namespace netgen
   {
     int i;
   public:
-    PointIndex () { ; }
+    PointIndex () = default;
+    PointIndex (const PointIndex&) = default;
+    PointIndex (PointIndex &&) = default;
+    PointIndex & operator= (const PointIndex&) = default;
+    PointIndex & operator= (PointIndex&&) = default;
+     
     PointIndex (int ai) : i(ai) { ; }
-    PointIndex & operator= (const PointIndex &ai) { i = ai.i; return *this; }
-    // PointIndex & operator= (int ai) { i = ai; return *this; }
+    // PointIndex & operator= (const PointIndex &ai) { i = ai.i; return *this; }
     operator int () const { return i; }
-    // int GetInt () const { return i; }
-    // PointIndex operator+ (int i2) { return PointIndex (i+i2); }
-    // PointIndex operator++ (int) { int hi = i; i++; return PointIndex(hi); }
-    // PointIndex operator-- (int) { int hi = i; i--; return PointIndex(hi); }
     PointIndex operator++ (int) { PointIndex hi(*this); i++; return hi; }
     PointIndex operator-- (int) { PointIndex hi(*this); i--; return hi; }
     PointIndex operator++ () { i++; return *this; }
     PointIndex operator-- () { i--; return *this; }
-
+    void Invalidate() { i = PointIndex::BASE-1; }
+    bool IsValid() const { return i != PointIndex::BASE-1; }
 #ifdef BASE0
     enum { BASE = 0 };
 #else
     enum { BASE = 1 };
 #endif  
+
+    ngstd::Archive & DoArchive (ngstd::Archive & ar) { return ar & i; }
   };
 
+  inline ngstd::Archive & operator & (ngstd::Archive & archive, PointIndex & mp)
+  { return mp.DoArchive(archive);   }
+
+  
   inline istream & operator>> (istream & ist, PointIndex & pi)
   {
-    int i; ist >> i; pi = i; return ist;
+    int i; ist >> i; pi = PointIndex(i); return ist;
   }
 
   inline ostream & operator<< (ostream & ost, const PointIndex & pi)
@@ -148,7 +188,18 @@ namespace netgen
     return (ost << int(pi));
   }
 
-
+  template <int N> class PointIndices;
+  template <> class PointIndices<2> : public INDEX_2
+  {
+  public:
+    PointIndices () = default;
+    PointIndices (INDEX_2 i2) : INDEX_2(i2) { ; }
+    PointIndices (PointIndex i1, PointIndex i2) : INDEX_2(i1,i2) { ; } 
+    PointIndex operator[] (int i) const { return PointIndex(INDEX_2::operator[](i)); }
+    PointIndex & operator[] (int i) { return reinterpret_cast<PointIndex&>(INDEX_2::operator[](i)); }
+    static PointIndices Sort(PointIndex i1, PointIndex i2) { return INDEX_2::Sort(i1, i2); } 
+  };
+  
 
 
   class ElementIndex
@@ -160,8 +211,10 @@ namespace netgen
     ElementIndex & operator= (const ElementIndex & ai) { i = ai.i; return *this; }
     ElementIndex & operator= (int ai) { i = ai; return *this; }
     operator int () const { return i; }
-    ElementIndex & operator++ (int) { i++; return *this; }
-    ElementIndex & operator-- (int) { i--; return *this; }
+    ElementIndex operator++ (int) { return ElementIndex(i++); }    
+    ElementIndex operator-- (int) { return ElementIndex(i--); }
+    ElementIndex & operator++ () { ++i; return *this; }
+    ElementIndex & operator-- () { --i; return *this; }
   };
 
   inline istream & operator>> (istream & ist, ElementIndex & pi)
@@ -179,17 +232,29 @@ namespace netgen
   {
     int i;
   public:
-    SurfaceElementIndex () { ; }
+    SurfaceElementIndex () = default;
     SurfaceElementIndex (int ai) : i(ai) { ; }
+    /*
     SurfaceElementIndex & operator= (const SurfaceElementIndex & ai) 
     { i = ai.i; return *this; }
+    */
+    SurfaceElementIndex & operator= (const SurfaceElementIndex & ai) = default;
     SurfaceElementIndex & operator= (int ai) { i = ai; return *this; }
     operator int () const { return i; }
-    SurfaceElementIndex & operator++ (int) { i++; return *this; }
+    SurfaceElementIndex operator++ (int) { SurfaceElementIndex hi(*this); i++; return hi; }
+    SurfaceElementIndex operator-- (int) { SurfaceElementIndex hi(*this); i--; return hi; }
+    SurfaceElementIndex & operator++ () { ++i; return *this; }
+    SurfaceElementIndex & operator-- () { --i; return *this; }
     SurfaceElementIndex & operator+= (int inc) { i+=inc; return *this; }
-    SurfaceElementIndex & operator-- (int) { i--; return *this; }
+
+    ngstd::Archive & DoArchive (ngstd::Archive & ar) { return ar & i; }
   };
 
+  inline ngstd::Archive & operator & (ngstd::Archive & archive, SurfaceElementIndex & mp)
+  { return mp.DoArchive(archive);   }
+
+  
+
   inline istream & operator>> (istream & ist, SurfaceElementIndex & pi)
   {
     int i; ist >> i; pi = i; return ist;
@@ -257,6 +322,8 @@ namespace netgen
       singular = 0; 
     }
 
+    void Scale(double factor) { *testout << "before: " << x[0] << endl; x[0] *= factor; x[1] *= factor; x[2] *= factor; *testout << "after: " << x[0] << endl;}
+
     int GetLayer() const { return layer; }
 
     POINTTYPE Type() const { return type; }
@@ -270,8 +337,19 @@ namespace netgen
     static MPI_Datatype MyGetMPIType ( );
 #endif
 
+    ngstd::Archive & DoArchive (ngstd::Archive & ar)
+    {
+      ar & x[0] & x[1] & x[2] & layer & singular;
+      ar & (unsigned char&)(type);
+      return ar;
+    }
   };
 
+  inline ngstd::Archive & operator & (ngstd::Archive & archive, MeshPoint & mp)
+  { return mp.DoArchive(archive);   }
+  
+
+  
   inline ostream & operator<<(ostream  & s, const MeshPoint & pt)
   { 
     return (s << Point<3> (pt)); 
@@ -295,9 +373,9 @@ namespace netgen
     PointGeomInfo geominfo[ELEMENT2D_MAXPOINTS];
 
     /// surface nr
-    int index:16;
+    short int index;
     ///
-    ELEMENT_TYPE typ:6;
+    ELEMENT_TYPE typ;
     /// number of points
     unsigned int np:4;
     bool badel:1;
@@ -309,7 +387,7 @@ namespace netgen
     // Set a new property for each element, to 
     // control whether it is visible or not
     bool visible:1;  // element visible
-
+    bool is_curved:1;   // element is (high order) curved
     /// order for hp-FEM
     unsigned int orderx:6;
     unsigned int ordery:6;
@@ -323,15 +401,19 @@ namespace netgen
 
   public:
     ///
-    Element2d ();
+    Element2d () = default;
+    Element2d (const Element2d &) = default;
+    Element2d (Element2d &&) = default;
+    Element2d & operator= (const Element2d &) = default;
+    Element2d & operator= (Element2d &&) = default;
     ///
-    Element2d (int anp);
+    DLL_HEADER Element2d (int anp);
     ///
     DLL_HEADER Element2d (ELEMENT_TYPE type);
     ///
-    Element2d (int pi1, int pi2, int pi3);
+    DLL_HEADER Element2d (int pi1, int pi2, int pi3);
     ///
-    Element2d (int pi1, int pi2, int pi3, int pi4);
+    DLL_HEADER Element2d (int pi1, int pi2, int pi3, int pi4);
     ///
     ELEMENT_TYPE GetType () const { return typ; }
     /// 
@@ -346,8 +428,9 @@ namespace netgen
 	case QUAD6: np = 6; break;
 	case QUAD8: np = 8; break;
 	default:
-	  PrintSysError ("Element2d::SetType, illegal type ", typ);
+	  PrintSysError ("Element2d::SetType, illegal type ", int(typ));
 	}
+      is_curved = (np >= 4); 
     }
     ///
     int GetNP() const { return np; }
@@ -360,7 +443,7 @@ namespace netgen
         {
 #ifdef DEBUG
           if (typ != QUAD && typ != QUAD6 && typ != QUAD8)
-            PrintSysError ("element2d::GetNV not implemented for typ", typ)
+            PrintSysError ("element2d::GetNV not implemented for typ", typ);
 #endif
           return 4;
         }
@@ -390,6 +473,10 @@ namespace netgen
 
     FlatArray<const PointIndex> PNums () const 
     { return FlatArray<const PointIndex> (np, &pnum[0]); }
+    FlatArray<PointIndex> PNums ()
+    { return FlatArray<PointIndex> (np, &pnum[0]); }
+    auto Vertices() const
+    { return FlatArray<const PointIndex> (GetNV(), &pnum[0]); }
     
     ///
     PointIndex & PNum (int i) { return pnum[i-1]; }
@@ -410,6 +497,22 @@ namespace netgen
     ///
     const PointGeomInfo & GeomInfoPiMod (int i) const { return geominfo[(i-1) % np]; }
 
+    ngstd::Archive & DoArchive (ngstd::Archive & ar)
+    {
+      short _np, _typ;
+      bool _curved, _vis, _deleted;
+      if (ar.Output())
+        { _np = np; _typ = typ; _curved = is_curved;
+          _vis = visible; _deleted = deleted; }
+      ar & _np & _typ & index & _curved & _vis & _deleted;
+      // ar & next; don't need 
+      if (ar.Input())
+        { np = _np; typ = ELEMENT_TYPE(_typ); is_curved = _curved;
+          visible = _vis; deleted = _deleted; }
+      for (size_t i = 0; i < np; i++)
+        ar & pnum[i];
+      return ar;
+    }
 
     void SetIndex (int si) { index = si; }
     ///
@@ -453,9 +556,13 @@ namespace netgen
 
     void GetShape (const Point2d & p, class Vector & shape) const;
     void GetShapeNew (const Point<2> & p, class FlatVector & shape) const;
+    template <typename T>
+    void GetShapeNew (const Point<2,T> & p, TFlatVector<T> shape) const;    
     /// matrix 2 * np
     void GetDShape (const Point2d & p, class DenseMatrix & dshape) const;
-    void GetDShapeNew (const Point<2> & p, class MatrixFixWidth<2> & dshape) const;
+    template <typename T>
+    void GetDShapeNew (const Point<2,T> & p, class MatrixFixWidth<2,T> & dshape) const;
+    
     /// matrix 2 * np
     void GetPointMatrix (const Array<Point2d> & points,
 			 class DenseMatrix & pmat) const; 
@@ -470,8 +577,13 @@ namespace netgen
 					int pi, Vec2d & dir, double & dd) const;
 
 
-
-    void Delete () { deleted = 1; pnum[0] = pnum[1] = pnum[2] = pnum[3] = PointIndex::BASE-1; }
+    
+    void Delete ()
+    {
+      deleted = 1;
+      for (PointIndex & p : pnum) p.Invalidate(); 
+    }
+    
     bool IsDeleted () const 
     {
 #ifdef DEBUG
@@ -498,6 +610,9 @@ namespace netgen
     bool TestStrongRefinementFlag () const
     { return strongrefflag; }
 
+
+    bool IsCurved () const { return is_curved; }
+    void SetCurved (bool acurved) { is_curved = acurved; }
   
     SurfaceElementIndex NextElement() { return next; }
 
@@ -515,6 +630,8 @@ namespace netgen
 #endif
   };
 
+  inline ngstd::Archive & operator & (ngstd::Archive & archive, Element2d & mp)
+  { return mp.DoArchive(archive);   }
 
   ostream & operator<<(ostream  & s, const Element2d & el);
 
@@ -547,7 +664,7 @@ namespace netgen
     /// point numbers
     PointIndex pnum[ELEMENT_MAXPOINTS];
     ///
-    ELEMENT_TYPE typ:6;
+    ELEMENT_TYPE typ;
     /// number of points (4..tet, 5..pyramid, 6..prism, 8..hex, 10..quad tet, 12..quad prism)
     int np:5;
     ///
@@ -556,7 +673,7 @@ namespace netgen
       bool marked:1;  // marked for refinement
       bool badel:1;   // angles worse then limit
       bool reverse:1; // for refinement a la Bey
-      bool illegal:1; // illegal, will be split or swaped 
+      bool illegal:1; // illegal, will be split or swapped 
       bool illegal_valid:1; // is illegal-flag valid ?
       bool badness_valid:1; // is badness valid ?
       bool refflag:1;     // mark element for refinement
@@ -576,6 +693,7 @@ namespace netgen
        unsigned int levelz:6; */ 
     /// stored shape-badness of element
     float badness;
+    bool is_curved:1;   // element is (high order) curved
   
 #ifdef PARALLEL
     /// number of partition for parallel computation 
@@ -587,13 +705,18 @@ namespace netgen
     flagstruct flags;
 
     ///
-    Element ();
+    DLL_HEADER Element () = default;
+    Element (const Element &) = default;
+    Element (Element &&) = default;
+    Element & operator= (const Element &) = default;
+    Element & operator= (Element &&) = default;
+
     ///
     Element (int anp);
     ///
     Element (ELEMENT_TYPE type);
     ///
-    Element & operator= (const Element & el2);
+    // Element & operator= (const Element & el2);
   
     ///
     void SetNP (int anp);
@@ -602,27 +725,28 @@ namespace netgen
     ///
     int GetNP () const { return np; }
     ///
-    int GetNV() const
+    uint8_t GetNV() const
     {
+      __assume(typ >= TET && typ <= HEX);        
       switch (typ)
 	{
-	case TET: 
-	case TET10: 
-	  return 4;
-	case PRISM12: 
-	case PRISM: 
+        case TET: 
+        case TET10: 
+          return 4;
+        case PRISM12: 
+        case PRISM: 
 	  return 6; 
 	case PYRAMID:
 	  return 5;
 	case HEX:
 	  return 8;
-	default:
+        default: // not a 3D element
 #ifdef DEBUG
-	  PrintSysError ("Element3d::GetNV not implemented for typ ", typ)
+          PrintSysError ("Element3d::GetNV not implemented for typ ", typ);
 #endif
-	    ;
-	}
-      return np;
+          __assume(false);
+          return -1;
+        }
     }
 
     bool operator==(const Element & el2) const;
@@ -649,7 +773,20 @@ namespace netgen
     PointIndex & PNumMod (int i) { return pnum[(i-1) % np]; }
     ///
     const PointIndex & PNumMod (int i) const { return pnum[(i-1) % np]; }
-  
+
+    ngstd::Archive & DoArchive (ngstd::Archive & ar)
+    {
+      short _np, _typ;
+      if (ar.Output())
+        { _np = np; _typ = typ; }
+      ar & _np & _typ & index;
+      if (ar.Input())
+        { np = _np; typ = ELEMENT_TYPE(_typ); }
+      for (size_t i = 0; i < np; i++)
+        ar & pnum[i];
+      return ar;
+    }
+
     ///
     void SetIndex (int si) { index = si; }
     ///
@@ -666,10 +803,10 @@ namespace netgen
 
     ///
     void GetBox (const T_POINTS & points, Box3d & box) const;
-    /// Calculates Volume of elemenet
+    /// Calculates Volume of element
     double Volume (const T_POINTS & points) const;
     ///
-    void Print (ostream & ost) const;
+    DLL_HEADER void Print (ostream & ost) const;
     ///
     int GetNFaces () const
     {
@@ -680,6 +817,7 @@ namespace netgen
 	case PYRAMID: return 5;
 	case PRISM: 
 	case PRISM12: return 5;
+        case HEX: return 6;
 	default:
 #ifdef DEBUG
 	  PrintSysError ("element3d::GetNFaces not implemented for typ", typ)
@@ -717,10 +855,13 @@ namespace netgen
 			    class DenseMatrix & trans) const;
 
     void GetShape (const Point<3> & p, class Vector & shape) const;
-    void GetShapeNew (const Point<3> & p, class FlatVector & shape) const;
+    // void GetShapeNew (const Point<3> & p, class FlatVector & shape) const;
+    template <typename T>
+    void GetShapeNew (const Point<3,T> & p, TFlatVector<T> shape) const;    
     /// matrix 2 * np
     void GetDShape (const Point<3> & p, class DenseMatrix & dshape) const;
-    void GetDShapeNew (const Point<3> & p, class MatrixFixWidth<3> & dshape) const;
+    template <typename T>
+    void GetDShapeNew (const Point<3,T> & p, class MatrixFixWidth<3,T> & dshape) const;
     /// matrix 3 * np
     void GetPointMatrix (const T_POINTS & points,
 			 class DenseMatrix & pmat) const; 
@@ -773,6 +914,8 @@ namespace netgen
       return flags.deleted; 
     }
 
+    bool IsCurved () const { return is_curved; }
+    void SetCurved (bool acurved) { is_curved = acurved; }
 
 
 #ifdef PARALLEL
@@ -785,6 +928,9 @@ namespace netgen
     int hp_elnr;
   };
 
+  inline ngstd::Archive & operator & (ngstd::Archive & archive, Element & mp)
+  { return mp.DoArchive(archive);   }
+  
   ostream & operator<<(ostream  & s, const Element & el);
 
 
@@ -818,7 +964,9 @@ namespace netgen
     unsigned int seginfo:2;
 
     /// surface decoding index
-    int si;          
+    int si;
+    /// co dim 2 deconding index
+    int cd2i;
     /// domain number inner side
     int domin;
     /// domain number outer side
@@ -844,6 +992,7 @@ namespace netgen
 
   private:
     string* bcname;
+    bool is_curved;
 
   public:
     /*
@@ -890,18 +1039,36 @@ namespace netgen
     PointIndex & operator[] (int i) { return pnums[i]; }
     const PointIndex & operator[] (int i) const { return pnums[i]; }
 
+
+    bool IsCurved () const { return is_curved; }
+    void SetCurved (bool acurved) { is_curved = acurved; }
+
 #ifdef PARALLEL
     int GetPartition () const { return partitionNumber; }
     void SetPartition (int nr) { partitionNumber = nr; }; 
 #else
     int GetPartition () const { return 0; }
 #endif
+    ngstd::Archive & DoArchive (ngstd::Archive & ar);
   };
 
+  inline ngstd::Archive & operator & (ngstd::Archive & archive, Segment & mp)
+  { return mp.DoArchive(archive);   }
+
   ostream & operator<<(ostream  & s, const Segment & seg);
 
 
+  class Element0d
+  {
+  public:
+    PointIndex pnum;
+    int index;
+    Element0d () = default;
+    Element0d (PointIndex _pnum, int _index)
+      : pnum(_pnum), index(_index) { ; } 
+  };
 
+  ostream & operator<<(ostream  & s, const Element0d & el);
 
   // class Surface;  
   // class FaceDescriptor;
@@ -924,9 +1091,10 @@ namespace netgen
     // other face data
     /// surface colour (Default: R=0.0 ; G=1.0 ; B=0.0)
     Vec3d surfcolour;
-
+    
     ///
-    string * bcname;
+    static string default_bcname;
+    string * bcname = &default_bcname;
     /// root of linked list 
     SurfaceElementIndex firstelement;
   
@@ -955,14 +1123,14 @@ namespace netgen
     // Philippose - 06/07/2009
     // Get Surface colour
     Vec3d SurfColour () const { return surfcolour; }
-    const string & GetBCName () const; 
+    DLL_HEADER const string & GetBCName () const { return *bcname; }
     // string * BCNamePtr () { return bcname; }
     // const string * BCNamePtr () const  { return bcname; }
     void SetSurfNr (int sn) { surfnr = sn; }
     void SetDomainIn (int di) { domin = di; }
     void SetDomainOut (int dom) { domout = dom; }
     void SetBCProperty (int bc) { bcprop = bc; }
-    void SetBCName (string * bcn) { bcname = bcn; }
+    void SetBCName (string * bcn); //  { bcname = bcn; }
     // Philippose - 06/07/2009
     // Set the surface colour
     void SetSurfColour (Vec3d colour) { surfcolour = colour; }
@@ -973,8 +1141,14 @@ namespace netgen
     SurfaceElementIndex FirstElement() { return firstelement; }
     // friend ostream & operator<<(ostream  & s, const FaceDescriptor & fd);
     friend class Mesh;
+
+    ngstd::Archive & DoArchive (ngstd::Archive & ar);
   };
 
+  inline ngstd::Archive & operator & (ngstd::Archive & archive, FaceDescriptor & mp)
+  { return mp.DoArchive(archive);   }
+  
+
   ostream & operator<< (ostream  & s, const FaceDescriptor & fd);
 
  
@@ -1011,9 +1185,9 @@ namespace netgen
        // h .. Histogramm, no pause
        // H .. Histogramm, pause
        */
-    const char * optimize3d;
+    string optimize3d = "cmdmustm";
     /// number of 3d optimization steps
-    int optsteps3d;
+    int optsteps3d = 3;
     /**
        2d optimization strategy:
        // s .. swap, opt 6 lines/node
@@ -1023,90 +1197,129 @@ namespace netgen
        // P .. plot, pause
        // c .. combine
        **/
-    const char * optimize2d;
+    string optimize2d = "smsmsmSmSmSm";
     /// number of 2d optimization steps
-    int optsteps2d;
+    int optsteps2d = 3;
     /// power of error (to approximate max err optimization)
-    double opterrpow;
+    double opterrpow = 2;
     /// do block filling ?  
-    int blockfill;
+    int blockfill = 1;
     /// block filling up to distance
-    double filldist;
+    double filldist = 0.1;
     /// radius of local environment (times h)
-    double safety;
+    double safety = 5;
     /// radius of active environment (times h)
-    double relinnersafety;
+    double relinnersafety = 3;
     /// use local h ?
-    int uselocalh;
+    int uselocalh = 1;
     /// grading for local h
-    double grading;
+    double grading = 0.3;
     /// use delaunay meshing
-    int delaunay;
+    int delaunay = 1;
     /// maximal mesh size
-    double maxh;
+    double maxh = 1e10;
     /// minimal mesh size
-    double minh;
+    double minh = 0.0;
     /// file for meshsize
-    const char * meshsizefilename;
+    string meshsizefilename = "";
     /// start surfacemeshing from everywhere in surface
-    int startinsurface;
+    int startinsurface = 0;
     /// check overlapping surfaces (debug)
-    int checkoverlap;
+    int checkoverlap = 1;
     /// check overlapping surface mesh before volume meshing
-    int checkoverlappingboundary;
+    int checkoverlappingboundary = 1;
     /// check chart boundary (sometimes too restrictive)
-    int checkchartboundary;
-    /// safty factor for curvatures (elemetns per radius)
-    double curvaturesafety;
+    int checkchartboundary = 1;
+    /// safety factor for curvatures (elements per radius)
+    double curvaturesafety = 2;
     /// minimal number of segments per edge
-    double segmentsperedge;
+    double segmentsperedge = 1;
     /// use parallel threads
-    int parthread;
+    int parthread = 0;
     /// weight of element size w.r.t element shape
-    double elsizeweight;
+    double elsizeweight = 0.2;
     /// init with default values
 
+    /// start at step
+    int perfstepsstart = 0;
+    /// end at step
+    int perfstepsend = 6;
+
 
     /// from mp3:
     /// give up quality class, 2d meshing
-    int giveuptol2d;
+    int giveuptol2d = 200;
     /// give up quality class, 3d meshing
-    int giveuptol;
+    int giveuptol = 10;
     /// maximal outer steps
-    int maxoutersteps;
+    int maxoutersteps = 10;
     /// class starting star-shape filling
-    int starshapeclass;
+    int starshapeclass = 5;
     /// if non-zero, baseelement must have baseelnp points
-    int baseelnp;        
+    int baseelnp = 0;        
     /// quality tolerances are handled less careful
-    int sloppy;
+    int sloppy = 1;
   
     /// limit for max element angle (150-180)
-    double badellimit;
+    double badellimit = 175;
 
-    bool check_impossible;
+    bool check_impossible = 0;
+
+    int only3D_domain_nr = 0;
   
     ///
-    int secondorder;
+    int secondorder = 0;
     /// high order element curvature
-    int elementorder;
+    int elementorder = 1;
     /// quad-dominated surface meshing
-    int quad;
+    int quad = 0;
     ///
-    int inverttets;
+    bool try_hexes = false;
     ///
-    int inverttrigs;
+    int inverttets = 0;
     ///
-    int autozrefine;
+    int inverttrigs = 0;
+    ///
+    int autozrefine = 0;
     ///
     MeshingParameters ();
     ///
+    MeshingParameters (const MeshingParameters & mp2) = default;
+    MeshingParameters (MeshingParameters && mp2) = default;
+    MeshingParameters & operator= (const MeshingParameters & mp2) = default;
+    MeshingParameters & operator= (MeshingParameters && mp2) = default;
+    ///
     void Print (ostream & ost) const;
+    /// 
+    // void CopyFrom(const MeshingParameters & other);
 
-    void CopyFrom(const MeshingParameters & other);
+    class MeshSizePoint
+    {
+    public:
+      Point<3> pnt;
+      double h;
+      MeshSizePoint (Point<3> _pnt, double _h) : pnt(_pnt), h(_h) { ; }
+      MeshSizePoint () = default;
+      MeshSizePoint (const MeshSizePoint &) = default;
+      MeshSizePoint (MeshSizePoint &&) = default;
+      MeshSizePoint & operator= (const MeshSizePoint &) = default;
+      MeshSizePoint & operator= (MeshSizePoint &&) = default;      
+    };
+    Array<MeshSizePoint> meshsize_points;
+    
+    void (*render_function)(bool) = NULL;
+    void Render(bool blocking = false)
+    {
+      if (render_function) 
+        (*render_function)(blocking);
+    }
   };
 
-
+  inline ostream & operator<< (ostream & ost, const MeshingParameters & mp)
+  {
+    mp.Print (ost);
+    return ost;
+  }
 
   class DebugParameters 
   {
@@ -1216,17 +1429,17 @@ namespace netgen
   class Identifications
   {
   public:
-    enum ID_TYPE { UNDEFINED = 1, PERIODIC = 2, CLOSESURFACES = 3, CLOSEEDGES = 4};
+    enum ID_TYPE : unsigned char { UNDEFINED = 1, PERIODIC = 2, CLOSESURFACES = 3, CLOSEEDGES = 4};
   
 
   private:
     class Mesh & mesh;
 
     /// identify points (thin layers, periodic b.c.)  
-    INDEX_2_HASHTABLE<int> * identifiedpoints;
+    INDEX_2_HASHTABLE<int> identifiedpoints;
   
     /// the same, with info about the id-nr
-    INDEX_3_HASHTABLE<int> * identifiedpoints_nr;
+    INDEX_3_HASHTABLE<int> identifiedpoints_nr;
 
     /// sorted by identification nr
     TABLE<INDEX_2> idpoints_table;
@@ -1257,22 +1470,23 @@ namespace netgen
     bool Get (PointIndex pi1, PointIndex pi2, int identnr) const;
     bool GetSymmetric (PointIndex pi1, PointIndex pi2, int identnr) const;
 
+    // bool HasIdentifiedPoints() const { return identifiedpoints != nullptr; } 
     ///
     INDEX_2_HASHTABLE<int> & GetIdentifiedPoints () 
     { 
-      return *identifiedpoints; 
+      return identifiedpoints; 
     }
 
     bool Used (PointIndex pi1, PointIndex pi2)
     {
-      return identifiedpoints->Used (INDEX_2 (pi1, pi2));
+      return identifiedpoints.Used (INDEX_2 (pi1, pi2));
     }
 
     bool UsedSymmetric (PointIndex pi1, PointIndex pi2)
     {
       return 
-	identifiedpoints->Used (INDEX_2 (pi1, pi2)) ||
-	identifiedpoints->Used (INDEX_2 (pi2, pi1));
+	identifiedpoints.Used (INDEX_2 (pi1, pi2)) ||
+	identifiedpoints.Used (INDEX_2 (pi2, pi1));
     }
 
     ///
@@ -1300,10 +1514,14 @@ namespace netgen
     /// remove secondorder
     void SetMaxPointNr (int maxpnum);
 
-    void Print (ostream & ost) const;
+    DLL_HEADER void Print (ostream & ost) const;
+
+    ngstd::Archive & DoArchive (ngstd::Archive & ar);
   };
 
-
+  inline ngstd::Archive & operator & (ngstd::Archive & archive, Identifications & mp)
+  { return mp.DoArchive(archive);   }
+  
 }
 
 
