@@ -527,6 +527,55 @@ namespace netgen
 
     MPI_Waitall (sendrequests.Size(), &sendrequests[0], MPI_STATUS_IGNORE);
 
+    PrintMessage ( 3, "Sending domain+bc - names");
+
+    sendrequests.SetSize(6*(ntasks-1));
+    /** Send bc-names **/
+    int nbcs = bcnames.Size();
+    Array<int> bcname_sizes(nbcs);
+    int tot_bcsize = 0;
+    for(int k=0;k<nbcs;k++) {
+      bcname_sizes[k] = bcnames[k]->size();
+      tot_bcsize += bcname_sizes[k];
+    }
+    char compiled_bcnames[tot_bcsize];
+    tot_bcsize = 0;
+    for(int k=0;k<nbcs;k++)
+      for(int j=0;j<bcname_sizes[k];j++)
+	compiled_bcnames[tot_bcsize++] = (*bcnames[k])[j];
+    
+    for(int k=1;k<ntasks;k++) {
+      sendrequests[6*(k-1)] = MyMPI_ISend(FlatArray<int>(1, &nbcs), k, MPI_TAG_MESH+6);
+      sendrequests[6*(k-1)+1] = MyMPI_ISend(bcname_sizes, k, MPI_TAG_MESH+6);
+      (void) MPI_Isend(compiled_bcnames, tot_bcsize, MPI_CHAR, k, MPI_TAG_MESH+6, MPI_COMM_WORLD, &sendrequests[6*(k-1)+2]);
+    }
+    
+    /** Send mat-names **/
+    int nmats = materials.Size();
+    Array<int> mat_sizes(nmats);
+    int tot_matsize = 0;
+    for(int k=0;k<nmats;k++) {
+      mat_sizes[k] = materials[k]->size();
+      tot_matsize += mat_sizes[k];
+    }
+    char compiled_mats[tot_matsize];
+    tot_matsize = 0;
+    for(int k=0;k<nmats;k++)
+      for(int j=0;j<mat_sizes[k];j++)
+	compiled_mats[tot_matsize++] = (*materials[k])[j];
+    for(int k=1;k<ntasks;k++) {
+      sendrequests[6*(k-1)+3] = MyMPI_ISend(FlatArray<int>(1, &nmats), k, MPI_TAG_MESH+6);
+      sendrequests[6*(k-1)+4] = MyMPI_ISend(mat_sizes, k, MPI_TAG_MESH+6);
+      (void) MPI_Isend(compiled_mats, tot_matsize, MPI_CHAR, k, MPI_TAG_MESH+6, MPI_COMM_WORLD, &sendrequests[6*(k-1)+5]);
+    }
+
+    /* now wait ... **/
+    PrintMessage( 3, "now wait for domain+bc - names");
+
+    MPI_Waitall (sendrequests.Size(), &sendrequests[0], MPI_STATUS_IGNORE);
+    
+    PrintMessage( 3, "send mesh complete");
+
     MPI_Barrier(MPI_COMM_WORLD);
   }
 
@@ -698,6 +747,42 @@ namespace netgen
 	  
 	}
     }
+
+    /** Recv bc-names **/
+    int nbcs;
+    MyMPI_Recv(nbcs, 0, MPI_TAG_MESH+6);
+    Array<int> bcs(nbcs);
+    MyMPI_Recv(bcs, 0, MPI_TAG_MESH+6);
+    int size_bc = 0;
+    for(int k=0;k<nbcs;k++)
+      size_bc += bcs[k];
+    char compiled_bcnames[size_bc];
+    MPI_Recv(compiled_bcnames, size_bc, MPI_CHAR, 0, MPI_TAG_MESH+6, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
+    
+    SetNBCNames(nbcs);
+    int cnt = 0;
+    for(int k=0;k<nbcs;k++) {
+      SetBCName(k, string(&compiled_bcnames[cnt], bcs[k]));
+      cnt += bcs[k];
+    }
+
+    /** Recv mat-names **/
+    int nmats;
+    MyMPI_Recv(nmats, 0, MPI_TAG_MESH+6);
+    Array<int> matsz(nmats);
+    MyMPI_Recv(matsz, 0, MPI_TAG_MESH+6);
+    int size_mats = 0;
+    for(int k=0;k<nmats;k++)
+      size_mats += matsz[k];
+    char compiled_mats[size_mats];
+    MPI_Recv(compiled_mats, size_mats, MPI_CHAR, 0, MPI_TAG_MESH+6, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
+    cnt = 0;
+    materials.SetSize(nmats);
+    for(int k=0;k<nmats;k++) {
+      // setmaterial is 1-based ... 
+      SetMaterial(k+1, string(&compiled_mats[cnt], matsz[k]));
+      cnt += matsz[k];
+    }
     
     MPI_Barrier(MPI_COMM_WORLD);
 
@@ -718,7 +803,7 @@ namespace netgen
 
     NgProfiler::StopTimer (timerloc2);
 
-    topology -> Update();
+    topology.Update();
     clusters -> Update();
 
     // paralleltop -> UpdateCoarseGrid();
@@ -794,10 +879,10 @@ namespace netgen
     eptr.Append (eind.Size());
     Array<idx_t> epart(ne), npart(nn);
 
-    int nparts = ntasks-1;
-    int edgecut;
+    idxtype nparts = ntasks-1;
+    idxtype edgecut;
 
-    int ncommon = 3;
+    idxtype ncommon = 3;
     METIS_PartMeshDual (&ne, &nn, &eptr[0], &eind[0], NULL, NULL, &ncommon, &nparts,
 			NULL, NULL,
 			&edgecut, &epart[0], &npart[0]);
@@ -1029,11 +1114,11 @@ namespace netgen
     eptr.Append (eind.Size());
     Array<idx_t> epart(ne), npart(nn);
 
-    int nparts = ntasks-1;
-    int edgecut;
+    idxtype nparts = ntasks-1;
+    idxtype edgecut;
 
 
-    int ncommon = 3;
+    idxtype ncommon = 3;
     METIS_PartMeshDual (&ne, &nn, &eptr[0], &eind[0], &nwgt[0], NULL, &ncommon, &nparts,
 			NULL, NULL,
 			&edgecut, &epart[0], &npart[0]);
@@ -1218,7 +1303,7 @@ namespace netgen
     int ne = GetNE();
     
     int nn = GetNP();
-    int nedges = topology->GetNEdges();
+    int nedges = topology.GetNEdges();
 
     idxtype  *xadj, * adjacency, *v_weights = NULL, *e_weights = NULL;
 
@@ -1240,7 +1325,7 @@ namespace netgen
     for ( int edge = 1; edge <= nedges; edge++ )
       {
 	int v1, v2;
-	topology->GetEdgeVertices ( edge, v1, v2);
+	topology.GetEdgeVertices ( edge, v1, v2);
 	cnt[v1-1] ++;
 	cnt[v2-1] ++;
       }
@@ -1257,7 +1342,7 @@ namespace netgen
     for ( int edge = 1; edge <= nedges; edge++ )
       {
 	int v1, v2;
-	topology->GetEdgeVertices ( edge, v1, v2);
+	topology.GetEdgeVertices ( edge, v1, v2);
 	adjacency[ xadj[v1-1] + cnt[v1-1] ] = v2-1;
 	adjacency[ xadj[v2-1] + cnt[v2-1] ] = v1-1;
 	cnt[v1-1]++;
@@ -1266,7 +1351,7 @@ namespace netgen
 
     for ( int vert = 0; vert < nn; vert++ )
       {
-	FlatArray<int> array ( cnt[vert], &adjacency[ xadj[vert] ] );
+	FlatArray<idxtype> array ( cnt[vert], &adjacency[ xadj[vert] ] );
 	BubbleSort(array);
       }
 
@@ -1313,7 +1398,7 @@ namespace netgen
     
     // int nn = GetNP();
     // int nedges = topology->GetNEdges();
-    int nfaces = topology->GetNFaces();
+    int nfaces = topology.GetNFaces();
 
     idxtype  *xadj, * adjacency, *v_weights = NULL, *e_weights = NULL;
 
@@ -1340,7 +1425,7 @@ namespace netgen
     for ( int el=1; el <= ne; el++ )
       {
 	Element volel = VolumeElement(el);
-	topology->GetElementFaces(el, elfaces);
+	topology.GetElementFaces(el, elfaces);
 	for ( int i = 0; i < elfaces.Size(); i++ )
 	  {
 	    if ( facevolels1[elfaces[i]-1] == -1 )
@@ -1377,7 +1462,7 @@ namespace netgen
 
     for ( int el = 0; el < ne; el++ )
       {
-	FlatArray<int> array ( cnt[el], &adjacency[ xadj[el] ] );
+	FlatArray<idxtype> array ( cnt[el], &adjacency[ xadj[el] ] );
 	BubbleSort(array);
       }
 
@@ -1428,7 +1513,7 @@ namespace netgen
   void Mesh :: PartDualHybridMesh2D ( ) 
   {
 #ifdef METIS
-    int ne = GetNSE();
+    idxtype ne = GetNSE();
     int nv = GetNV();
 
     Array<idxtype> xadj(ne+1);
@@ -1484,11 +1569,11 @@ namespace netgen
 
     idxtype *v_weights = NULL, *e_weights = NULL;
 
-    int weightflag = 0;
+    idxtype weightflag = 0;
     // int numflag = 0;
-    int nparts = ntasks - 1;
+    idxtype nparts = ntasks - 1;
 
-    int edgecut;
+    idxtype edgecut;
     Array<idxtype> part(ne);
 
     for ( int el = 0; el < ne; el++ )
