@@ -13,35 +13,137 @@ namespace netgen
 
   void Refinement :: Refine (Mesh & mesh)
   {
+    PrintMessage (3, "Refine mesh");
+
+    if (ntasks > 1 && id == 0)
+      return;
+
+
     // reduce 2nd order
     mesh.ComputeNVertices();
     mesh.SetNP(mesh.GetNV());
 
+    if (mesh.mlbetweennodes.Size() < mesh.GetNV())
+      {
+        mesh.mlbetweennodes.SetSize(mesh.GetNV());
+        mesh.mlbetweennodes = INDEX_2(PointIndex::BASE-1,PointIndex::BASE-1);
+      }
 
+    
     INDEX_2_HASHTABLE<PointIndex> between(mesh.GetNP() + 5);
-      
-    int oldne, oldns, oldnf;
 
+
+    // new version with consistent ordering across sub-domains
+
+    Array<INDEX_2> parents;
+    for (SegmentIndex si = 0; si < mesh.GetNSeg(); si++)
+      {
+	const Segment & el = mesh[si];
+	INDEX_2 i2 = INDEX_2::Sort(el[0], el[1]);
+        if (!between.Used(i2))
+          {
+            between.Set (i2, 0);          
+            parents.Append(i2);
+          }
+      }
+    for (SurfaceElementIndex sei = 0; sei < mesh.GetNSE(); sei++)
+      {
+	const Element2d & el = mesh[sei];
+	switch (el.GetType())
+	  {
+	  case TRIG:
+	  case TRIG6:
+	    {
+              static int betw[3][3] =
+		{ { 1, 2, 3 },
+		  { 0, 2, 4 },
+		  { 0, 1, 5 } };
+              for (int j = 0; j < 3; j++)
+                {
+                  auto i2 = PointIndices<2>::Sort(el[betw[j][0]],el[betw[j][1]]);
+                  if (!between.Used(i2))
+                    {
+                      between.Set (i2, 0);          
+                      parents.Append(i2);
+                    }
+                }
+              break;
+            }
+          default:
+            throw NgException ("currently refinement for quad-elements is not supported");
+          }
+      }
+    for (ElementIndex ei = 0; ei < mesh.GetNE(); ei++)
+      {
+	const Element & el = mesh[ei];
+	switch (el.GetType())
+	  {
+	  case TET:
+	  case TET10:
+	    {
+	      static int betw[6][3] =
+                { { 1, 2, 5 },
+                  { 1, 3, 6 },
+                  { 1, 4, 7 },
+                  { 2, 3, 8 },
+                  { 2, 4, 9 },
+                  { 3, 4, 10 } };
+
+              for (int j = 0; j < 6; j++)
+                {
+                  INDEX_2 i2 = INDEX_2::Sort(el.PNum(betw[j][0]),el.PNum(betw[j][1]));
+                  if (!between.Used(i2))
+                    {
+                      between.Set (i2, 0);          
+                      parents.Append(i2);
+                    }
+                }
+              break;
+            }
+          default:
+            throw NgException ("currently refinement for non-tet elements is not supported");
+          }
+      }
+
+    PrintMessage (5, "have points");
+    
+    Array<int> par_nr(parents.Size());
+    for (int i = 0; i < par_nr.Size(); i++)
+      par_nr[i] = i;
+    QuickSort (parents, par_nr);
+    mesh.mlbetweennodes.SetSize(mesh.GetNV()+parents.Size());
+    for (int i = 0; i < parents.Size(); i++)
+      {
+        between.Set (parents[i], mesh.GetNV()+i+PointIndex::BASE);
+        mesh.mlbetweennodes[mesh.GetNV()+i+PointIndex::BASE] = parents[i];
+      }
+    
+    mesh.SetNP(mesh.GetNV() + parents.Size());
+    Array<bool, PointIndex::BASE> pointset(mesh.GetNP());
+    pointset = false;
+    
+    PrintMessage (5, "sorting complete");
+    
     // refine edges
-
     Array<EdgePointGeomInfo,PointIndex::BASE> epgi;
 
-    oldns = mesh.GetNSeg();
+    int oldns = mesh.GetNSeg();
     for (SegmentIndex si = 0; si < oldns; si++)
       {
 	const Segment & el = mesh.LineSegment(si);
 
 	INDEX_2 i2 = INDEX_2::Sort(el[0], el[1]);
-	PointIndex pinew;
+	PointIndex pinew = between.Get(i2);
 	EdgePointGeomInfo ngi;
 
-	if (between.Used(i2))
+	if (pointset[pinew])
 	  {
-	    pinew = between.Get(i2);
+	    // pinew = between.Get(i2);
 	    ngi = epgi[pinew]; 
 	  }
 	else
 	  {
+            pointset[pinew] = true;
 	    Point<3> pnew;
 	    PointBetween (mesh.Point (el[0]),
 			  mesh.Point (el[1]), 0.5,
@@ -49,9 +151,9 @@ namespace netgen
 			  el.epgeominfo[0], el.epgeominfo[1],
 			  pnew, ngi);
 
-	    pinew = mesh.AddPoint (pnew);
-	    between.Set (i2, pinew);
-
+	    // pinew = mesh.AddPoint (pnew);
+            mesh.Point(pinew) = pnew;
+	    // between.Set (i2, pinew);
 
 	    if (pinew >= epgi.Size()+PointIndex::BASE)
 	      epgi.SetSize (pinew+1-PointIndex::BASE);
@@ -69,6 +171,8 @@ namespace netgen
 	mesh.AddSegment (ns2);
       }
 
+    PrintMessage (5, "have 1d elements");
+    
     // refine surface elements
     Array<PointGeomInfo,PointIndex::BASE> surfgi (8*mesh.GetNP());
     for (int i = PointIndex::BASE;
@@ -76,10 +180,9 @@ namespace netgen
       surfgi[i].trignum = -1;
 
 
-    oldnf = mesh.GetNSE();
+    int oldnf = mesh.GetNSE();
     for (SurfaceElementIndex sei = 0; sei < oldnf; sei++)
       {
-	int j, k;
 	const Element2d & el = mesh.SurfaceElement(sei);
 
 	switch (el.GetType())
@@ -95,13 +198,13 @@ namespace netgen
 		  { 1, 3, 5 },
 		  { 1, 2, 6 } };
 
-	      for (j = 1; j <= 3; j++)
+	      for (int j = 1; j <= 3; j++)
 		{
 		  pnums.Elem(j) = el.PNum(j);
 		  pgis.Elem(j) = el.GeomInfoPi(j);
 		}
 
-	      for (j = 0; j < 3; j++)
+	      for (int j = 0; j < 3; j++)
 		{
 		  PointIndex pi1 = pnums.Elem(betw[j][0]);
 		  PointIndex pi2 = pnums.Elem(betw[j][1]);
@@ -120,6 +223,14 @@ namespace netgen
 
 
 		  pgis.Elem(4+j) = pgi;
+                  PointIndex pinew = between.Get(i2); 
+                  pnums.Elem(4+j) = pinew;
+                  if (!pointset[pinew])
+                    {
+                      pointset[pinew] = true;
+                      mesh.Point(pinew) = pb;                      
+                    }
+                  /*
 		  if (between.Used(i2))
 		    pnums.Elem(4+j) = between.Get(i2);
 		  else
@@ -127,7 +238,7 @@ namespace netgen
 		      pnums.Elem(4+j) = mesh.AddPoint (pb);
 		      between.Set (i2, pnums.Get(4+j));
 		    }
-
+                  */
 		  if (surfgi.Size() < pnums.Elem(4+j))
 		    surfgi.SetSize (pnums.Elem(4+j));
 		  surfgi.Elem(pnums.Elem(4+j)) = pgis.Elem(4+j);
@@ -141,10 +252,10 @@ namespace netgen
 		  { 6, 4, 5 } };
 
 	      int ind = el.GetIndex();
-	      for (j = 0; j < 4; j++)
+	      for (int j = 0; j < 4; j++)
 		{
 		  Element2d nel(TRIG);
-		  for (k = 1; k <= 3; k++)
+		  for (int k = 1; k <= 3; k++)
 		    {
 		      nel.PNum(k) = pnums.Get(reftab[j][k-1]);
 		      nel.GeomInfoPi(k) = pgis.Get(reftab[j][k-1]);
@@ -172,13 +283,13 @@ namespace netgen
 		  { 1, 4, 8 },
 		  { 5, 7, 9 } };
 
-	      for (j = 1; j <= 4; j++)
+	      for (int j = 1; j <= 4; j++)
 		{
 		  pnums.Elem(j) = el.PNum(j);
 		  pgis.Elem(j) = el.GeomInfoPi(j);
 		}
 
-	      for (j = 0; j < 5; j++)
+	      for (int j = 0; j < 5; j++)
 		{
 		  int pi1 = pnums.Elem(betw[j][0]);
 		  int pi2 = pnums.Elem(betw[j][1]);
@@ -219,10 +330,10 @@ namespace netgen
 		  { 9, 6, 3, 7 } };
 
 	      int ind = el.GetIndex();
-	      for (j = 0; j < 4; j++)
+	      for (int j = 0; j < 4; j++)
 		{
 		  Element2d nel(QUAD);
-		  for (k = 1; k <= 4; k++)
+		  for (int k = 1; k <= 4; k++)
 		    {
 		      nel.PNum(k) = pnums.Get(reftab[j][k-1]);
 		      nel.GeomInfoPi(k) = pgis.Get(reftab[j][k-1]);
@@ -241,12 +352,13 @@ namespace netgen
 	  }
       }
 
+    PrintMessage (5, "have 2d elements");
+    // cout << "id = " << id << ", ne = " << mesh.GetNE() << endl;
     // refine volume elements
-    oldne = mesh.GetNE();
+    int oldne = mesh.GetNE();
+    mesh.VolumeElements().SetAllocSize(8*oldne);
     for (ElementIndex ei = 0; ei < oldne; ei++)
       {
-	int j, k;
-
 	const Element & el = mesh.VolumeElement(ei);
 	switch (el.GetType())
 	  {
@@ -264,18 +376,19 @@ namespace netgen
 
 	     int elrev = el.flags.reverse;
 
-	     for (j = 1; j <= 4; j++)
-	     pnums.Elem(j) = el.PNum(j);
+	     for (int j = 1; j <= 4; j++)
+               pnums.Elem(j) = el.PNum(j);
 	     if (elrev)
 	     swap (pnums.Elem(3), pnums.Elem(4));
 
-	     for (j = 0; j < 6; j++)
-	     {
-	       INDEX_2 i2;
-	       i2.I1() = pnums.Get(betw[j][0]);
-	       i2.I2() = pnums.Get(betw[j][1]);
-	       i2.Sort();
+	     for (int j = 0; j < 6; j++)
+               {
+                 PointIndex pi1 = pnums.Get(betw[j][0]);
+                 PointIndex pi2 = pnums.Get(betw[j][1]);
+                 INDEX_2 i2 (pi1, pi2);
+                 i2.Sort();
 
+	       /*
 	       if (between.Used(i2))
 	          pnums.Elem(5+j) = between.Get(i2);
 	       else
@@ -285,6 +398,15 @@ namespace netgen
 			   mesh.Point(i2.I2())));
 		  between.Set (i2, pnums.Elem(5+j));
 	       }
+	       */
+	       PointIndex pinew = between.Get(i2);
+	       pnums.Elem(j+5) = pinew;
+	       if (!pointset[pinew])
+		 {
+		   pointset[pinew] = true;
+		   mesh.Point(pinew) = Center(mesh.Point(pi1),
+					      mesh.Point(pi2));
+		 }
 	    }
 
 	    static int reftab[8][4] =
@@ -312,10 +434,10 @@ namespace netgen
 	   };
 
 	   int ind = el.GetIndex();
-	   for (j = 0; j < 8; j++)
+	   for (int j = 0; j < 8; j++)
 	   {
-	      Element nel;
-	      for (k = 1; k <= 4; k++)
+             Element nel(TET);
+	      for (int k = 1; k <= 4; k++)
 	        nel.PNum(k) = pnums.Get(reftab[j][k-1]);
 	      nel.SetIndex(ind);
 	      nel.flags.reverse = reverse[j];
@@ -368,7 +490,7 @@ namespace netgen
 	       };
              */
              
-             // udpated by anonymous supporter, donations please to Karo W.
+             // updated by anonymous supporter, donations please to Karo W.
              static int fbetw[12][3] =
                { { 11, 12, 22 },
                  { 9, 10, 22 },
@@ -386,11 +508,11 @@ namespace netgen
 
 	     pnums = PointIndex(-1);
 
-	     for (j = 1; j <= 8; j++)
+	     for (int j = 1; j <= 8; j++)
                pnums.Elem(j) = el.PNum(j);
 
 
-	     for (j = 0; j < 13; j++)
+	     for (int j = 0; j < 13; j++)
 	     {
 	       INDEX_2 i2;
 	       i2.I1() = pnums.Get(betw[j][0]);
@@ -408,7 +530,7 @@ namespace netgen
 	       }
 	    }
 
-	    for (j = 0; j < 6; j++)
+	    for (int j = 0; j < 6; j++)
 	    {
 	       INDEX_2 i2a, i2b;
 	       i2a.I1() = pnums.Get(fbetw[2*j][0]);
@@ -444,10 +566,10 @@ namespace netgen
 
 
 	   int ind = el.GetIndex();
-	   for (j = 0; j < 8; j++)
+	   for (int j = 0; j < 8; j++)
 	   {
 	      Element nel(HEX);
-	      for (k = 1; k <= 8; k++)
+	      for (int k = 1; k <= 8; k++)
 	        nel.PNum(k) = pnums.Get(reftab[j][k-1]);
 	      nel.SetIndex(ind);
 
@@ -496,12 +618,12 @@ namespace netgen
 	     //int elrev = el.flags.reverse;
            pnums = PointIndex(-1);
            
-           for (j = 1; j <= 6; j++)
+           for (int j = 1; j <= 6; j++)
 	     pnums.Elem(j) = el.PNum(j);
 	    // if (elrev)
 	    // swap (pnums.Elem(3), pnums.Elem(4));
 
-	     for (j = 0; j < 9; j++)
+	     for (int j = 0; j < 9; j++)
 	     {
 	       INDEX_2 i2;
 	       i2.I1() = pnums.Get(betw[j][0]);
@@ -519,7 +641,7 @@ namespace netgen
 	       }
 	    }
 
-	    for (j = 0; j < 3; j++)
+	    for (int j = 0; j < 3; j++)
 	    {
 	       INDEX_2 i2a, i2b;
 	       i2a.I1() = pnums.Get(fbetw[2*j][0]);
@@ -556,10 +678,10 @@ namespace netgen
 
 
 	   int ind = el.GetIndex();
-	   for (j = 0; j < 8; j++)
+	   for (int j = 0; j < 8; j++)
 	   {
 	      Element nel(PRISM);
-	      for (k = 1; k <= 6; k++)
+	      for (int k = 1; k <= 6; k++)
 	        nel.PNum(k) = pnums.Get(reftab[j][k-1]);
 	      nel.SetIndex(ind);
 
@@ -608,8 +730,10 @@ namespace netgen
 
       }
 
+    PrintMessage (5, "have 3d elements");
     mesh.ComputeNVertices();
     mesh.RebuildSurfaceElementLists();
+    PrintMessage (5, "mesh updates complete");
     return;
 
     int cnttrials = 10;
@@ -646,12 +770,9 @@ namespace netgen
 
 	BitArray boundp(np);
 	boundp.Clear();
-	for (int i = 1; i <= mesh.GetNSE(); i++)
-	  {
-	    const Element2d & sel = mesh.SurfaceElement(i);
-	    for (int j = 1; j <= sel.GetNP(); j++)
-	      boundp.Set(sel.PNum(j));
-	  }
+	for (auto & sel : mesh.SurfaceElements())
+          for (auto pi : sel.PNums())
+            boundp.Set(pi);
 
 
 	double lam = 0.5;
@@ -710,11 +831,9 @@ namespace netgen
 		    (*testout) << "p " << i << endl;
 
 		(*testout) << "surf points: " << endl;
-		for (int i = 1; i <= mesh.GetNSE(); i++)
-		  for (int j = 1; j <= 3; j++)
-		    (*testout) << mesh.SurfaceElement(i).PNum(j) << endl;
-		  
-
+		for (auto & sel : mesh.SurfaceElements())
+		  for (auto pi : sel.PNums())
+		    (*testout) << pi << endl;
 
 		mesh.CalcSurfacesOfNode();
 		free.Invert();
