@@ -52,6 +52,8 @@
 #include <gp_Pnt.hxx>
 #include <gp_Pnt2d.hxx>
 
+#define PI 3.14159265358979323846
+
 //=======================================================================
 //function : Partition_Loop2d
 //purpose  :
@@ -154,7 +156,7 @@ static Standard_Real preciseU (const BRepAdaptor_Surface&  theSurf,
   {
     // u for test
     u = u0 + du;
-    du *= 10; // for the next iteration: increase du untill du3d is large enough
+    du *= 10; // for the next iteration: increase du until du3d is large enough
 
     // find out how u is far from u0 in 3D
     gp_Pnt2d aP2d  = theC->Value( u );
@@ -192,7 +194,7 @@ static Standard_Boolean  SelectEdge(const BRepAdaptor_Surface& Surf,
   if (LE.Extent() > 1) {
     //--------------------------------------------------------------
     // Several possible edges.
-    // - Test the edges differents of CE
+    // - Test the edge difference of CE
     //--------------------------------------------------------------
     TopoDS_Face FForward = Surf.Face();
     TopoDS_Edge aPrevNE;
@@ -210,7 +212,7 @@ static Standard_Boolean  SelectEdge(const BRepAdaptor_Surface& Surf,
     Cc->D1(uc, PC, CTg1);
     if (!isForward) CTg1.Reverse();
 
-    Standard_Real anglemin = 3 * PI, tolAng = 1.e-8;
+    Standard_Real anglemin = 3 * M_PI, tolAng = 1.e-8;
 
     // select an edge whose first derivative is most left of CTg1
     // ie an angle between Tg1 and CTg1 is least
@@ -234,7 +236,7 @@ static Standard_Boolean  SelectEdge(const BRepAdaptor_Surface& Surf,
       // -PI < angle < PI
       Standard_Real angle = Tg1.Angle(CTg1);
 
-      if (PI - Abs(angle) <= tolAng)
+      if (M_PI - Abs(angle) <= tolAng)
       {
         // an angle is too close to PI; assure that an angle sign really
         // reflects an edge position: +PI - an edge is worst,
@@ -732,7 +734,7 @@ void Partition_Loop2d::Perform()
           }
         }
       }
-      // removal of doulbe edges can explode a wire into parts,
+      // removal of double edges can explode a wire into parts,
       // make new wires of them.
       // A Loop like previous one but without 2d check
       while ( !WEL.IsEmpty() ) {
@@ -899,7 +901,8 @@ const TopTools_ListOfShape&  Partition_Loop2d::NewFaces() const
 
 //=======================================================================
 //function : findEqual
-//purpose  : move wires form <WL> to <EqWL> pairs of wires build of the same edges
+//purpose  : move wires form <WL> to <EqWL> pairs of wires build of the
+//           same edges
 //=======================================================================
 
 static void findEqual (TopTools_ListOfShape& WL,
