@@ -198,7 +198,7 @@ namespace netgen
       TopTools_IndexedMapOfShape fmap, emap, vmap, somap, shmap, wmap;
       Array<bool> fsingular, esingular, vsingular;
       Box<3> boundingbox;
-
+      Array<string> fnames, enames, snames;
       // Philippose - 29/01/2009
       // OpenCascade XDE Support
       // XCAF Handle to make the face colours available to the rest of
@@ -244,10 +244,10 @@ namespace netgen
       }
 
 
-     virtual void Save (string filename) const;
+      DLL_HEADER virtual void Save (string filename) const;
 
 
-      void BuildFMap();
+      DLL_HEADER void BuildFMap();
 
       Box<3> GetBoundingBox()
       {  return boundingbox;}
@@ -275,8 +275,8 @@ namespace netgen
          return OCCSurface (TopoDS::Face(fmap(surfi)), PLANESPACE);
       }
 
-      void CalcBoundingBox ();
-      void BuildVisualizationMesh (double deflection);
+      DLL_HEADER void CalcBoundingBox ();
+      DLL_HEADER void BuildVisualizationMesh (double deflection);
 
       void RecursiveTopologyTree (const TopoDS_Shape & sh,
          stringstream & str,
@@ -284,17 +284,17 @@ namespace netgen
          bool free,
          const char * lname);
 
-      void GetTopologyTree (stringstream & str);
+      DLL_HEADER void GetTopologyTree (stringstream & str);
 
-      void PrintNrShapes ();
+      DLL_HEADER void PrintNrShapes ();
 
-      void CheckIrregularEntities (stringstream & str);
+      DLL_HEADER void CheckIrregularEntities (stringstream & str);
 
-      void SewFaces();
+      DLL_HEADER void SewFaces();
 
-      void MakeSolid();
+      DLL_HEADER void MakeSolid();
 
-      void HealGeometry();
+      DLL_HEADER void HealGeometry();
 
       // Philippose - 15/01/2009
       // Sets the maximum mesh size for a given face
@@ -381,21 +381,20 @@ namespace netgen
             vvispar[i-1].Lowlight();
       }
 
-      void GetUnmeshedFaceInfo (stringstream & str);
-      void GetNotDrawableFaces (stringstream & str);
-      bool ErrorInSurfaceMeshing ();
+      DLL_HEADER void GetUnmeshedFaceInfo (stringstream & str);
+      DLL_HEADER void GetNotDrawableFaces (stringstream & str);
+      DLL_HEADER bool ErrorInSurfaceMeshing ();
 
-     void WriteOCC_STL(char * filename);
+//      void WriteOCC_STL(char * filename);
 
-     virtual int GenerateMesh (Mesh*& mesh, MeshingParameters & mparam, 
-         int perfstepsstart, int perfstepsend);
+     DLL_HEADER virtual int GenerateMesh (shared_ptr<Mesh> & mesh, MeshingParameters & mparam);
 
-      virtual const Refinement & GetRefinement () const;
+      DLL_HEADER virtual const Refinement & GetRefinement () const;
    };
 
 
 
-   class OCCParameters
+   class DLL_HEADER OCCParameters
    {
    public:
 
@@ -431,25 +430,24 @@ namespace netgen
 
    void PrintContents (OCCGeometry * geom);
 
-   OCCGeometry * LoadOCC_IGES (const char * filename);
-   OCCGeometry * LoadOCC_STEP (const char * filename);
-   OCCGeometry * LoadOCC_BREP (const char * filename);
+   DLL_HEADER OCCGeometry * LoadOCC_IGES (const char * filename);
+   DLL_HEADER OCCGeometry * LoadOCC_STEP (const char * filename);
+   DLL_HEADER OCCGeometry * LoadOCC_BREP (const char * filename);
 
-   extern OCCParameters occparam;
+   DLL_HEADER extern OCCParameters occparam;
 
 
    // Philippose - 31.09.2009
    // External access to the mesh generation functions within the OCC
    // subsystem (Not sure if this is the best way to implement this....!!)
-   extern int OCCGenerateMesh (OCCGeometry & occgeometry, Mesh*& mesh,
-			       MeshingParameters & mparam,
-			       int perfstepsstart, int perfstepsend);
+   DLL_HEADER extern int OCCGenerateMesh (OCCGeometry & occgeometry, shared_ptr<Mesh> & mesh,
+			       MeshingParameters & mparam);
 
-  extern void OCCSetLocalMeshSize(OCCGeometry & geom, Mesh & mesh);
+   DLL_HEADER extern void OCCSetLocalMeshSize(OCCGeometry & geom, Mesh & mesh);
 
-   extern void OCCMeshSurface (OCCGeometry & geom, Mesh & mesh, int perfstepsend);
+   DLL_HEADER extern void OCCMeshSurface (OCCGeometry & geom, Mesh & mesh, int perfstepsend);
 
-   extern void OCCFindEdges (OCCGeometry & geom, Mesh & mesh);
+   DLL_HEADER extern void OCCFindEdges (OCCGeometry & geom, Mesh & mesh);
 }
 
 #endif
