@@ -7,13 +7,17 @@
 #include <occgeom.hpp>
 
 
-#include <incvis.hpp>
+#include <inctcl.hpp>
 #include <visual.hpp>
 
+#include <meshing.hpp>
 #include "../meshing/bcfunctions.hpp"
 
 #include "vsocc.hpp"
 
+// __declspec(dllimport) void AutoColourBcProps(Mesh & mesh, const char *bccolourfile);
+// __declspec(dllimport) void GetFaceColours(Mesh & mesh, Array<Vec3d> & face_colours);
+// __declspec(dllimport) bool ColourMatch(Vec3d col1, Vec3d col2, double eps = 2.5e-05);
 
 extern "C" int Ng_occ_Init (Tcl_Interp * interp);
 
@@ -21,8 +25,8 @@ extern "C" int Ng_occ_Init (Tcl_Interp * interp);
 
 namespace netgen
 {
-  extern AutoPtr<NetgenGeometry> ng_geometry;
-  extern AutoPtr<Mesh> mesh;
+  extern DLL_HEADER shared_ptr<NetgenGeometry> ng_geometry;
+  extern DLL_HEADER shared_ptr<Mesh> mesh;
  
   char * err_needsoccgeometry = (char*) "This operation needs an OCC geometry";
   extern char * err_needsmesh;
@@ -59,7 +63,7 @@ namespace netgen
   {
 #ifdef OCCGEOMETRY
     int showvolume;
-	OCCGeometry * occgeometry = dynamic_cast<OCCGeometry*> (ng_geometry.Ptr());
+    OCCGeometry * occgeometry = dynamic_cast<OCCGeometry*> (ng_geometry.get());
 
     showvolume = atoi (Tcl_GetVar (interp, "::occoptions.showvolumenr", 0));
 
@@ -123,10 +127,10 @@ namespace netgen
 		     int argc, tcl_const char *argv[])
   {
 #ifdef OCCGEOMETRY
-    OCCGeometry * occgeometry = dynamic_cast<OCCGeometry*> (ng_geometry.Ptr());
+    OCCGeometry * occgeometry = dynamic_cast<OCCGeometry*> (ng_geometry.get());
 
-    static char buf[1000];
-    buf[0] = 0;
+    // static char buf[1000];
+    // buf[0] = 0;
     stringstream str;
 
     if (argc >= 2)
@@ -153,7 +157,7 @@ namespace netgen
 		     int argc, tcl_const char *argv[])
   {
 #ifdef OCCGEOMETRY
-    OCCGeometry * occgeometry = dynamic_cast<OCCGeometry*> (ng_geometry.Ptr());
+    OCCGeometry * occgeometry = dynamic_cast<OCCGeometry*> (ng_geometry.get());
 
     stringstream str;
     if (argc >= 2)
@@ -572,7 +576,7 @@ namespace netgen
 	   return TCL_ERROR;
     }
 
-    OCCGeometry * occgeometry = dynamic_cast<OCCGeometry*> (ng_geometry.Ptr());
+    OCCGeometry * occgeometry = dynamic_cast<OCCGeometry*> (ng_geometry.get());
     if (!occgeometry)
     {
       Tcl_SetResult (interp, (char *)"Ng_SurfaceMeshSize currently supports only OCC (STEP/IGES) Files", TCL_STATIC);
@@ -673,7 +677,7 @@ namespace netgen
         return TCL_ERROR;
      }
 
-     if(!mesh.Ptr())
+     if(!mesh)
      {
         Tcl_SetResult (interp, (char *)"Ng_GetCurrentFaceColours: Valid netgen mesh required...please mesh the Geometry first", TCL_STATIC);
 	     return TCL_ERROR;
@@ -846,7 +850,7 @@ namespace netgen
         return TCL_ERROR;
      }
 
-     if(!mesh.Ptr())
+     if(!mesh)
      {
         Tcl_SetResult (interp, (char *)"Ng_AutoColourBcProps: Valid netgen mesh required...please mesh the Geometry first", TCL_STATIC);
 	     return TCL_ERROR;
@@ -945,7 +949,7 @@ namespace netgen
 
   VisualScene * OCCGeometryRegister :: GetVisualScene (const NetgenGeometry * geom) const
   {
-    OCCGeometry * geometry = dynamic_cast<OCCGeometry*> (ng_geometry.Ptr());
+    OCCGeometry * geometry = dynamic_cast<OCCGeometry*> (ng_geometry.get());
     if (geometry)
       {
 	vsoccgeom.SetGeometry (geometry);
