@@ -121,8 +121,8 @@ static void STLFindEdges (STLGeometry & geom,
 	  */
 	  Point3d hp, hp2;
 	  Segment seg;
-	  seg[0] = p1;
-	  seg[1] = p2;
+	  seg[0] = p1 + PointIndex::BASE-1;
+	  seg[1] = p2 + PointIndex::BASE-1;
 	  seg.si = geom.GetTriangle(trig1).GetFaceNum();
 	  seg.edgenr = i;
 
@@ -177,8 +177,8 @@ static void STLFindEdges (STLGeometry & geom,
 
 
 	  Segment seg2;
-	  seg2[0] = p2;
-	  seg2[1] = p1;
+	  seg2[0] = p2 + PointIndex::BASE-1;;
+	  seg2[1] = p1 + PointIndex::BASE-1;;
 	  seg2.si = geom.GetTriangle(trig2).GetFaceNum();
 	  seg2.edgenr = i;
 
@@ -430,7 +430,7 @@ int STLSurfaceMeshing (STLGeometry & geom, class Mesh & mesh)
 	{
 	  PrintMessage(5,"mesh is closed, verifying ...");
 
-	  // no open elements, check wrong elemetns (intersecting..)
+	  // no open elements, check wrong elements (intersecting..)
 
 
 
@@ -730,8 +730,7 @@ void STLSurfaceMeshing1 (STLGeometry & geom,
 	compress[icompress[i]] = 0;
       
       
-      extern void Render();
-      Render();
+      mparam.Render();
     }     
   
   // NgProfiler::Print(stdout);
@@ -756,7 +755,7 @@ void STLSurfaceOptimization (STLGeometry & geom,
   PrintMessage(5,"optimize string = ", meshparam.optimize2d, " elsizew = ", meshparam.elsizeweight);
 
   for (int i = 1; i <= meshparam.optsteps2d; i++)
-    for (size_t j = 1; j <= strlen(meshparam.optimize2d); j++)
+    for (size_t j = 1; j <= meshparam.optimize2d.length(); j++)
       {
 	if (multithread.terminate)
 	  break;
