@@ -18,6 +18,7 @@ void STLMeshing (STLGeometry & geom,
   geom.Clear();
   geom.BuildEdges();
   geom.MakeAtlas(mesh);
+  if (multithread.terminate) { return; }
   geom.CalcFaceNums();
   geom.AddFaceEdges();
   geom.LinkEdges();
@@ -48,8 +49,8 @@ void STLMeshing (STLGeometry & geom,
   meshchart = 0; // initialize all ?? JS
 
   if (geomsearchtreeon)
-    searchtree = new Box3dTree (GetBoundingBox().PMin() - Vec3d(1,1,1),
-				GetBoundingBox().PMax() + Vec3d(1,1,1));
+    searchtree = new BoxTree<3> (GetBoundingBox().PMin() - Vec3d(1,1,1),
+                                 GetBoundingBox().PMax() + Vec3d(1,1,1));
   else
     searchtree = NULL;
 
@@ -91,10 +92,9 @@ void STLGeometry :: Save (string filename) const
 
 
 
-int STLGeometry :: GenerateMesh (Mesh*& mesh, MeshingParameters & mparam,
-				 int perfstepsstart, int perfstepsend)
+int STLGeometry :: GenerateMesh (shared_ptr<Mesh> & mesh, MeshingParameters & mparam)
 {
-  return STLMeshingDummy (this, mesh, mparam, perfstepsstart, perfstepsend);
+  return STLMeshingDummy (this, mesh, mparam);
 }
 
 
@@ -182,7 +182,7 @@ void STLGeometry :: SmoothNormals()
 
   Vec<3> ri;
 
-  double wnb = stldoctor.smoothnormalsweight;   // neigbour normal weight
+  double wnb = stldoctor.smoothnormalsweight;   // neighbour normal weight
   double wgeom = 1-wnb;   // geometry normal weight
 
 
@@ -456,7 +456,8 @@ int STLGeometry :: AddEdge(int ap1, int ap2)
   STLEdge e(ap1,ap2);
   e.SetLeftTrig(GetLeftTrig(ap1,ap2));
   e.SetRightTrig(GetRightTrig(ap1,ap2));
-  return edges.Append(e);
+  edges.Append(e);
+  return edges.Size();
 }
 
 void STLGeometry :: STLDoctorConfirmEdge()
@@ -2115,7 +2116,7 @@ int STLGeometry :: CheckGeometryOverlapping()
   Point<3> pmin = geombox.PMin();
   Point<3> pmax = geombox.PMax();
 
-  Box3dTree setree(pmin, pmax);
+  BoxTree<3> setree(pmin, pmax);
 
   int oltrigs = 0;
   markedtrigs.SetSize(GetNT());
@@ -2138,13 +2139,13 @@ int STLGeometry :: CheckGeometryOverlapping()
     }
 
 
-#pragma omp parallel
   {
-    Array<int> inters;
+    mutex inters_mutex;
     
-#pragma omp for
-    for (int i = 1; i <= GetNT(); i++)
-      {
+    ParallelFor( 1, GetNT()+1, [&] (int first, int next)
+    {
+      Array<int> inters;
+      for (int i=first; i<next; i++) {
 	const STLTriangle & tri = GetTriangle(i);
 	
 	Point<3> tpmin = tri.box.PMin();
@@ -2176,7 +2177,7 @@ int STLGeometry :: CheckGeometryOverlapping()
 
 	    if (IntersectTriangleTriangle (&trip1[0], &trip2[0]))
 	      {
-#pragma omp critical
+                lock_guard<mutex> guard(inters_mutex);
 		{
 		  oltrigs++;
 		  PrintMessage(5,"Intersecting Triangles: trig ",i," with ",inters.Get(j),"!");
@@ -2186,6 +2187,7 @@ int STLGeometry :: CheckGeometryOverlapping()
 	      }
 	  }
       }
+    });
   }
   PrintMessage(3,"Check overlapping geometry ... ", oltrigs, " triangles overlap");
   return oltrigs;
