@@ -930,8 +930,8 @@ void STLGeometry :: RestrictLocalH(class Mesh & mesh, double gh)
       double mindist = 1E50;
       
       PrintMessage(7,"build search tree...");
-      Box3dTree* lsearchtree = new Box3dTree (GetBoundingBox().PMin() - Vec3d(1,1,1),
-					     GetBoundingBox().PMax() + Vec3d(1,1,1));
+      BoxTree<3> * lsearchtree = new BoxTree<3> (GetBoundingBox().PMin() - Vec3d(1,1,1),
+                                                 GetBoundingBox().PMax() + Vec3d(1,1,1));
       
       Array<Point3d> pmins(GetNLines());
       Array<Point3d> pmaxs(GetNLines());
@@ -1137,7 +1137,7 @@ void STLGeometry :: RestrictHChartDistOneChart(int chartnum, Array<int>& acttrig
   Array<Point3d> plimes1;
   Array<Point3d> plimes2;
 	  
-  Array<int> plimes1trigs; //check from wich trig the points come
+  Array<int> plimes1trigs; //check from which trig the points come
   Array<int> plimes2trigs;
 	  
   Array<int> plimes1origin; //either the original pointnumber or zero, if new point
@@ -1329,19 +1329,21 @@ void STLGeometry :: RestrictHChartDistOneChart(int chartnum, Array<int>& acttrig
 }
 
 
-int STLMeshingDummy (STLGeometry* stlgeometry, Mesh*& mesh, MeshingParameters & mparam,
-			    int perfstepsstart, int perfstepsend)
+int STLMeshingDummy (STLGeometry* stlgeometry, shared_ptr<Mesh> & mesh, MeshingParameters & mparam)
 {
-  if (perfstepsstart > perfstepsend) return 0;
+  if (mparam.perfstepsstart > mparam.perfstepsend) return 0;
 
   multithread.terminate = 0;
   int success = 1;
   //int trialcntouter = 0;
 
-  if (perfstepsstart <= MESHCONST_MESHEDGES)
+  if (mparam.perfstepsstart <= MESHCONST_MESHEDGES)
     {
+      if (mesh)
+        mesh -> DeleteMesh();
+      else
+        mesh = make_shared<Mesh>();
 
-      mesh = new Mesh();
       mesh->geomtype = Mesh::GEOM_STL;
 
       mesh -> SetGlobalH (mparam.maxh);
@@ -1365,8 +1367,8 @@ int STLMeshingDummy (STLGeometry* stlgeometry, Mesh*& mesh, MeshingParameters &
   if (multithread.terminate)
     return 0;
 
-  if (perfstepsstart <= MESHCONST_MESHSURFACE && 
-      perfstepsend >= MESHCONST_MESHSURFACE)
+  if (mparam.perfstepsstart <= MESHCONST_MESHSURFACE && 
+      mparam.perfstepsend >= MESHCONST_MESHSURFACE)
     {
 
       if (!stlgeometry->edgesfound) 
@@ -1414,8 +1416,8 @@ int STLMeshingDummy (STLGeometry* stlgeometry, Mesh*& mesh, MeshingParameters &
 
   if (success)
     {
-      if (perfstepsstart <= MESHCONST_OPTSURFACE && 
-	  perfstepsend >= MESHCONST_OPTSURFACE)
+      if (mparam.perfstepsstart <= MESHCONST_OPTSURFACE && 
+	  mparam.perfstepsend >= MESHCONST_OPTSURFACE)
 	{
 	  if (!stlgeometry->edgesfound) 
 	    {
@@ -1459,16 +1461,15 @@ int STLMeshingDummy (STLGeometry* stlgeometry, Mesh*& mesh, MeshingParameters &
 	      (*statout) << GetTime() << " & ";
 #endif
 
-	      extern void Render();
-	      Render();
+	      mparam.Render();
 	    }
 	  stlgeometry->surfaceoptimized = 1;
 	}
       if (multithread.terminate)
 	return 0;
 
-      if (perfstepsstart <= MESHCONST_MESHVOLUME && 
-	  perfstepsend >= MESHCONST_MESHVOLUME)
+      if (mparam.perfstepsstart <= MESHCONST_MESHVOLUME && 
+	  mparam.perfstepsend >= MESHCONST_MESHVOLUME)
 	{
 	  if (stlgeometry->volumemeshed) 
 	    {
@@ -1538,8 +1539,8 @@ int STLMeshingDummy (STLGeometry* stlgeometry, Mesh*& mesh, MeshingParameters &
       if (multithread.terminate)
 	return 0;
 
-      if (perfstepsstart <= MESHCONST_OPTVOLUME && 
-	  perfstepsend >= MESHCONST_OPTVOLUME)
+      if (mparam.perfstepsstart <= MESHCONST_OPTVOLUME && 
+	  mparam.perfstepsend >= MESHCONST_OPTVOLUME)
 	{
 	  if (!stlgeometry->edgesfound) 
 	    {
@@ -1576,8 +1577,7 @@ int STLMeshingDummy (STLGeometry* stlgeometry, Mesh*& mesh, MeshingParameters &
 		     << mesh->GetNP() << " " << '\\' << '\\' << " \\" << "hline" << endl;
 #endif
 
-	  extern void Render();
-	  Render();
+	  mparam.Render();
 	}
     }
   
