@@ -17,19 +17,14 @@ Visualization
 #define PARALLELGL
 #endif
 
-#include "../include/incvis.hpp"
+#include "../include/incopengl.hpp"
 
 #include "vispar.hpp"
 #include "soldata.hpp"
 #include "mvdraw.hpp"
 
 #include <complex>
-
-namespace netgen
-{
 #include "vssolution.hpp"
 #include "meshdoc.hpp"
-}
-
 
 #endif
