@@ -15,7 +15,7 @@
 
 namespace netgen
 {
-  extern AutoPtr<Mesh> mesh;
+  // extern shared_ptr<Mesh> mesh;
   extern NetgenGeometry * ng_geometry;
 
   VisualSceneMesh vsmesh;
@@ -35,21 +35,21 @@ namespace netgen
     pointnumberlist = 0;
     domainsurflist = 0;
 
-    vstimestamp = GetTimeStamp();
-    selecttimestamp = GetTimeStamp();
-    filledtimestamp = GetTimeStamp();
-    linetimestamp = GetTimeStamp();
-    edgetimestamp = GetTimeStamp();
-    pointnumbertimestamp = GetTimeStamp();
+    vstimestamp = -1; // GetTimeStamp();
+    selecttimestamp = -1; // GetTimeStamp();
+    filledtimestamp = -1; // GetTimeStamp();
+    linetimestamp = -1; // GetTimeStamp();
+    edgetimestamp = -1; // GetTimeStamp();
+    pointnumbertimestamp = -1; // GetTimeStamp();
 
-    tettimestamp = GetTimeStamp();
-    prismtimestamp = GetTimeStamp();
-    hextimestamp = GetTimeStamp();
-    pyramidtimestamp = GetTimeStamp();
+    tettimestamp = -1; // GetTimeStamp();
+    prismtimestamp = -1; // GetTimeStamp();
+    hextimestamp = -1; // GetTimeStamp();
+    pyramidtimestamp = -1; // GetTimeStamp();
 
-    badeltimestamp = GetTimeStamp();
-    identifiedtimestamp = GetTimeStamp();
-    domainsurftimestamp = GetTimeStamp();
+    badeltimestamp = -1; // GetTimeStamp();
+    identifiedtimestamp = -1; // GetTimeStamp();
+    domainsurftimestamp = -1; // GetTimeStamp();
 
 
     selface = -1;
@@ -73,6 +73,10 @@ namespace netgen
 
   void VisualSceneMesh :: DrawScene ()
   {
+    try
+      {
+        shared_ptr<Mesh> mesh = GetMesh();
+
     if (!mesh)
       {
 	VisualScene::DrawScene();
@@ -87,6 +91,7 @@ namespace netgen
 
     BuildScene();
 
+    glEnable(GL_DEPTH_TEST);
     glClearColor(backcolor, backcolor, backcolor, 1.0);
     glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 
@@ -301,12 +306,25 @@ namespace netgen
       }
     
     glFinish();
+
+    
+      }
+    catch (bad_weak_ptr e)
+      {
+        // cout << "don't have a mesh to visualize" << endl;
+        VisualScene::DrawScene();      
+      }
+
   }
 
 
   void VisualSceneMesh :: BuildScene (int zoomall)
   {
-    if (!mesh)
+    try
+      {
+        shared_ptr<Mesh> mesh = GetMesh();
+        
+        if (!mesh)
       {
 	VisualScene::BuildScene (zoomall);
 	return;
@@ -422,12 +440,12 @@ namespace netgen
 	char buf[30];
 
 	if (vispar.drawpointnumbers)
-	  for (int i = 1; i <= mesh->GetNP(); i++)
+	  for (PointIndex pi : mesh->Points().Range())
             {
-	      const Point3d & p = mesh->Point(i);
+	      const Point3d & p = mesh->Point(pi);
 	      glRasterPos3d (p.X(), p.Y(), p.Z());
 
-	      sprintf (buf, "%d", i);
+	      sprintf (buf, "%d", int(pi));
 
 	      // glCallLists (strlen (buf), GL_UNSIGNED_BYTE, buf);
 	      MyOpenGLText (buf);
@@ -645,12 +663,12 @@ namespace netgen
 
 
 
-	for (int i = 1; i <= mesh->GetNE(); i++)
+	for (ElementIndex ei : mesh->VolumeElements().Range())
 	  {
-            if (mesh->VolumeElement(i).flags.badel)
+            if (mesh->VolumeElement(ei).flags.badel)
 	      {
 		// copy to be thread-safe
-		Element el = mesh->VolumeElement (i);
+		Element el = mesh->VolumeElement (ei);
 		if ( (el.GetNP() == 4) || (el.GetNP() == 10))
 		  {
 		    glBegin (GL_LINES);
@@ -729,17 +747,16 @@ namespace netgen
 	      }
 	  }
 
-
-	for (int i = 1; i <= mesh->GetNSE(); i++)
+        for (SurfaceElementIndex sei : mesh->SurfaceElements().Range())
 	  {
-            Element2d el = mesh->SurfaceElement(i);
+            Element2d el = mesh->SurfaceElement(sei); // copy to be thread-safe
             if (!el.BadElement())
 	      continue;
 
-            int drawel = 1;
+            bool drawel = true;
             for (int j = 1; j <= el.GetNP(); j++)
-	      if (!el.PNum(j))
-		drawel = 0;
+	      if (!el.PNum(j).IsValid())
+		drawel = false;
 
             if (!drawel)
 	      continue;
@@ -842,12 +859,13 @@ namespace netgen
 
 	//  for (i = 1; i <= mesh->GetNSeg(); i++)
 
-	if (& mesh -> GetIdentifications() )
+	if (mesh -> HasIdentifications() )
 	  {
-            INDEX_2_HASHTABLE<int> & idpts =
-	      mesh->GetIdentifications().GetIdentifiedPoints();
-            if (&idpts)
+            // if (mesh->GetIdentifications().HasIdentifiedPoints())
 	      {
+                INDEX_2_HASHTABLE<int> & idpts =
+                  mesh->GetIdentifications().GetIdentifiedPoints();
+                
 		for (int i = 1; i <= idpts.GetNBags(); i++)
                   for (int j = 1; j <= idpts.GetBagSize(i); j++)
 		    {
@@ -880,6 +898,13 @@ namespace netgen
       }
 
     vstimestamp = meshtimestamp;
+      }
+    catch (bad_weak_ptr e)
+      {
+        PrintMessage (3, "vsmesh::buildscene: don't have a mesh to visualize");
+        VisualScene::BuildScene (zoomall);
+      }
+
   }
 
 
@@ -887,6 +912,8 @@ namespace netgen
 
   void VisualSceneMesh :: BuildFilledList (bool names)
   {
+    shared_ptr<Mesh> mesh = GetMesh();
+    
     static int timer = NgProfiler::CreateTimer ("Mesh::BuildFilledList");
     NgProfiler::RegionTimer reg (timer);
 
@@ -1275,6 +1302,8 @@ namespace netgen
 
   void VisualSceneMesh :: BuildLineList()
   {
+    shared_ptr<Mesh> mesh = GetMesh();
+
     static int timer = NgProfiler::CreateTimer ("Mesh::BuildLineList");
     NgProfiler::RegionTimer reg (timer);
 
@@ -1555,6 +1584,8 @@ namespace netgen
 
   void VisualSceneMesh :: BuildEdgeList()
   {
+    shared_ptr<Mesh> mesh = GetMesh();
+
     if (!lock)
       {
 	lock = new NgLock (mesh->Mutex());
@@ -1730,6 +1761,8 @@ namespace netgen
 
   void VisualSceneMesh :: BuildTetList()
   {
+    shared_ptr<Mesh> mesh = GetMesh();
+
     if (tettimestamp > mesh->GetTimeStamp () &&
 	tettimestamp > vispar.clipping.timestamp )
       return;
@@ -2078,6 +2111,8 @@ namespace netgen
 
   void VisualSceneMesh :: BuildPrismList()
   {
+    shared_ptr<Mesh> mesh = GetMesh();
+    
     if (prismtimestamp > mesh->GetTimeStamp () &&
 	prismtimestamp > vispar.clipping.timestamp )
       return;
@@ -2406,6 +2441,8 @@ namespace netgen
 
   void VisualSceneMesh :: BuildHexList()
   {
+    shared_ptr<Mesh> mesh = GetMesh();
+    
     if (hextimestamp > mesh->GetTimeStamp () &&
 	hextimestamp > vispar.clipping.timestamp )
       return;
@@ -2614,6 +2651,8 @@ namespace netgen
 
   void VisualSceneMesh :: BuildPyramidList()
   {
+    shared_ptr<Mesh> mesh = GetMesh();
+    
     if (pyramidtimestamp > mesh->GetTimeStamp () &&
 	pyramidtimestamp > vispar.clipping.timestamp )
       return;
@@ -2968,6 +3007,8 @@ namespace netgen
 
   void VisualSceneMesh :: BuildDomainSurfList()
   {
+    shared_ptr<Mesh> mesh = GetMesh();
+    
     if (domainsurflist)
       glDeleteLists (domainsurflist, 1);
 
@@ -3084,6 +3125,8 @@ namespace netgen
 
   void VisualSceneMesh :: MouseDblClick (int px, int py)
   {
+    shared_ptr<Mesh> mesh = GetMesh();
+
     BuildFilledList (true);
 
     MouseDblClickSelect(px,py,clipplane,backcolor,transformationmat,center,rad,
@@ -3299,9 +3342,11 @@ namespace netgen
 			    const Point3d & center,
 			    const double rad,
 			    const int displaylist,
-			    int & selelement, int & selface, int & seledge, int & selpoint,
-			    int & selpoint2, int & locpi)
+			    int & selelement, int & selface, int & seledge, PointIndex & selpoint,
+			    PointIndex & selpoint2, int & locpi)
   {
+    auto mesh = vsmesh.GetMesh();
+
     int i, hits;
 
     // select surface triangle by mouse click
@@ -3480,3 +3525,54 @@ namespace netgen
 }
 
 
+
+
+#ifdef NG_PYTHON
+#include <../general/ngpython.hpp>
+
+DLL_HEADER void ExportMeshVis(py::module &m)
+{
+  using namespace netgen;
+  vispar.drawcolorbar = true;
+  vispar.drawnetgenlogo = true;
+  vispar.drawcoordinatecross = true;
+  vispar.drawfilledtrigs = true;
+  vispar.drawdomainsurf = true;
+  vispar.drawhexes = true;
+  vispar.drawtets = true;
+  vispar.drawprisms = true;
+  vispar.drawoutline = true;
+  py::class_<VisualSceneMesh, shared_ptr<VisualSceneMesh>>
+    (m, "VisualSceneMesh")
+    .def("Draw", &VisualSceneMesh::DrawScene)
+    ;
+
+  m.def("VS", FunctionPointer
+          ([](shared_ptr<Mesh> mesh)
+           {
+             auto vs = make_shared<VisualSceneMesh>();
+             // vs->SetMesh(mesh);
+             SetGlobalMesh (mesh);
+             return vs;
+           }));
+
+  m.def("MouseMove", FunctionPointer
+          ([](VisualSceneMesh &vsmesh, int oldx, int oldy, int newx, int 
+              newy, char mode)
+           {
+             vsmesh.MouseMove(oldx, oldy, newx, newy, mode);
+           }));
+  m.def("SelectFace", FunctionPointer
+      ([] (int facenr) {
+       vsmesh.SetSelectedFace(facenr);
+       }));
+  m.def("GetGlobalMesh", FunctionPointer
+      ([] () {
+       return vsmesh.GetMesh();
+       }));
+}
+// BOOST_PYTHON_MODULE(libvisual)
+// {
+//   ExportMeshVis();
+// }
+#endif
