@@ -1,3 +1,12 @@
+if { $tcl_platform(os) eq "Linux" && [tk scaling] > 1.5 } {
+  # On some Linux systems, the scaling setting is only applied after
+  # overwriting some default font settings.
+  # This is a workaround to scale up fonts on high resolution displays.
+  font create ngFont -family Helvetica
+  option add *font ngFont
+  ttk::style configure "." -font ngFont
+}
+
 # netgen menus:
 
 menu .ngmenu -tearoff 0  -relief raised -bd 2
@@ -103,7 +112,7 @@ menu .ngmenu.file
 
 
 .ngmenu.file add cascade -label "Recent Files" -menu .ngmenu.file.recent 
-menu .ngmenu.file.recent
+menu .ngmenu.file.recent -tearoff 0
 
 
 proc AddRecentFile { filename } {
@@ -310,7 +319,7 @@ proc demoredraw { } {
     global videoactive
     if { $videoactive == 1 } {
         puts "addframe"
-        .ndraw Ng_VideoClip addframe
+        Ng_VideoClip .ndraw addframe
     }
     if { $result == 0 && $stopdemo == 0 } {
 	after 1 { demoredraw }
@@ -343,7 +352,7 @@ proc demoredraw { } {
 	set file [tk_getSaveFile -filetypes $types]
 #  -defaultextension ".ppm"]
 	if {$file != ""} {
-	    .ndraw Ng_SnapShot $file }
+	    Ng_SnapShot .ndraw $file }
     }
 
 
@@ -358,14 +367,14 @@ set videoactive 0
  	}
  	set file [tk_getSaveFile -filetypes $types]
  	if {$file != ""} {
- 	    .ndraw Ng_VideoClip init $file 
+ 	    Ng_VideoClip .ndraw init $file 
             global videoactive
             set videoactive 1
         }
      }
 
 .ngmenu.file.video add command -label "add frame..." \
-    -command {.ndraw Ng_VideoClip addframe }
+    -command {Ng_VideoClip .ndraw addframe }
 
 .ngmenu.file.video add command -label "one cycle" \
     -command {
@@ -374,14 +383,14 @@ set videoactive 0
 	    puts "j =  $j"
 	    Ng_Vis_Set time [expr (1000 * $j / 100)]
 	    redraw
-	    .ndraw Ng_VideoClip addframe 
+	    Ng_VideoClip .ndraw addframe 
 	    after 200
 	}
     }
 
 .ngmenu.file.video add command -label "finalize..." \
     -command {
-        .ndraw Ng_VideoClip finalize 
+        Ng_VideoClip .ndraw finalize 
         global videoactive
         set videoactive 0
     }
@@ -397,7 +406,7 @@ set videoactive 0
 .ngmenu.file add separator
 
 
-## herbert tcl load menue
+## herbert tcl load menu
 # .ngmenu.file add command -label "Run tests ..." \
 \#    -command { runtestdialog }
 ##
@@ -412,6 +421,7 @@ set videoactive 0
             # puts "error: $result"
         } 
 
+        after cancel { timer2 }
         Ng_Exit; 
         destroy . 
     }
@@ -787,7 +797,7 @@ menu .ngmenu.help
 # .ngmenu.help add command -label "Ng Help..." \
 \#	-command { help_main }
 # .ngmenu.view add checkbutton -variable showsensitivehelp \
-#	-label "Sensitve Help" \
+#	-label "Sensitive Help" \
 #	-command { sensitivehelpdialog $showsensitivehelp }
 .ngmenu.view add checkbutton -label "Help Line" -variable showhelpline \
 	-command {
@@ -815,11 +825,13 @@ tk_messageBox -message "This is NETGEN \nmainly written by \nJoachim Schoeberl \
 #                                                   #
 #####################################################
 
-frame .bubar -relief raised -bd 2
+ttk::frame .bubar
+# -relief raised
+#-relief raised -bd 2
 pack .bubar -side top -fill x
 
-button .bubar.testb -text "Test" -command { Ng_SaveGeometry }
-button .bubar.surfm -text "Generate Mesh" -command \
+ttk::button .bubar.testb -text "Test" -command { Ng_SaveGeometry }
+ttk::button .bubar.surfm -text "Generate Mesh" -command \
     { 
 	.ngmenu.mesh invoke "Generate Mesh"; 
 #	set selectvisual mesh; 
@@ -827,13 +839,13 @@ button .bubar.surfm -text "Generate Mesh" -command \
 #	Ng_GenerateMesh ${meshoptions.firststep} ${meshoptions.laststep}
 #	redraw 
     }
-button .bubar.stopm -text "Stop" -command \
+ttk::button .bubar.stopm -text "Stop" -command \
     { 
 	# Ng_StopMeshing;  
 	set multithread_terminate 1;
 	set stopdemo 1;
     }
-button .bubar.exitb -text "Quit" \
+ttk::button .bubar.exitb -text "Quit" \
     -command { 
 	   set ans [tk_messageBox -title "Quit Netgen?" -message "Do you really want to quit Netgen?" -type yesno -default "no" -icon question]
 	   if { $ans == "yes" } {
@@ -845,27 +857,38 @@ pack  .bubar.exitb .bubar.surfm .bubar.stopm -side left
 #button .bubar.scan -text "Scan" \
 #    -command { Ng_ParseGeometry; set selectvisual geometry; Ng_SetVisParameters; redraw }
 
-button .bubar.zoomall -text "Zoom All" \
+ttk::button .bubar.zoomall -text "Zoom All" \
     -command { Ng_ZoomAll; redraw }
 
-button .bubar.center -text "Center" \
+ttk::button .bubar.center -text "Center" \
     -command { Ng_Center; redraw }
 
 # tk_optionMenu .bubar.modesel drawmode "rotate" "move  " "zoom  "
-tixOptionMenu .bubar.modesel \
-    -options {
-	label.width  0
-	label.anchor e
-	menubutton.width 6
-    } \
-    -variable drawmode
+# tixOptionMenu .bubar.modesel \
+     # -options {
+ 	# label.width  0
+ 	# label.anchor e
+ 	# menubutton.width 6
+     # } \
+     # -variable drawmode
 
-.bubar.modesel add command rotate -label Rotate
-.bubar.modesel add command move -label Move
-.bubar.modesel add command zoom -label Zoom
+# .bubar.modesel add command rotate -label Rotate
+# .bubar.modesel add command move -label Move
+# .bubar.modesel add command zoom -label Zoom
+
+ttk::menubutton .bubar.modesel -menu .bubar.modesel.menu -text "" -width 6
+
+menu .bubar.modesel.menu  -tearoff 0
+
+.bubar.modesel.menu add command -label "Rotate" -command "set drawmode \"rotate\" ;.bubar.modesel configure -text \"Rotate\""
+.bubar.modesel.menu add command -label "Move" -command "set drawmode \"move\" ;.bubar.modesel configure -text \"Move\""
+.bubar.modesel.menu add command -label "Zoom" -command "set drawmode \"zoom\" ;.bubar.modesel configure -text \"Zoom\""
+
+.bubar.modesel.menu invoke "Rotate"
 
 
 
+      
 set viewvals { geometry specpoints mesh solution}
 if { $userlevel == 3} {
     set viewvals { geometry mesh specpoints surfmeshing modelview solution}
@@ -879,26 +902,27 @@ set viewvallabs(surfmeshing) "Mesh Gen"
 set viewvallabs(modelview)     "Modeller" 
 set viewvallabs(solution)     "Solution" 
 
-tixOptionMenu .bubar.selview \
-    -options {
-	label.width  0
-	label.anchor e
-	menubutton.width 10
-    } \
+# tixOptionMenu .bubar.selview \
+    # -options {
+	# label.width  0
+	# label.anchor e
+	# menubutton.width 10
+    # } \
 
-foreach viewv $viewvals {
-    .bubar.selview add command $viewv -label $viewvallabs($viewv)
-}
+# foreach viewv $viewvals {
+    # .bubar.selview add command $viewv -label $viewvallabs($viewv)
+# }
+
+# .bubar.selview config -variable selectvisual
+# .bubar.selview config -command { Ng_SetVisParameters; redraw }
 
 
 
-.bubar.selview config -variable selectvisual
-.bubar.selview config -command { Ng_SetVisParameters; redraw }
+# pack .bubar.modesel -side right
+# pack forget .bubar.modesel
 
-
-pack .bubar.modesel -side right
-pack forget .bubar.modesel
-pack .bubar.center .bubar.zoomall .bubar.selview -side right
+pack .bubar.center .bubar.zoomall -side right
+# pack .bubar.selview -side right
 
 .ngmenu.view add checkbutton -variable viewrotatebutton \
     -label "Enable LeftButton Selection" \
@@ -910,7 +934,21 @@ pack .bubar.center .bubar.zoomall .bubar.selview -side right
 	}
     }
 
+menu .bubar.selviewmenu
+ttk::menubutton .bubar.selview1 -menu .bubar.selviewmenu -text "Geometry"
+foreach viewv $viewvals {
+    .bubar.selviewmenu add command -label $viewvallabs($viewv) -command \
+        ".bubar.selview1 configure -text \"$viewvallabs($viewv)\" ; set selectvisual $viewv ; Ng_SetVisParameters; redraw"
+}
+pack .bubar.selview1 -side right
+# .bubar.selviewmenu invoke $viewvallabs($selectvisual)
 
+trace add variable selectvisual write selvis_monitor
+proc selvis_monitor { name args } {
+    global selectvisual viewvallabs    
+    .bubar.selviewmenu invoke $viewvallabs($selectvisual)
+} 
+# set selectvisual solution
 
 
 #####################################################
@@ -919,21 +957,22 @@ pack .bubar.center .bubar.zoomall .bubar.selview -side right
 #                                                   #
 #####################################################
 
-label .helpline -text "None"
+ttk::label .helpline -text "None"
 pack forget .helpline -side bottom -fill x
 
-frame .statbar -relief flat -bd 2
+ttk::frame .statbar -relief flat
+# -bd 2
 pack .statbar -side bottom -fill x
 
-label .statbar.ptslabel -text "Points: "
-label .statbar.ptsval -textvariable status_np
-label .statbar.elslabel -text "   Elements: "
-label .statbar.elsval -textvariable status_ne
-label .statbar.selslabel -text "   Surf Elements: "
-label .statbar.selsval -textvariable status_nse
+ttk::label .statbar.ptslabel -text "   Points: "
+ttk::label .statbar.ptsval -textvariable status_np
+ttk::label .statbar.elslabel -text "   Elements: "
+ttk::label .statbar.elsval -textvariable status_ne
+ttk::label .statbar.selslabel -text "   Surf Elements: "
+ttk::label .statbar.selsval -textvariable status_nse
 # label .statbar.memlabel -text "   Mem: "
 # label .statbar.memval -textvariable mem_moveable
-label .statbar.task -textvariable status_task
+ttk::label .statbar.task -textvariable status_task
 
 pack .statbar.ptslabel .statbar.ptsval -side left -ipady 3p 
 pack .statbar.elslabel .statbar.elsval -side left -ipady 3p 
@@ -944,8 +983,9 @@ pack .statbar.selslabel .statbar.selsval -side left -ipady 3p
 # }
 
 
-tixMeter .statbar.per -value 0 -text 0%
-.statbar.per configure -fillcolor blue
+#tixMeter .statbar.per -value 0 -text 0%
+ttk::progressbar .statbar.per -value 0 -maximum 1
+#.statbar.per configure -fillcolor blue
 
 pack .statbar.per -side right
 pack .statbar.task -side right -ipady 4
@@ -971,21 +1011,37 @@ proc timer2 { } {
     Ng_ReadStatus 
 
     if { $multithread_redraw == 1 } {
+        # non-blocking redraw
 	set multithread_redraw 0;
 	redraw;
         
         global videoactive
         if { $videoactive == 1 } {
             puts "addframe"
-            .ndraw Ng_VideoClip addframe
+            Ng_VideoClip .ndraw addframe
         }
     }
+    if { $multithread_redraw == 2 } {
+        # blocking redraw
+	redraw;
+	set multithread_redraw 0;
+        
+        global videoactive
+        if { $videoactive == 1 } {
+            puts "addframe"
+            Ng_VideoClip .ndraw addframe
+        }
+        after 1 { timer2 }
+        return
+    }
+
 
     # global mem_moveable
     # set mem_moveable [Ng_MemInfo moveable]
 
 
-    .statbar.per config -value [expr $status_percent/100] -text [format %2.1f [expr 0.1*int(10*$status_percent)]]%
+    .statbar.per configure -value [expr $status_percent/100]
+#    -text [format %2.1f [expr 0.1*int(10*$status_percent)]]%
 
 
     if { $multithread_running } {
@@ -1066,7 +1122,7 @@ proc timer2 { } {
 	}
 
     }
-    after 30 { timer2 }
+    after 10 { timer2 }
 }
 # after 1000 { timer2 }
 timer2
