@@ -29,14 +29,13 @@
 
 
 #include <meshing.hpp>
-
+#include "../interface/writeuser.hpp"
 
 void (*NGS_ParallelRun) (const string & message) = NULL;
 
 
 namespace netgen
 {
-#include "../interface/writeuser.hpp"
   extern string ngdir;
 
 #ifdef OPENGL
@@ -48,8 +47,10 @@ void Parallel_Exit();
 
 
 namespace netgen {
-  extern AutoPtr<Mesh>  mesh;
+  extern std::shared_ptr<NetgenGeometry> ng_geometry;
+  extern shared_ptr<Mesh>  mesh;
   extern VisualSceneMesh vsmesh;
+  extern VisualSceneSolution & GetVSSolution();
   extern Flags parameters;
   extern DLL_HEADER MeshingParameters mparam;
 }
@@ -61,6 +62,8 @@ using netgen::RegisterUserFormats;
 
 void ParallelRun()
 {   
+  VisualSceneSolution vssolution = GetVSSolution();
+
   string message;
   MPI_Status status;
       
@@ -107,8 +110,11 @@ void ParallelRun()
       else if ( message == "mesh" )
 	{
 	  VT_USER_START ("Mesh::ReceiveParallelMesh");
-	  mesh.Reset( new netgen::Mesh);
+	  mesh.reset( new netgen::Mesh);
 	  mesh->SendRecvMesh();
+// 	  vsmesh.SetMesh (mesh);
+// 	  vssolution.SetMesh (mesh);
+      SetGlobalMesh (mesh);
 	  VT_USER_END ("Mesh::ReceiveParallelMesh");
 	}
 
@@ -134,7 +140,7 @@ void ParallelRun()
 	  string redraw_cmd;
 	  // MyMPI_Recv (redraw_cmd, 0, MPI_TAG_VIS);
 	  redraw_cmd = MyMPI_RecvCmd();
-	    
+	  
 	  // PrintMessage (1, "Redraw - ", redraw_cmd);
                   
 	  static string displname;
@@ -143,17 +149,17 @@ void ParallelRun()
 	  static Display * display = NULL;
 	  static GLXContext context;
 	  static XVisualInfo * visinfo = 0;
-	    
+	  
 	  // if (!display)
 	  if (redraw_cmd == "init")
 	    {
 	      MyMPI_Recv (displname, 0, MPI_TAG_VIS);
 	      MyMPI_Recv (curDrawable, 0, MPI_TAG_VIS);
 	      MyMPI_Recv (contextid, 0, MPI_TAG_VIS);
-		
+	      
 	      display = XOpenDisplay (displname.c_str());
-		
 
+	      
 	      /*
 		PrintMessage (3, "displ - name = ", displname);
 		PrintMessage (3, "display = ", display,
@@ -162,23 +168,27 @@ void ParallelRun()
 		" , h = ", XDisplayHeight (display, 0));
 	      */
 
-	      Window win;
-	      int wx, wy;
-	      unsigned int ww, wh, bw, depth;
-	      // cout << "got drawable: " << curDrawable << endl;
-		
-	      XGetGeometry(display, curDrawable, &win,
-			   &wx, &wy, &ww, &wh,
-			   &bw, &depth);
-		
 	      /*
+		Window win;
+		int wx, wy;
+		unsigned int ww, wh, bw, depth;
+		cout << "got drawable: " << curDrawable << ", contextid: " << contextid <<  endl;
+		
+		cout << "get geometriy..." << endl;
+		XGetGeometry(display, curDrawable, &win,
+		&wx, &wy, &ww, &wh,
+		&bw, &depth);
+		cout << "have!" << endl;
+	      
 		cout << "P" << id << ": window-props:  x = " << wx << ", y = " << wy 
 		<< ", w = " << ww << ", h = " << wh << ", depth = " << depth << endl;
 	      */
-		
+	      
 #define VISUAL
 #ifdef VISUAL
-		
+
+#ifdef VISINFO_OLD
+	      //this does not seem to work anymore (but might still be with togl1.7?)
 	      // make a new GLXContext
 	      // first, generate a visual (copied from togl)
 		
@@ -217,51 +227,68 @@ void ParallelRun()
 		  attrib_list[attrib_count++] = GLX_DOUBLEBUFFER;
 
 		  attrib_list[attrib_count++] = None;
-                    
+
 		  visinfo = glXChooseVisual(display, 0, 
 					    attrib_list);
+		  cout << "have vis?" << endl;
+
 		  if (visinfo) {
 		    /* found a GLX visual! */
 		    // cout << "found VISINFO !!!" << endl;
+		    cout << "found VISINFO !!!" << endl;
 
 		    /*
-		      int hi = 0;
-		      std::cout << "attribs = ";
-		      while (attrib_list[hi] != None)
+		    int hi = 0;
+		    std::cout << "attribs = ";
+		    while (attrib_list[hi] != None)
 		      std::cout << attrib_list[hi++] << " ";
-		      std::cout << std::endl;
+		    std::cout << std::endl;
 		    */
-
+		    
 		    break;
 		  }
 		}
 	      if (!visinfo)
 		cerr << "no VISINFO found" << endl;
 
+#else
+	      //get all possible confs
+	      int nconfs;
+	      auto cptr = glXGetFBConfigs (display,0, &nconfs);
+	      Array<int> conf_ids(nconfs);
+	      for(int k=0;k<nconfs;k++)
+		glXGetFBConfigAttrib(display, cptr[k], GLX_FBCONFIG_ID, &(conf_ids[k]));
+	      
+	      //get drawable->FBConfig->visual
+	      unsigned int d_fbc_id;
+	      glXQueryDrawable( display, curDrawable, GLX_FBCONFIG_ID, &d_fbc_id); 
+	      GLXFBConfig d_fbc;
+	      for(int k=0;k<nconfs;k++)
+		if(d_fbc_id==conf_ids[k])
+		  d_fbc = cptr[k];
+	      visinfo = glXGetVisualFromFBConfig(display,d_fbc);
+#endif
+
 	      // context = glXCreateContext( display, visinfo, 0, /* curContext, */ False );
 	      context = glXCreateContext( display, visinfo, glXImportContextEXT ( display, contextid ), False);
-	      // cout << "context = " << context << endl;
-		
 	      glXMakeCurrent (display, curDrawable, context);
 
 
 #else
 	      // try to get GLXcontext from the master. 
 	      // this needs an indirect context (BUT DOES NOT WORK ????)
-		
-	      context = glXImportContextEXT ( display, contextid );
 
+	      context = glXImportContextEXT ( display, contextid );
 
 	      PrintMessage (1, "GLX-contextid = " , contextid,
 			    " imported context ", context);
 
-
 	      glXMakeCurrent (display, curDrawable, context);
 #endif
 
 	      // PrintMessage (1, "redraw - init complete");
 	    }
-	    
+	  
 	  if (redraw_cmd == "broadcast")
 	    {
 	      vsmesh.Broadcast ();
@@ -279,7 +306,6 @@ void ParallelRun()
 	      vsmesh.BuildFilledList (false);
 	    }
 
-
 	  if (redraw_cmd == "solsurfellist")
 	    {
 	      vssolution.DrawSurfaceElements();
@@ -307,6 +333,8 @@ void ParallelRun()
 
       else if ( message ==  "end" )
 	{
+	  mesh.reset();
+	  ng_geometry.reset();
 	  break;
 	}
       
