@@ -28,7 +28,7 @@
 
 namespace netgen {
    extern void MeshFromSpline2D (SplineGeometry2d & geometry,
-                                 Mesh *& mesh, 
+                                 shared_ptr<Mesh> & mesh, 
                                  MeshingParameters & mp);
 }
 
@@ -45,11 +45,12 @@ namespace netgen
 #endif
 
 
+/*
 namespace netgen
 {
   int id = 0, ntasks = 1;
 }
-
+*/
 
 
 /*
@@ -72,6 +73,9 @@ using namespace netgen;
 
 namespace nglib
 {
+  inline void NOOP_Deleter(void *) { ; }
+
+  
    // initialize, deconstruct Netgen library:
    DLL_HEADER void Ng_Init ()
    {
@@ -501,8 +505,6 @@ namespace nglib
    }
 
 
-
-
    DLL_HEADER Ng_Result Ng_GenerateMesh_2D (Ng_Geometry_2D * geom,
                                             Ng_Mesh ** mesh,
                                             Ng_Meshing_Parameters * mp)
@@ -511,12 +513,13 @@ namespace nglib
       //  MeshingParameters mparam;  
       mp->Transfer_Parameters();
 
-      Mesh * m;
+      shared_ptr<Mesh> m(new Mesh, &NOOP_Deleter);
       MeshFromSpline2D (*(SplineGeometry2d*)geom, m, mparam);
-
+      // new shared_ptr<Mesh> (m);  // hack to keep mesh m alive 
+      
       cout << m->GetNSE() << " elements, " << m->GetNP() << " points" << endl;
 
-      *mesh = (Ng_Mesh*)m;
+      *mesh = (Ng_Mesh*)m.get();
       return NG_OK;
    }
 
@@ -651,14 +654,17 @@ namespace nglib
                        stlgeometry->GetBoundingBox().PMax() + Vec3d(10, 10, 10),
                        0.3);
 
-      me -> LoadLocalMeshSize (mp->meshsize_filename);
+      // cout << "meshsize = " << mp->meshsize_filename << endl;
+      if (mp->meshsize_filename)
+        me -> LoadLocalMeshSize (mp->meshsize_filename);
+
       /*
       if (mp->meshsize_filename)
-      {
-      ifstream infile (mp->meshsize_filename);
-      if (!infile.good()) return NG_FILE_NOT_FOUND;
-      me -> LoadLocalMeshSize (infile);
-      }
+        {
+          ifstream infile (mp->meshsize_filename);
+          if (!infile.good()) return NG_FILE_NOT_FOUND;
+          me -> LoadLocalMeshSize (infile);
+        }
       */
 
       STLMeshing (*stlgeometry, *me);
@@ -1056,8 +1062,10 @@ namespace nglib
       mparam.secondorder = second_order;
       mparam.quad = quad_dominated;
 
-      mparam.meshsizefilename = meshsize_filename;
-
+      if (meshsize_filename)
+        mparam.meshsizefilename = meshsize_filename;
+      else
+        mparam.meshsizefilename = "";
       mparam.optsteps2d = optsteps_2d;
       mparam.optsteps3d = optsteps_3d;
 
@@ -1176,7 +1184,7 @@ namespace netgen
 {
    char geomfilename[255];
 
-   DLL_HEADER void MyError (const char * ch)
+   DLL_HEADER void MyError2 (const char * ch)
    {
       cerr << ch;
    }
@@ -1185,7 +1193,7 @@ namespace netgen
 
 
    //Destination for messages, errors, ...
-   DLL_HEADER void Ng_PrintDest(const char * s)
+   DLL_HEADER void Ng_PrintDest2(const char * s)
    {
 #ifdef PARALLEL
      int id = 0;
@@ -1196,21 +1204,20 @@ namespace netgen
    }
 
 
-
-
+  /*
    DLL_HEADER double GetTime ()
    {
       return 0;
    }
+  */
 
 
-
-
+#ifndef WIN32
    void ResetTime ()
    {
       ;
    }
-
+#endif
 
 
 
@@ -1221,22 +1228,28 @@ namespace netgen
 
 
 
+  //void Render() { ; }
 
-   void Render()
-   {
-      ; 
-   }
 } // End of namespace netgen
 
 
+/*
 
-
+#ifndef WIN32
 void Ng_Redraw () { ; }
-void Ng_ClearSolutionData () { ; }
+void Ng_ClearSolutionData() { ; }
+#endif
 void Ng_SetSolutionData (Ng_SolutionData * soldata) 
 { 
   delete soldata->solclass;
 }
 void Ng_InitSolutionData (Ng_SolutionData * soldata) { ; }
+*/
 
+// Force linking libinterface to libnglib
+#include <../interface/writeuser.hpp>
+void MyDummyToForceLinkingLibInterface(Mesh &mesh, NetgenGeometry &geom)
+{
+  netgen::WriteUserFormat("", mesh, /* geom, */ "");
+}
 
